/* global WE, top */

/**
 webEdition CMS
 *
 * $Rev: 14598 $
 * $Author: lukasimhof $
 * $Date: 2017-11-29 14:02:05 +0100 (Wed, 29 Nov 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_view_abstract(uploader) {
	this.uploader = uploader;

	this.isDragAndDrop = true;
	this.elems = {};
	this.extProgress = {
		isExtProgress: true,
		name: '',
		width: 0,
		parentElemId: 'progressbar'
	};
	this.showFilenamesFull = false;
	this.previewSize = 110;
	this.useOriginalAsPreviewIfNotEdited = false;
	this.lastLoupIndex = -1;
	this.loupeVisible = false;
	this.scrollTopOnAddFiles = 0;
	this.widthInputFilename = 240;

	this.CONTROLS_SHOW_FALSE = 0;
	this.CONTROLS_SHOW_TRUE = 1;
	this.CONTROLS_SHOW_AUTO = 2;

	this.controls = {
		showControls: this.CONTROLS_SHOW_TRUE,
		showEditOptions: this.CONTROLS_SHOW_FALSE,
		showImportMeta: this.CONTROLS_SHOW_FALSE
	};

	this.init = function (conf) {
		this.controller = this.uploader.controller; // on init all components are initialized
		this.sender = this.uploader.sender;
		this.imageEdit = this.uploader.imageEdit;
		this.utils = this.uploader.utils;

		this.htmlUploadItem = conf.htmlUploadItem ? conf.htmlUploadItem : '';
		this.isDragAndDrop = typeof conf.isDragAndDrop !== 'undefined' ? conf.isDragAndDrop : this.isDragAndDrop;
		this.controls = conf.controls || this.controls;
		this.showFilenamesFull = !!conf.showFilenamesFull;
		this.isBinaryReplacement = conf.isBinaryReplacement;
	};

	this.onload = function () {
		var doc = this.uploader.doc;

		this.elems.fileSelect = doc.getElementById(this.uploader.fieldName);
		this.elems.fileDrag = doc.getElementById('div_' + this.uploader.fieldName + '_fileDrag');//FIXME: change to div_fileDrag
		this.elems.fileInputWrapper = doc.getElementById('div_' + this.uploader.fieldName + '_fileInputWrapper');//FIXME: change to div_fileInputWrapper

		this.onload_sub();
	};

	this.onload_sub = function () {
		var doc = this.uploader.doc;
		// TODO: move most of these to abstract onload
		this.elems.divEmptyDropzone = doc.getElementById('div_empty_dropzone');
		this.elems.divUploadItem = doc.getElementById('div_upload_files');
		this.elems.divControlls = doc.getElementById('fileupload_controlls');
		this.elems.divEditOptions = doc.getElementById('fileupload_editoptions');
		this.elems.divImportMeta = doc.getElementById('fileupload_importmeta');
		this.elems.divBtnReset = doc.getElementById('div_fileupload_btnReset');
		this.elems.divBtnCancel = doc.getElementById('div_fileupload_btnCancel');
		this.elems.divBtnUpload = doc.getElementById('div_fileupload_btnUpload');

		this.controller.processEvents(this.controller.ON.GUI_INIT);
	};

	this.appendItem = function (item) {
		var container = this.uploader.doc.getElementById('div_upload_files');
		container.innerHTML = item;
		container.previousSibling.style.display = 'none';
		container.style.display = 'block';
	};

	this.prepareItemAndAppend = function (f, index) {
		var doc = this.uploader.doc;
		var entry;
		var parts = f.file.name.split('.');
		var filenameFull = f.file.name.replace(/_/g, '_&shy;').replace(/\./g, '.&shy;');
		var item = this.htmlUploadItem.replace(/##INDEX##/g, index).replace(/##FILE##/g, (f.file.name)).
						replace(/##FILENAME##/g, (parts[0])).
						replace(/##EXTENSION##/g, (parts[1])).
						replace(/##FILESIZE##/g, (this.utils.computeSize(f.size))).
						replace(/##FILENAMEFULL##/g, filenameFull).
						replace(/##FILETEXT##/g, f.file.name);

		this.appendItem(item);

		entry = doc.getElementById('uploadItem_' + index);
		doc.getElementById('div_upload_files').scrollTop = this.scrollTopOnAddFiles;
		this.addTextCutLeft(entry.getElementsByClassName('textShowFilename')[0], f.file.name, (this.isBinaryReplacement ? 290 : this.widthInputFilename));

		if (f.isEditable) {
			entry.getElementsByClassName('elemIcon')[0].style.display = 'none';
			entry.getElementsByClassName('elemImagePreview')[0].style.display = 'block';
			entry.getElementsByClassName('elemImageEdit')[0].style.display = 'block';
		} else {
			var ext = f.file.name.substr(f.file.name.lastIndexOf('.') + 1).toUpperCase();
			entry.getElementsByClassName('elemIcon')[0].innerHTML = WE().util.getTreeIcon(f.type) + ' ' + ext;
		}

		WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
		if (f.isSizeOk) {
			if (f.isTypeOk && this.uploader.fileuploadType === 'importer') {
				this.sender.isCancelled = false;
			}
		} else {
			this.setStateItem(this.controller.STATE.ITEM_SIZE_NOK, index, f);
		}
		if (!f.isTypeOk) {
			this.setStateItem(this.controller.STATE.ITEM_TYPE_NOK, index, f);
		} else {
			if (this.showFilenamesFull) {
				entry.getElementsByClassName('textShowFilename')[0].style.display = 'none';
				entry.getElementsByClassName('elemFilenameFull')[0].style.display = 'block';
			}
		}

		f.index = index;
		f.entry = entry;

		entry.getElementsByClassName('inputEditFilename')[0].addEventListener('change', this.controller.editFilename.bind(this.controller), false);
		entry.getElementsByClassName('inputEditFilename')[0].addEventListener('blur', this.controller.editFilename.bind(this.controller), false);
		if(this.isBinaryReplacement){
			entry.getElementsByClassName('weBtn btnEditFilename')[0].style.display = 'none';
		} else {
			entry.getElementsByClassName('weBtn btnEditFilename')[0].addEventListener('click', this.controller.editFilename.bind(this.controller), false);
			entry.getElementsByClassName('textShowFilename')[0].addEventListener('click', this.controller.editFilename.bind(this.controller), false);
		}
		entry.getElementsByClassName('btnImgEditRefresh_item')[0].addEventListener('click', this.controller.handleButtonEditImageItem.bind(this.controller), false);
		entry.getElementsByClassName('fuOpts_addRotationLeft')[0].addEventListener('click', this.controller.handleButtonRotateItem.bind(this.controller), false);
		entry.getElementsByClassName('fuOpts_addRotationRight')[0].addEventListener('click', this.controller.handleButtonRotateItem.bind(this.controller), false);
		entry.getElementsByClassName('elemScaleNotice')[0].addEventListener('mouseenter', this.controller.handleIconHelpEnter.bind(this.controller), false);
		entry.getElementsByClassName('elemScaleNotice')[0].addEventListener('mouseleave', this.controller.handleIconHelpLeave.bind(this.controller), false);
		entry.getElementsByClassName('elemSizeError')[0].addEventListener('mouseenter', this.controller.handleAlertIconEnter.bind(this.controller), false);
		entry.getElementsByClassName('elemSizeError')[0].addEventListener('mouseleave', this.controller.handleAlertIconLeave.bind(this.controller), false);
		entry.getElementsByClassName('elemUploadError')[0].addEventListener('mouseenter', this.controller.handleAlertIconEnter.bind(this.controller), false);
		entry.getElementsByClassName('elemUploadError')[0].addEventListener('mouseleave', this.controller.handleAlertIconLeave.bind(this.controller), false);

		// init progressbar
		this.uploader.win.progressBar['_' + this.uploader.fieldName + '_' + index] = {'koef': this.uploader.win.progressBar['_' + this.uploader.fieldName + '_##INDEX##'].koef};
	};

	this.repaintItem = function (fileobj) {
		if (!fileobj.entry) {
			fileobj.entry = this.uploader.doc.getElementById('uploadItem_' + fileobj.index);
			if (!fileobj.entry) {
				this.uploader.win.console.log('an error occured: fileobj.entry is undefined');
				return;
			}
		}

		fileobj.entry.getElementsByClassName('elemSize')[0].innerHTML = this.utils.computeSize(fileobj.size);

		if (fileobj.isEditable) {
			// FIXME: obsolete?
			fileobj.entry.getElementsByClassName('elemIcon')[0].style.display = 'none';
			fileobj.entry.getElementsByClassName('elemImagePreview')[0].style.display = 'block';
			fileobj.entry.getElementsByClassName('elemImageEdit')[0].style.display = 'block';
			fileobj.entry.getElementsByClassName('rowDimensionsOriginal')[0].innerHTML = (fileobj.img.origWidth ? fileobj.img.origWidth + ' x ' + fileobj.img.origHeight + ' px' : '--') + ', ' + this.utils.computeSize(fileobj.img.originalSize);

			this.replacePreviewCanvas(fileobj);
			this.setStateItemEdit('', fileobj.index, false);
		} else {
			// FIXME: obsolete?
			fileobj.entry.getElementsByClassName('elemIcon')[0].style.display = 'block';
			fileobj.entry.getElementsByClassName('elemImagePreview')[0].style.display = 'none';
			fileobj.entry.getElementsByClassName('elemContentBottom')[0].style.display = 'none';

			var ext = fileobj.file.name.substr(fileobj.file.name.lastIndexOf('.') + 1).toUpperCase();
			fileobj.entry.getElementsByClassName('elemIcon')[0].innerHTML = WE().util.getTreeIcon(fileobj.type) + ' ' + ext;
		}

		if (fileobj.isEditable) {
			this.setStateItemEdit('', fileobj.index, false);
		}
		if (!fileobj.isSizeOk) {
			this.setStateItem(this.controller.STATE.ITEM_SIZE_NOK, fileobj.index, fileobj);
		} else {
			this.setStateItem(this.controller.STATE.ITEM_SIZE_OK, fileobj.index, fileobj);
		}

		// call repaintItem and the following in new processEvents
		this.setStateGUI((this.sender.getIsUploadableFiles() ? this.controller.STATE.READY : this.controller.STATE.NOTREADY), fileobj);
	};

	this.previewSyncRotation = function (pos, rotation) {
		pos = pos || 0;
		var indices = this.imageEdit.getImageEditIndices(pos, pos === -1, false);

		for (var i = 0; i < indices.length; i++) {
			this.imageEdit.processimageRotatePreview(this.sender.preparedFiles[indices[i]], rotation);
			this.replacePreviewCanvas(this.sender.preparedFiles[indices[i]]);
		}
	};

	this.setPreviewLoupe = function (fileobj, pt) {
		var doc = this.uploader.doc;

		if (fileobj.isUploadStarted) {
			return;
		}
		pt = pt ? pt : 0;
		if (pt === 1) {
			var info = '',
							dimension = fileobj.img.fullPrev.height + 'x' + fileobj.img.fullPrev.width + ' px';
			if (fileobj.isEdited) {
				var deg = parseInt(fileobj.img.processedOptions.rotate),
								degText = deg === 0 ? '' : (deg === 90 ? '90&deg; ' + WE().consts.g_l.fileupload.editRotationRight : (deg === 270 ? '90&deg; ' + WE().consts.g_l.fileupload.editRotationLeft : '180&deg;'));
				info += (fileobj.img.processedOptions.scale ? WE().consts.g_l.fileupload.editScaled + ' ' : '') + dimension;
				info += deg ? (info ? ', ' : '') + WE().consts.g_l.fileupload.editRotation + ': ' + degText : '';
				info += fileobj.img.processedOptions.quality !== this.imageEdit.OPTS_QUALITY_NEUTRAL_VAL && fileobj.type === 'image/jpeg' ? (info ? ', ' : '') + WE().consts.g_l.fileupload.editQuality + ': ' + fileobj.img.processedOptions.quality + '%' : '';
			} else {
				info = WE().consts.g_l.fileupload.editNotEdited + ': ' + dimension;
			}
			doc.getElementById('we_fileUpload_loupeInfo').innerHTML = info;

			fileobj.loupInner.innerHTML = ''; // be sure img is appended as firstChild!
			fileobj.loupInner.appendChild(fileobj.img.fullPrev);
			doc.getElementById('we_fileUpload_loupeInfo').style.display = 'block';
			doc.getElementById('we_fileUpload_spinner').style.display = 'none';
			doc.getElementsByClassName('editorCrosshairH')[0].style.display = 'block';
			doc.getElementsByClassName('editorCrosshairV')[0].style.display = 'block';
			fileobj.focusPoint = doc.getElementById('we_fileUpload_focusPoint');
			fileobj.focusPointFixed = doc.getElementById('editorFocuspointFixed');
			fileobj.focusPoint.style.display = 'block';
			return;
		}

		if (this.lastLoupIndex !== -1 && this.lastLoupIndex !== fileobj.index && this.sender.preparedFiles[this.lastLoupIndex]) {
			// in importer we delete fullPreview of an fileobj when moving to an other file
			this.sender.preparedFiles[this.lastLoupIndex].img.fullPrev = null;
		}
		this.lastLoupIndex = fileobj.index;

		if (!(fileobj.img.fullPrev || fileobj.dataUrl || fileobj.dataArray)) {
			doc.getElementById('we_fileUpload_loupeFallback').innerHTML = 'Für dieses Bild bzw. für die aktuellen Bearbeitungsoptionen<br/>wurde noch keine Vorschau erstellt.';
			doc.getElementById('we_fileUpload_loupeFallback').style.display = 'block';
			this.loupeVisible = false;
			return;
		}
		this.loupeVisible = true;

		fileobj.loupInner = doc.getElementById('we_fileUpload_loupeInner');
		fileobj.loupInner.style.display = 'block';
		doc.getElementById('we_fileUpload_loupe').style.display = 'block';
		doc.getElementById('we_fileUpload_spinner').style.display = 'block';

		if (fileobj.img.fullPrev) {
			this.setPreviewLoupe(fileobj, 1); // we always keep 1 rendered image in loupe
		} else if (fileobj.dataUrl || fileobj.dataArray) { // we have dataURL or dataArray
			fileobj.img.fullPrev = new Image();
			fileobj.img.fullPrev.onload = this.handleLoupImageOnLoad.bind(this, fileobj);

			this.utils.logTimeFromStart('start load fullpreview', true);
			window.setTimeout(this.loadLoupFromdataUrl, 10, fileobj);
		} else {
			/*
			 * we sould not load data here: it lasts too long!
			 * as long as memory limit is not reached we save dataURLs to fileobjects
			 * => when limit is reached one must load dataURL using btnMakePreview!
			 *
			 */
			/*
			 var reader = new FileReader();
			 reader.onload = function() {
			 fileobj.img.fullPrev = new Image();
			 fileobj.img.fullPrev.onload = function(){
			 instance.view.setPreviewLoupe(fileobj, 1);
			 };
			 fileobj.img.fullPrev.src = reader.result;
			 };
			 reader.readAsDataURL(fileobj.file);
			 */
		}
	};

	this.loadLoupFromdataUrl = function (fileobj) {
		/*
		 var bin = '', base64 = '';
		 if(!fileobj.dataUrl){
		 for (var i = 0; i < fileobj.dataArray.byteLength; i++) {
		 bin += String.fromCharCode(fileobj.dataArray[ i ]);
		 }
		 base64 = 'data:' + fileobj.type + ';base64,' + self.uploader.win.btoa(bin);
		 } else {
		 base64 = fileobj.dataUrl;
		 }
		 */
		fileobj.img.fullPrev.src = fileobj.dataUrl;
	};

	this.handleLoupImageOnLoad = function (fileobj) {
		this.utils.logTimeFromStart('end load fullpreview');
		this.setPreviewLoupe(fileobj, 1);
	};

	this.setStateItem = function (state, index, fileobj) {
		var item;
		var items = [];
		var parentNode;
		var preview;

		if (index !== -1) {
			if ((item = this.uploader.doc.getElementById('uploadItem_' + index))) {
				items = [item];
				parentNode = item;
			}
		} else {
			items = this.uploader.doc.getElementsByClassName('uploadItem');
			parentNode = this.uploader.doc;
		}

		if (!parentNode) {
			return;
		}

		var progresses = parentNode.getElementsByClassName('elemProgress');
		var typeErrors = parentNode.getElementsByClassName('elemTypeError');
		var textTypeErrorTypes = parentNode.getElementsByClassName('textTypeErrorType');
		var textTypeErrorExplains = parentNode.getElementsByClassName('textTypeErrorExplain');
		var elemSizes = parentNode.getElementsByClassName('elemSize');
		var elemSizeNoks = parentNode.getElementsByClassName('elemSizeNok');
		var imgPreviews = parentNode.getElementsByClassName('imgPreview');
		var elemUploadErrors = parentNode.getElementsByClassName('elemUploadError');
		var textShowFilenames = parentNode.getElementsByClassName('textShowFilename');
		var elemFilenameFulls = parentNode.getElementsByClassName('elemFilenameFull');



		for (var i = 0; i < items.length; i++) {//UPLOADING, CHUNK_DONE, UPLOAD_DONE, SIZE_OK, SIZE_NOK, TYPE_OK, TYPE_NOK,
			switch (state) {
				case this.controller.STATE.ITEM_UPLOADING:
					fileobj = this.sender.preparedFiles[items[i].getAttribute('data-index')];
					if (fileobj && fileobj.isUploadable) {
						progresses[i].style.display = 'block';
						if ((preview = items[i].getElementsByClassName('imgPreview')[0])) {
							preview.style.opacity = 0.2;
						}
					}

					textShowFilenames[i].style.display = 'inline';
					elemFilenameFulls[i].style.display = 'none';

					// FIXME: remove listener btn delete and loupe listners
					items[i].getElementsByClassName('textShowFilename')[0].removeEventListener('click', this.controller.editFilename);

					var imgPreview = items[i].getElementsByClassName('imgPreview')[0];
					var preview;
					if (imgPreview && (preview = imgPreview.firstChild)) {
						// FIXME: preview exists but it doesn't work...
						preview.removeEventListener('mouseenter', this.setPreviewLoupe);
						preview.removeEventListener('mousemove', this.movePreviewLoupe);
						preview.removeEventListener('mouseleave', this.unsetPreviewLoupe);
						preview.removeEventListener('click', this.grabFocusPoint);
					}
					break;
				case this.controller.STATE.ITEM_UPLOAD_NOK:
					if (fileobj.failedRequest) {
						elemUploadErrors[i].style.display = 'block';
						elemUploadErrors[i].lastChild.innerHTML = fileobj.failedRequest;
					}
					this.setInternalProgressCompleted(false, index);
					break;
				case this.controller.STATE.ITEM_UPLOADED:
					//items[i].style.backgroundColor = 'white';
					break;
				case this.controller.STATE.ITEM_SIZE_OK:
					elemSizes[i].classList.remove('hidden');
					elemSizeNoks[i].classList.add('hidden');
					break;
				case this.controller.STATE.ITEM_SIZE_NOK:
					elemSizes[i].classList.add('hidden');
					elemSizeNoks[i].classList.remove('hidden');
					items[i].style.backgroundColor = '#eeeeee';
					break;
				case this.controller.STATE.ITEM_TYPE_OK:
					break;
				case this.controller.STATE.ITEM_TYPE_NOK:
					typeErrors[i].style.display = 'block';
					textTypeErrorTypes[i].innerHTML = fileobj.type;
					textTypeErrorExplains[i].innerHTML = fileobj.isTypeForbiddenByUploader ? WE().consts.g_l.fileupload.typeNokExplainNotAlloud : WE().consts.g_l.fileupload.typeNokExplainNoPerms;
					items[i].style.backgroundColor = '#eeeeee';
					break;
			}
		}
	};

	// TODO: implement uploadable = false here! it is called after each relevant actoion!
	this.setStateItemEdit = function (preset, index, general, setDimensions) {
		var indices;
		var parentNode;
		var elems;

		if (index === -1) {
			indices = this.imageEdit.getImageEditIndices(-1, true, false);
			parentNode = this.uploader.doc;
			elems = parentNode.getElementsByClassName('uploadItem');
		} else {
			indices = [index];
			parentNode = this.uploader.doc.getElementById('uploadItem_' + index);
			elems = [parentNode];
		}

		var sizes = parentNode.getElementsByClassName('elemSize');
		var buttons = parentNode.getElementsByClassName('rowBtnProcess');
		var asteriskes = parentNode.getElementsByClassName('rowEditHot');
		var infoTops = parentNode.getElementsByClassName('rowDimensionsProcessed');
		var infoMiddles = parentNode.getElementsByClassName('infoMiddle');
		var infoMiddlesRight = parentNode.getElementsByClassName('rowRotation');
		var infoBottoms = parentNode.getElementsByClassName('infoBottom');
		var progresses = parentNode.getElementsByClassName('elemProgress');
		var elemScaleNotices = parentNode.getElementsByClassName('elemScaleNotice');
		var fileobj, i, j, state, deg, dimensions;

		for (i = 0; i < indices.length; i++) {
			j = indices[i];
			fileobj = this.sender.preparedFiles[j];
			state = preset ? preset : (fileobj.isEdited ? this.controller.STATE.IMG_PROCESSED : (fileobj.img.editOptions.doEdit ? this.controller.STATE.IMG_NOTPROCESSED : this.controller.STATE.IMG_DONOTEDIT));

			switch (state) {
				case this.controller.STATE.IMG_NOTPROCESSED:
					elems[i].style.backgroundColor = 'rgb(244, 255, 244)';
					sizes[i].innerHTML = WE().consts.g_l.fileupload.sizeTextOk + '--';

					if (setDimensions === 'flip') {
						switch (fileobj.img.editOptions.scaleWhat) {
							case 'pixel_l':
								dimensions = fileobj.img.lastHeightShown + ' x ' + fileobj.img.lastWidthShown + ' px';
								break;
							case 'pixel_w':
							case 'pixel_h':
								dimensions = fileobj.img.editOptions.scale ? '--' : fileobj.img.lastHeightShown + ' x ' + fileobj.img.lastWidthShown + ' px';
						}
					}

					infoTops[i].innerHTML = setDimensions ? (setDimensions === 'flip' ? (fileobj.img.editOptions.scale ? WE().consts.g_l.fileupload.editScaled + ' ' : '') + dimensions : infoTops[i].innerHTML) : '--';
					infoMiddles[i].style.display = 'block';
					deg = fileobj.img.editOptions.rotate;
					infoMiddlesRight[i].innerHTML = (fileobj.img.orientationIsProposition ? WE().consts.g_l.fileupload.autocorrectLandscape + ': ' : '') + (deg === 0 ? '0&deg;' : (deg === 90 ? '90&deg; '/* + WE().consts.g_l.fileupload.editRotationRight*/ : (deg === 270 ? '270&deg; '/* + WE().consts.g_l.fileupload.editRotationLeft*/ : '180&deg;')));
					infoBottoms[i].style.display = 'block';
					elemScaleNotices[i].style.display = 'none';
					if (setDimensions === 'flip') {
						var tmp = fileobj.img.lastWidthShown;
						fileobj.img.lastWidthShown = fileobj.img.lastHeightShown;
						fileobj.img.lastHeightShown = tmp;
					}
					break;
				case this.controller.STATE.IMG_PROCESSED:
					elems[i].style.backgroundColor = 'rgb(216, 255, 216)';
					elems[i].style.backgroundImage = 'none';

					// FIXME: fileobj.img.editedWidth is undefined when only qualitiy was changed
					var w, h;
					if (fileobj.dataUrl) {//this.sender.preparedFiles[j];
						h = fileobj.img.editedHeight ? fileobj.img.editedHeight : (fileobj.img.lastHeightShown ? fileobj.img.lastHeightShown : fileobj.img.origHeight);
						w = fileobj.img.editedWidth ? fileobj.img.editedWidth : (fileobj.img.lastWidthtShown ? fileobj.img.lastWidthShown : fileobj.img.origWidth);
					}
					infoTops[i].innerHTML = (fileobj.img.editOptions.scale ? WE().consts.g_l.fileupload.editScaled + ' ' : '') + (this.sender.preparedFiles[j].dataUrl ? w + ' x ' + h + ' px' : '--');
					deg = fileobj.img.editOptions.rotate;
					elemScaleNotices[i].style.display = fileobj.img.tooSmallToScale ? 'inline-block' : 'none';
					infoMiddles[i].style.display = 'block';
					infoMiddlesRight[i].innerHTML = (deg === 0 ? '0&deg;' : (deg === 90 ? '90&deg; '/* + WE().consts.g_l.fileupload.editRotationRight*/ : (deg === 270 ? '270&deg; '/* + WE().consts.g_l.fileupload.editRotationLeft*/ : '180&deg;')));
					infoBottoms[i].style.display = 'block';
					// FIXME: fileobj.img.editedWidth is undefined when only qualitiy was changed
					fileobj.img.lastWidthShown = fileobj.img.editedWidth ? fileobj.img.editedWidth : fileobj.img.lastWidthShown;
					fileobj.img.lastHeightShown = fileobj.img.editedHeight ? fileobj.img.editedHeight : fileobj.img.lastHeightShown;
					break;
				case this.controller.STATE.UPLOADING:
					progresses[i].style.display = 'block';
					return;
				case this.controller.STATE.UPLOADED:
					//elems[i].style.backgroundColor = 'white';
					return;
				case this.controller.STATE.IMG_DONOTEDIT:
				/*falls through*/
				default:
					infoTops[i].innerHTML = fileobj.img.origWidth ? fileobj.img.origWidth + ' x ' + fileobj.img.origHeight + ' px' : '--';
					elems[i].style.backgroundColor = 'rgb(232, 232, 255)';
					elems[i].style.backgroundImage = 'none';
					elemScaleNotices[i].style.display = fileobj.img.tooSmallToScale ? 'inline-block' : 'none';
					infoMiddles[i].style.display = 'none';
					infoMiddlesRight[i].innerHTML = '';
					infoBottoms[i].style.display = 'none';
					fileobj.img.lastWidthShown = fileobj.img.origWidth;
					fileobj.img.lastHeightShown = fileobj.img.origHeight;
			}
			buttons[i].disabled = !!fileobj.dataUrl; // FIXME: error in base
			asteriskes[i].style.color = fileobj.dataUrl ? 'lightgray' : 'black';
		}
	};

	this.setStateGUI = function (state, fileobj) {
		switch (state) {
			case this.controller.STATE.READY:
				fileobj = fileobj || this.sender.preparedFiles[0];
				this.setDisplay('divControlls', (this.controls.showControls !== this.CONTROLS_SHOW_FALSE ? 'block' : 'none'));
				this.setDisplay('divEditOptions', (this.controls.showEditOptions === this.CONTROLS_SHOW_TRUE || (this.controls.showEditOptions === this.CONTROLS_SHOW_AUTO && fileobj.isEditable) ? 'block' : 'none'));
				this.setDisplay('divImportMeta', (this.controls.showImportMeta === this.CONTROLS_SHOW_TRUE || (this.controls.showImportMeta === this.CONTROLS_SHOW_AUTO && fileobj.isMetaToImport) ? 'block' : 'none'));
				this.disableBtnUploadCancel(false, true);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);

				return;
			case this.controller.STATE.NOTREADY:
				fileobj = fileobj || this.sender.preparedFiles[0];
				this.setDisplay('divControlls', (this.controls.showControls !== this.CONTROLS_SHOW_FALSE ? 'block' : 'none'));
				if (fileobj.isTypeOk && fileobj.isEditable && this.controls.showEditOptions !== this.CONTROLS_SHOW_FALSE) { // too big but editable!
					this.setDisplay('divControlls', 'block');
					this.setDisplay('divImportMeta', 'block');
				}
				this.setDisplay('divImportMeta', (this.controls.showImportMeta === this.CONTROLS_SHOW_TRUE || (this.controls.showImportMeta === this.CONTROLS_SHOW_AUTO && fileobj.isEditable) ? 'block' : 'none'));
				this.disableBtnUploadCancel(true, true);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				return;
			case this.controller.STATE.UPLOADING:
				this.disableBtnUploadCancel(true, false);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', true);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', true);
				break;
			case this.controller.STATE.UPLOAD_NOK:
				this.disableBtnUploadCancel(true, true);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				break;
			case this.controller.STATE.FINISHED:
				this.disableBtnUploadCancel(true, true);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', true);
				break;
			case this.controller.STATE.CANCELLED:
				this.disableBtnUploadCancel(true, true);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				return;
			case this.controller.STATE.RESET:
				this.setDisplay('divEmptyDropzone', 'block');
				this.setDisplay('divUploadItem', 'none');
				if (this.controls.showControls !== this.CONTROLS_SHOW_TRUE) {
					this.setDisplay('divControlls', 'none');
				}
				this.setDisplay('divEditOptions', (this.controls.showEditOptions !== this.CONTROLS_SHOW_TRUE) ? 'none' : 'block');
				this.setDisplay('divImportMeta', (this.controls.showImportMeta !== this.CONTROLS_SHOW_TRUE) ? 'none' : 'block');
				this.disableBtnUploadCancel(true, true);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', true);
				return;
		}
	};

	this.disableBtnUploadCancel = function (disableUpload, disableCancel) {
		WE().layout.button.disable(this.uploader.doc, 'upload_btn', disableUpload);
		WE().layout.button.disable(this.uploader.doc, 'cancel_btn', disableCancel);
	};

	this.showMessageCancel = function () {
		WE().util.showMessage(WE().consts.g_l.fileupload.uploadCancelled, WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);
	};

	this.setImageEditMessage = function () {
		this.setImageEditMessageItem(0);
	};

	this.setImageEditMessageItem = function (index) {
		index = index ? index : 0;
		var item = this.uploader.doc.getElementById('uploadItem_' + index);
		var busy = item.getElementsByClassName('containerItemBusy')[0];

		busy.style.display = 'block';
		busy.getElementsByClassName('elemText')[0].innerHTML = this.imageEdit.currentEdit.task === 'add' ? /*WE().consts.g_l.fileupload.maskReadImage*/'' : WE().consts.g_l.fileupload.maskProcessImage;// IMI FIXME
	};

	this.unsetImageEditMessage = function () {
		this.unsetImageEditMessageItem(0);
	};

	this.unsetImageEditMessageItem = function (index) {
		index = index ? index : 0;
		var item = this.uploader.doc.getElementById('uploadItem_' + index);

		item.getElementsByClassName('containerItemBusy')[0].style.display = 'none';
	};

	this.repaintImageEditMessage = function (increment) {
		this.repaintImageEditMessageItem(0, increment);
	};

	this.repaintImageEditMessageItem = function (index, increment) {
		var item = this.uploader.doc.getElementById('uploadItem_' + index);
		var textDiv = item.getElementsByClassName('elemText')[0];
		var text = textDiv.innerHTML;

		text = (increment ? text : WE().consts.g_l.fileupload.maskProcessImage) + '.';
		//text += '.';
		textDiv.innerHTML = text;
	};

	this.movePreviewLoupe = function (fileobj, e) {
		if (fileobj.isUploadStarted) {
			return;
		}
		try {
			if (e.timeStamp - this.lastklick < 10) {
				// in Chrome onclick fires mosemove too: this causes the newly set focuspoint to be slightly wrong...
				return;
			}

			if (fileobj.loupInner && fileobj.loupInner.firstChild) {
				var offsetLeft = (-fileobj.loupInner.firstChild.width / fileobj.img.previewCanvas.width * e.offsetX) + (fileobj.loupInner.parentNode.offsetWidth / 2);
				var offsetTop = (-fileobj.loupInner.firstChild.height / fileobj.img.previewCanvas.height * e.offsetY) + (fileobj.loupInner.parentNode.offsetHeight / 2);
				this.offesetLeft = offsetLeft;
				this.offsetTop = offsetTop;

				fileobj.loupInner.style.left = Math.round(offsetLeft) + 'px';
				fileobj.loupInner.style.top = Math.round(offsetTop) + 'px';

				fileobj.focusPoint.style.left = Math.round(offsetLeft + ((parseFloat(fileobj.img.focusX) + 1) / 2) * fileobj.img.fullPrev.width) + 'px';
				fileobj.focusPoint.style.top = Math.round(offsetTop + ((parseFloat(fileobj.img.focusY) + 1) / 2) * fileobj.img.fullPrev.height) + 'px';
			}
		} catch (ex) {
			//
		}
	};

	this.unsetPreviewLoupe = function (fileobj) {
		var doc = this.uploader.doc;

		this.loupeVisible = false;
		if (fileobj.loupInner) {
			fileobj.loupInner.style.display = 'none';
			fileobj.loupInner.parentNode.style.display = 'none';
			fileobj.loupInner.innerHTML = '';
		}
		if (fileobj.focusPoint) {
			fileobj.focusPoint.style.display = 'none';
			fileobj.focusPointFixed.style.display = 'none';
		}

		doc.getElementById('we_fileUpload_loupeInfo').style.display = 'none';
		doc.getElementById('we_fileUpload_loupeFallback').style.display = 'none';
		doc.getElementsByClassName('editorCrosshairH')[0].style.display = 'none';
		doc.getElementsByClassName('editorCrosshairV')[0].style.display = 'none';

		var mask = doc.getElementsByClassName('containerUploaderBusy__')[0];
		if (mask) {
			//mask.style.display = 'none';
		}
	};

	this.grabFocusPoint = function (fileobj, e) {
		if (!this.loupeVisible) {
			return;
		}
		this.lastklick = e.timeStamp;
		if (fileobj.img.previewCanvas.width && fileobj.img.previewCanvas.height) {
			fileobj.focusPoint.style.display = 'none';
			fileobj.focusPointFixed.style.display = 'block';
			var focusX = ((e.offsetX / fileobj.img.previewCanvas.width) * 2) - 1;
			var focusY = ((e.offsetY / fileobj.img.previewCanvas.height) * 2) - 1;
			fileobj.img.focusX = focusX.toFixed(2);
			fileobj.img.focusY = focusY.toFixed(2);
			this.writeFocusToForm(fileobj);
			window.setTimeout(this.unmarkFocusPoint, 400, fileobj);
		}
	};

	this.unmarkFocusPoint = function (fileobj) {
		if (fileobj.focusPoint && fileobj.focusPointFixed) {
			fileobj.focusPoint.style.top = (fileobj.loupInner.parentNode.offsetHeight / 2) + 'px';
			fileobj.focusPoint.style.left = (fileobj.loupInner.parentNode.offsetWidth / 2) + 'px';
			fileobj.focusPoint.style.display = 'block';
			fileobj.focusPointFixed.style.display = 'none';
		}
	};

	this.writeFocusToForm = function (filobj) {
		// to be overridden
	};

	//TODO: adapt these progress fns to standard progressbars
	this.setInternalProgressText = function (name, text, index) {
		var p = typeof index === 'undefined' || index === false ? '' : '_' + index;
		this.uploader.doc.getElementById('span_' + this.uploader.fieldName + '_' + name + p).innerHTML = text;
	};

	this.replacePreviewCanvas = function () {
		// to be overridden
	};

	this.setInternalProgress = function (progress, index) {
		var item, preview, opacity;

		index = index ? index : 0;

		// FIXME: check why we need this when adding file in base
		try {
			this.uploader.win.setProgress(progress, '_' + this.uploader.fieldName + '_' + index);

			item = this.uploader.doc.getElementById('uploadItem_' + index);
			if (item && (preview = item.getElementsByClassName('imgPreview')[0])) {
				opacity = Math.max(progress / 100, 0.2);
				preview.style.opacity = opacity.toFixed(2);
			}
		} catch (e) {
		}
	};

	this.setInternalProgressCompleted = function (success, index) {
		var preview, item;

		success = success || false;
		index = index ? index : 0;
		item = this.uploader.doc.getElementById('uploadItem_' + index);//elapsedTime_we_File_1

		this.uploader.doc.getElementById('elapsedTime_' + this.uploader.fieldName + '_' + index).classList.add('hidden');

		if (!success) {
			if (item) {
				item.getElementsByClassName('progress_image')[0].classList.add('progress_failed');
			}
			return;
		}

		this.uploader.win.setProgress(100, '_' + this.uploader.fieldName + '_' + index);
		if (item && (preview = item.getElementsByClassName('imgPreview')[0])) {
			preview.style.opacity = 1;
		}
	};

	this.setExternalProgress = function () {
		// ovverwrite
	};

	this.setExternalProgressText = function () {
		// overwrite
	};

	this.setDisplay = function (elem, val) {
		if (this.elems[elem]) {
			this.elems[elem].style.display = val;
		}
	};

	this.addTextCutLeft = function (elem, text, maxwidth) {
		if (!elem) {
			return;
		}

		maxwidth = maxwidth || 30;
		text = text ? text : '';
		var i = 500, first = true;
		elem.innerHTML = text;
		while (elem.offsetWidth > maxwidth && i > 0) {
			text = text.substr(first ? 4 : 2);
			first = false;
			elem.innerHTML = '...' + text;
			--i;
		}

	};
}

function Fileupload_view_base(uploader) {
	Fileupload_view_abstract.call(this, uploader);

	this.uploader = uploader;
}
Fileupload_view_base.prototype = Object.create(Fileupload_view_abstract.prototype);
Fileupload_view_base.prototype.constructor = Fileupload_view_base;

function Fileupload_view_bindoc(uploader) {
	Fileupload_view_abstract.call(this, uploader);

	this.uploader = uploader;

	this.icon = '';
	this.binDocType = 'other';
	this.previewSize = 110;
	this.useOriginalAsPreviewIfNotEdited = true;

	this.onload_sub = function () {
		var doc = this.uploader.doc;
		// TODO: move most of these to abstract onload
		this.elems.divEmptyDropzone = doc.getElementById('div_empty_dropzone');
		this.elems.divUploadItem = doc.getElementById('div_upload_files');
		this.elems.divControlls = doc.getElementById('fileupload_controlls');
		this.elems.divEditOptions = doc.getElementById('fileupload_editoptions');
		this.elems.divImportMeta = doc.getElementById('fileupload_importmeta');
		this.elems.divBtnReset = doc.getElementById('div_fileupload_btnReset');
		this.elems.divBtnCancel = doc.getElementById('div_fileupload_btnCancel');
		this.elems.divBtnUpload = doc.getElementById('div_fileupload_btnUpload');

		if (this.uploader.uiType === 'editor') {
			this.elems.divBtnCancelFooter = this.uploader.doc.getElementById('cancel_btn_footer').parentNode;
			this.elems.divBtnCloseFooter = this.uploader.doc.getElementById('close_btn_footer').parentNode;
		}

		this.controller.processEvents(this.controller.ON.GUI_INIT);
	};

	this.appendItem = function (item) {
		var container = this.uploader.doc.getElementById('div_upload_files');
		container.innerHTML = item;
		container.previousSibling.style.display = 'none';
		container.style.display = 'block';
	};

	this.disableBtnUploadCancel = function (disableUpload, disableCancel) {
		switch (this.uploader.uiType) {
			case 'editor':
				WE().layout.button.disable(this.uploader.doc, 'upload_btn_footer', disableUpload);
				WE().layout.button.disable(this.uploader.doc, 'cancel_btn_footer', disableCancel);
				WE().layout.button.disable(this.uploader.doc, 'close_btn_footer', !disableCancel);
				if (disableCancel) {
					this.elems.divBtnCancelFooter.classList.add('hidden');
					this.elems.divBtnCloseFooter.classList.remove('hidden');
				} else {
					this.elems.divBtnCancelFooter.classList.remove('hidden');
					this.elems.divBtnCloseFooter.classList.add('hidden');
				}
				break;
			case 'wedoc':
			default:
				WE().layout.button.disable(this.uploader.doc, 'upload_btn', disableUpload);
				WE().layout.button.disable(this.uploader.doc, 'cancel_btn', disableCancel);
		}
	};

	this.hideOptions = function (hide) {
		var doc = this.uploader.doc;
		if (hide) {
			doc.getElementById('editImage').style.display = 'none';
			doc.getElementById('div_importMeta').style.display = 'none';
			doc.getElementById('tr_alert').style.display = 'none';
		} else {
			doc.getElementById('editImage').style.display = 'block';
			doc.getElementById('div_importMeta').style.display = 'block';
			doc.getElementById('tr_alert').style.display = 'table-row';
		}
	};

	this.writeFocusToForm_ = function (fileobj) { // FIXME: obsolete?
		var doc = this.uploader.doc;

		doc.we_form.elements.fu_doc_focusX.value = fileobj.img.focusX;
		doc.we_form.elements.fu_doc_focusY.value = fileobj.img.focusY;
	};

	// copy from importer
	this.replacePreviewCanvas = function (fileobj) {
		//var elem = fileobj.entry.getElementsByClassName('imgPreview')[0];// why don't we have entry saved to fileobj?
		var elem = this.uploader.doc.getElementsByClassName('imgPreview')[0];
		elem.innerHTML = '';
		elem.appendChild(fileobj.img.previewCanvas);
		elem.firstChild.setAttribute('data-index', fileobj.index);

		elem.firstChild.addEventListener('mouseenter', this.setPreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('mousemove', this.movePreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('mouseleave', this.unsetPreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('click', this.grabFocusPoint.bind(this, fileobj), false);
	};
}
Fileupload_view_bindoc.prototype = Object.create(Fileupload_view_abstract.prototype);
Fileupload_view_bindoc.prototype.constructor = Fileupload_view_bindoc;

function Fileupload_view_import(uploader) {
	Fileupload_view_abstract.call(this, uploader);
	this.messageWindow = null;
	this.previewSize = 110;

	this.onload_sub = function () {
		this.elems.footer = this.uploader.win.top.wizbusy;
		this.widthInputFilename = 180;
	};

	this.appendItem = function (item) {
		var tmpDiv = this.uploader.doc.createElement('DIV');

		tmpDiv.innerHTML = item;
		this.uploader.doc.getElementById('uploadFiles').appendChild(tmpDiv.firstChild);
		tmpDiv = null;
	};

	// move to abstract
	this.replacePreviewCanvas = function (fileobj) {
		var elem = fileobj.entry.getElementsByClassName('imgPreview')[0];
		elem.innerHTML = '';
		elem.appendChild(fileobj.img.previewCanvas);
		elem.firstChild.setAttribute('data-index', fileobj.index);

		elem.firstChild.addEventListener('mouseenter', this.setPreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('mousemove', this.movePreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('mouseleave', this.unsetPreviewLoupe.bind(this, fileobj), false);
		elem.firstChild.addEventListener('click', this.grabFocusPoint.bind(this, fileobj), false);
	};

	this.setImageEditMessage = function () {
		if (this.imageEdit.currentEdit.mode === 'item') {
			this.setImageEditMessageItem(this.imageEdit.currentEdit.index);
			return;
		}

		var doc = this.uploader.doc;
		var elem;
		if ((elem = doc.getElementsByClassName('containerUploaderBusy')[0])) {
			elem.getElementsByClassName('elemText')[0].style.display = 'block';
			elem.getElementsByClassName('elemText')[0].innerHTML = this.imageEdit.imageEditOptions.doEdit ? WE().consts.g_l.fileupload.maskImporterProcessImages : WE().consts.g_l.fileupload.maskImporterReadImages;
			try {
				doc.getElementById('we_fileUploadImporter_messageNr').innerHTML = this.imageEdit.imageFilesToProcess.length;
			} catch (e) {
			}
			elem.style.zIndex = 800;
			doc.getElementsByClassName('uploaderMask')[0].style.display = 'block';
			elem.style.display = 'block';
		}
	};

	this.unsetImageEditMessage = function () {
		if (this.imageEdit.currentEdit.mode === 'item') {
			this.unsetImageEditMessageItem(this.imageEdit.currentEdit.index);
			return;
		}

		var doc = this.uploader.doc;
		var elem;
		if ((elem = doc.getElementsByClassName('containerUploaderBusy')[0])) {
			elem.style.display = 'none';
			doc.getElementsByClassName('uploaderMask')[0].style.display = 'none';
			elem.getElementsByClassName('elemText')[0].style.display = 'none';
		}
	};

	this.repaintImageEditMessage = function () {
		if (this.imageEdit.currentEdit.mode === 'item') {
			this.repaintImageEditMessageItem(this.imageEdit.currentEdit.index, true);
			return;
		}
		var doc = this.uploader.doc;
		var elem;
		if ((elem = doc.getElementsByClassName('containerUploaderBusy')[0])) {
			elem.getElementsByClassName('elemText')[0].style.display = 'block';
			elem.getElementsByClassName('elemText')[0].innerHTML = (this.imageEdit.currentEdit.task === 'edit' ? WE().consts.g_l.fileupload.maskImporterProcessImages : WE().consts.g_l.fileupload.maskImporterReadImages);
			doc.getElementById('we_fileUploadImporter_messageNr').innerHTML = this.imageEdit.imageFilesToProcess.length;
		}
	};

	this.removeItem = function (index) {
		var item = this.uploader.doc.getElementById('uploadItem_' + index);
		var container = this.uploader.doc.getElementById('uploadFiles');
		container.removeChild(item);

		this.imageEdit.memorymanagerUnregister(this.sender.preparedFiles[index]);
		this.sender.preparedFiles[index] = null;

		this.controller.processEvents(this.controller.ON.FILE_REMOVE);
	};

	this.showMessageCancel = function (filesFinished) {
		var message = WE().consts.g_l.fileupload.uploadCancelled +
						(filesFinished ? '</br></br>' + WE().consts.g_l.fileupload.uploadedBeforeCancelling + filesFinished : '');

		WE().util.showMessage(message, WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);
	};

	this.reloadOpener = function () {
		try {
			var activeFrame = WE().layout.weEditorFrameController.getActiveEditorFrame();
			if (parseInt(this.uploader.doc.we_form.fu_file_parentID.value) === activeFrame.EditorDocumentId && activeFrame.EditorEditPageNr === 16) {
				top.we_cmd('switch_edit_page', 16, activeFrame.EditorTransaction);
			}
			top.we_cmd('loadIfActive', WE().consts.tables.FILE_TABLE);
		} catch (e) {
			//
		}
	};

	this.setStateGUI = function (state) { // FIXME: use displaBtnUploadCancel here too
		var doc = this.uploader.doc;
		this.elems.footer.document.getElementsByClassName('weBtnCancel')[0].style.display = 'none';
		this.elems.footer.document.getElementsByClassName('weBtnClose')[0].style.display = 'inline-block';

		switch (state) {
			case this.controller.STATE.READY:
				WE().layout.button.disable(this.elems.footer.document, 'upload', false);
				break;
			case this.controller.STATE.NOTREADY:
				WE().layout.button.disable(this.elems.footer.document, 'upload', true);
				break;
			case this.controller.STATE.UPLOADING:
				this.elems.footer.document.getElementsByClassName('weBtnCancel')[0].style.display = '';
				this.elems.footer.document.getElementsByClassName('weBtnClose')[0].style.display = 'none';
				WE().layout.button.disable(this.elems.footer.document, 'back', true);
				WE().layout.button.disable(this.elems.footer.document, 'upload', true);
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', true);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', true);

				this.elems.footer.document.getElementById('progressbar').style.display = '';
				this.setExternalProgress(0);
				doc.getElementById('div_upload_files').scrollTop = 0;

				break;
			case this.controller.STATE.UPLOAD_FILE_NOK:
				// go on uploading next file
				break;
			case this.controller.STATE.FINISHED:
				break;
			case this.controller.STATE.CANCELLED:
				this.elems.footer.document.getElementById('progressbar').style.display = 'none';

				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				WE().layout.button.disable(this.elems.footer.document, 'upload', true);
				WE().layout.button.disable(this.elems.footer.document, 'back', false);
				break;
			case this.controller.STATE.RESET:
				doc.getElementById('uploadFiles').innerHTML = '';
				this.elems.footer.document.getElementById('progressbar').style.display = 'none';
				/* falls through */
			case this.controller.STATE.EMPTY:
				WE().layout.button.disable(this.uploader.doc, 'reset_btn', false);
				WE().layout.button.disable(this.uploader.doc, 'browse_harddisk_btn', false);
				WE().layout.button.disable(this.elems.footer.document, 'upload', true);
				WE().layout.button.disable(this.elems.footer.document, 'back', false);
				break;
		}
	};

	this.setExternalProgress = function (progress) {
		this.elems.footer.setProgress(progress);
	};

	this.setExternalProgressText = function (index) {
		this.elems.footer.setProgressText('progress_title', WE().consts.g_l.fileupload.doImport + ' ' + (index + 1));
	};
}
Fileupload_view_import.prototype = Object.create(Fileupload_view_abstract.prototype);
Fileupload_view_import.prototype.constructor = Fileupload_view_import;
