/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/* global WE, top */
'use strict';

function doUnload() {
	WE().util.jsWindow.prototype.closeAll(window);
}

function selectBranch() {
	var f = document.we_form;
	var txt = f.branch;
	var sel = f.branch_select.options[f.branch_select.selectedIndex].text;
	f.cmd.value = "switchBranch";
	txt.value = sel;
	submitForm();
}

function saveField() {
	document.we_form.cmd.value = "save_field";
	submitForm();
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window);
	var args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	var url = WE().util.getWe_cmdArgsUrl(args, WE().consts.dirs.WEBEDITION_DIR + "we_showMod.php?mod=customer&");

	var branch, field;
	switch (args[0]) {
		case "open_add_field":
			branch = document.we_form.branch.value;
			url += "&pnt=field_editor&art=add&branch=" + branch;
			new (WE().util.jsWindow)(caller, url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny, true, false, true);
			break;
		case "open_edit_field":
			field = document.we_form.fields_select.value;
			branch = document.we_form.branch.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				url += "&pnt=field_editor&art=edit&field=" + field + "&branch=" + branch;
				new (WE().util.jsWindow)(caller, url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny, true, false, true);
			}
			break;
		case "delete_field":
			field = document.we_form.fields_select.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
				break;
			}
			WE().util.showConfirm(window, "", WE().consts.g_l.customer.admin.del_fild_question, ["delete_field_do"]);
			break;
		case "delete_field_do":
			document.we_form.cmd.value = "delete_field";
			submitForm();
			break;
		case "move_field_up":
		case "move_field_down":
			field = document.we_form.fields_select.value;
			branch = document.we_form.branch.value;
			if (field === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_field, WE().consts.message.WE_MESSAGE_ERROR, window);
				break;
			}
			document.we_form.cmd.value = args[0];
			submitForm();

			break;
		case "open_edit_branch":
			branch = document.we_form.branch_select.options[document.we_form.branch_select.selectedIndex].text;
			if (branch === "") {
				WE().util.showMessage(WE().consts.g_l.customer.admin.no_branch, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else if (branch === WE().consts.g_l.customer.admin.other) {
				WE().util.showMessage(WE().consts.g_l.customer.admin.branch_no_edit, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				url += "&pnt=branch_editor&art=edit&&branch=" + branch;
				new (WE().util.jsWindow)(caller, url, "field_editor", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny, true, false, true);
			}
			break;
		case "save_branch":
		case "save_field":
			var field_name = document.we_form.name.value;
			if (field_name === "" || field_name.match(/[^a-zA-Z0-9\_]/) !== null) {
				WE().util.showMessage(WE().consts.g_l.customer.admin.we_fieldname_notValid, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				document.we_form.cmd.value = args[0];
				submitForm("field_editor");
			}
			break;
		case 'submitForm':
		if (opener.top.content) {
			submitForm();
			break;
		}
		//falls through
		default:
			//FIXME: this js is included in 2 dialogs - this causes problems! e.g. no customer selected
			if (opener.top.content) {
				opener.top.content.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			} else {
				opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			}
	}
}

function setStatusEncryption(type) {
	var encSelect = document.getElementsByName('field_encrypt')[0];
	if (!encSelect) {
		return;
	}
	switch (type) {
		case 'input':
		case 'textarea':
			encSelect.disabled = false;
			break;
		default:
			encSelect.disabled = true;
	}

}

function submitForm(target, action, method, form) {
	var f = form ? window.document.forms[form] : window.document.we_form;
	f.target = target ? target : "customer_admin";
	f.action = action ? action : WE().consts.dirs.WEBEDITION_DIR + "we_showMod.php?mod=customer";
	f.method = method ? method : "post";

	f.submit();
}

function setFieldsButtons(pos, max) {
	document.getElementById("editFieldButton").disabled = false;
	document.getElementById("deleteFieldButton").disabled = false;
	document.getElementById("moveFieldUpButton").disabled = (pos === 0);
	document.getElementById("moveFieldDownButton").disabled = (pos === (max - 1));
}