<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_ContentTypes{
	const IMAGE = 'image/*';
	const TEMPLATE = 'text/weTmpl';
	const XML = 'text/xml';
	const HTML = 'text/html';
	const WEDOCUMENT = 'text/webedition';
	const JS = 'text/js';
	const CSS = 'text/css';
	const HTACCESS = 'text/htaccess';
	const TEXT = 'text/plain';
	const FLASH = 'application/x-shockwave-flash';
	const VIDEO = 'video/*';
	const AUDIO = 'audio/*';
	const APPLICATION = 'application/*';
	const PDF = 'application/pdf';
	const FOLDER = 'folder';
	const CLASS_FOLDER = 'class_folder';
	const OBJECT = 'object';
	const OBJECT_FILE = 'objectFile';
	const COLLECTION = 'text/weCollection';
	const NAVIGATION = 'weNavigation';
	const NAVIGATIONRULE = 'weNavigationRule';
	const _CHARSET = '###CHARSET###';
	const CONTENT_TYPES = [
		self::IMAGE => [
			'Extension' => ['.gif', '.jpg', '.jpeg', '.png', '.svg', '.svgz'],
			'ContentTypes' => ['image/jpeg', 'image/pjpeg', 'image/gif', 'image/png', 'image/x-png', 'image/svg+xml', 'image/svg-xml', 'image/x-citrix-pjpeg'],
			'Permission' => we_base_perm::WORK_IMAGE_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_imageDocument'
		],
		self::XML => [//this entry must stay before text/html, text/we because fileextensions are not distinct
			'Extension' => '.xml',
			'Permission' => we_base_perm::WORK_TEXT_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_textDocument'
		],
		self::HTML => [
			'Extension' => ['.html', '.htm', '.shtm', '.shtml', '.stm', '.php', '.jsp', '.asp', '.aspx', '.pl', '.cgi', '.xml', '.xsl'],
			'Permission' => we_base_perm::WORK_HTML_NEW,
			'IsWebEditionFile' => true,
			'IsRealFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_htmlDocument'
		],
		self::WEDOCUMENT => [
			'Extension' => ['.html', '.htm', '.shtm', '.shtml', '.stm', '.php', '.jsp', '.asp', '.aspx', '.pl', '.cgi', '.xml'],
			'Permission' => we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW,
			'IsWebEditionFile' => true,
			'IsRealFile' => false,
			'Table' => [FILE_TABLE],
			'Class' => 'we_webEditionDocument'
		],
		self::TEMPLATE => [
			'Extension' => '.tmpl',
			'Permission' => we_base_perm::WORK_TEMPLATE_NEW,
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [TEMPLATES_TABLE],
			'Class' => 'we_template'
		],
		self::JS => [
			'Extension' => '.js',
			'Permission' => we_base_perm::WORK_JS_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_textDocument'
		],
		self::CSS => [
			'Extension' => ['.css', '.less', '.scss', '.sass'],
			'Permission' => we_base_perm::WORK_CSS_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_textDocument'
		],
		self::HTACCESS => [
			'Extension' => ['.htaccess', '.htpasswd'],
			'ExtensionIsFilename' => true,
			'Permission' => we_base_perm::WORK_HTACCESS_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_textDocument'
		],
		self::TEXT => [
			'Extension' => ['.txt', '.csv'],
			'Permission' => we_base_perm::WORK_TEXT_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_textDocument'
		],
		self::FOLDER => [
			'Extension' => '',
			'Permission' => '',
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [FILE_TABLE, TEMPLATES_TABLE, OBJECT_TABLE, OBJECT_FILES_TABLE, VFILE_TABLE],
			'Class' => 'we_folder'
		],
		self::CLASS_FOLDER => [
			'Extension' => '',
			'Permission' => '',
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [OBJECT_FILES_TABLE],
			'Class' => 'we_class_folder'
		],
		self::FLASH => ['Extension' => ['.swf'/* ,'.mp4','.m4v' */],
			'Permission' => we_base_perm::WORK_VIDEO_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_flashDocument'
		],
		self::VIDEO => [
			'Extension' => ['.mp4', '.m4v', '.ogg', '.webm'],
			'ContentTypes' => ['video/mp4', 'video/webm', 'application/ogg', 'video/ogg',],
			'Permission' => we_base_perm::WORK_VIDEO_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_document_video'
		],
		self::AUDIO => [
			'Extension' => ['.mp3', '.wav', '.ogg'],
			'ContentTypes' => ['audio/mp3', 'audio/ogg', 'audio/wav'],
			'Permission' => we_base_perm::WORK_OTHER_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_document_audio'
		],
		self::PDF => [
			'Extension' => ['.pdf'],
			'Permission' => we_base_perm::WORK_OTHER_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_contents_pdf'
		],
		self::APPLICATION => [
			'Extension' => ['.doc', '.xls', '.ppt', '.zip', '.sit', '.bin', '.hqx', '.exe', '.pdf'],
			'Permission' => we_base_perm::WORK_OTHER_NEW,
			'IsRealFile' => true,
			'IsWebEditionFile' => true,
			'Table' => [FILE_TABLE],
			'Class' => 'we_otherDocument'
		],
		self::OBJECT => ['Extension' => '',
			'Permission' => '',
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [OBJECT_TABLE],
			'Class' => 'we_object'
		],
		self::OBJECT_FILE => [
			'Extension' => '',
			'Permission' => '',
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [OBJECT_FILES_TABLE],
			'Class' => 'we_objectFile'
		],
		self::COLLECTION => [
			'Extension' => '',
			'Permission' => '',
			'IsRealFile' => false, //TODO: use this when saving
			'IsWebEditionFile' => false,
			'Table' => [VFILE_TABLE],
			'Class' => 'we_collection'
		],
		self::NAVIGATION => [
			'Extension' => '',
			'Permission' => '',
			'IsRealFile' => false,
			'IsWebEditionFile' => false,
			'Table' => [NAVIGATION_TABLE],
			'Class' => 'we_navigation_navigation'
		],
	];

	public static function hasContentType($name){
		return array_key_exists($name, self::CONTENT_TYPES);
	}

	public static function getContentTypes($filter = '', $filterOmitFolder = false){
		if($filter){
			$ret = [];
			foreach(self::CONTENT_TYPES as $k => $v){
				if(in_array($filter, $v['Table']) && !($filterOmitFolder && ($k === self::FOLDER || $k === self::CLASS_FOLDER))){
					$ret[] = $k;
				}
			}

			return $ret;
		}
		return array_keys(self::CONTENT_TYPES);
	}

	/**
	 * @param $name
	 * @param bool $ignoreIsFilename
	 * @return array
	 */
	public static function getExtension($name, $ignoreIsFilename = false){
		return array_key_exists($name, self::CONTENT_TYPES) && ($ignoreIsFilename || !array_key_exists('ExtensionIsFilename', self::CONTENT_TYPES[$name]) || !self::CONTENT_TYPES[$name]['ExtensionIsFilename']) ? self::CONTENT_TYPES[$name]['Extension'] : [];
	}

	public static function isWEFile($name){
		return array_key_exists($name, self::CONTENT_TYPES) ? self::CONTENT_TYPES[$name]['IsWebEditionFile'] : false;
	}

	public static function getWETypes(){
		$ret = [];
		foreach(self::CONTENT_TYPES as $name => $type){
			if($type['IsWebEditionFile']){
				$ret[] = $name;
			}
		}
		return $ret;
	}

	public static function getObject($name = ''){
		if(!$name){
			return false;
		}
		if(array_key_exists($name, self::CONTENT_TYPES) && class_exists(self::CONTENT_TYPES[$name]['Class'], true)){
			$classname = self::CONTENT_TYPES[$name]['Class'];
			return new $classname;
		}
		t_e('init with unknown contenttype', $name);
		$classname = 'we_' . $name;
		if(class_exists($classname, true)){
			return new $classname();
		}
		t_e('Can NOT initialize document of type -' . $name . '- ' . 'we_' . $name . '.inc.php');
		return false;
	}

	public static function getDefaultCode($name){
		$code = '';
		switch($name){
			case self::XML:
				$code = '<?xml version="1.0" encoding="' . self::_CHARSET . '" ?>';
				break;
			case self::HTML:
				$code = '<!doctype html>
<html>
	<head>
		<title></title>
		<meta charset="' . self::_CHARSET . '">
	</head>
	<body>
	<!-- webEdition demo template (' . WE_VERSION . ') -->
	</body>
</html>';
				break;
			case self::TEMPLATE:
				$code = '<!DOCTYPE HTML>
<html dir="ltr" lang="<we:pageLanguage type="language" doc="top" />">
<head>
	<we:title></we:title>
	<we:description></we:description>
	<we:keywords></we:keywords>
	<we:charset defined="UTF-8">UTF-8</we:charset>
</head>
<body>
	<!-- webEdition demo template (' . WE_VERSION . ') -->
	<article style="width:400px">
		<h1><we:input type="text" name="Headline" style="width:60em"/></h1>
		<p><b><we:input type="date" name="Date" currentdate="true" format="d.m.Y"/></b></p>
		<we:ifNotEmpty match="Image">
			<p><we:img name="Image" showthumbcontrol="true"/></p>
		</we:ifNotEmpty>
		<we:textarea name="Content" width="400" height="400" wysiwyg="true" removefirstparagraph="false" inlineedit="true"/>
	</article>
</body>
</html>';
				break;
		}
		return $code ? strtr($code, [self::_CHARSET => ($GLOBALS['we']['Charset']['Backend'] ?: 'UTF-8')]) : '';
	}

	public static function getPermission($name){
		return array_key_exists($name, self::CONTENT_TYPES) ? self::CONTENT_TYPES[$name]['Permission'] : '';
	}

	public static function getTypeForExtension($extension){
		foreach(self::CONTENT_TYPES as $type => $val){
			$ext = $val['Extension'];
			if((is_array($ext) && in_array($extension, $ext)) || $ext == $extension){
				return $type;
			}
		}
		return '';
	}

	public static function getRealContentTypes($name){
		return (array_key_exists($name, self::CONTENT_TYPES) && array_key_exists('ContentTypes', self::CONTENT_TYPES[$name])) ? self::CONTENT_TYPES[$name]['ContentTypes'] : [];
	}

	public static function getFiles(){
		$ret = [];
		foreach(self::CONTENT_TYPES as $type => $val){
			if($val['IsRealFile']){
				$ret[] = $type;
			}
		}
		return $ret;
	}

}
