<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_charsetHandler{
	private $charsets = [];
	private static $inst = null;

	const CHARSETS = [
		'UTF-8',
		'ISO-8859-1',
		'ISO-8859-2',
		'ISO-8859-3',
		'ISO-8859-4',
		'ISO-8859-5',
		'ISO-8859-6',
		'ISO-8859-7',
		'ISO-8859-8',
		'ISO-8859-9',
		'ISO-8859-10',
		'ISO-8859-11',
		'ISO-8859-12',
		'ISO-8859-13',
		'ISO-8859-14',
		'ISO-8859-15',
		'Windows-1251',
		'Windows-1252',
	];

	/**
	 * initialises with all available charsets
	 */
	private function __construct(){
		//	First ISO-8859-charsets
		$this->charsets = [
			'west_european' => [
				'national' => 'West Europe', //	Here is the name of the country in mother language
				'charset' => 'ISO-8859-1',
				'international' => g_l('charset', '[titles][west_european]'), //	Name in selected language
			],
			'central_european' => [
				'national' => 'Central Europe',
				'charset' => 'ISO-8859-2',
				'international' => g_l('charset', '[titles][central_european]'),
			],
			'south_european' => [
				'national' => 'South Europe',
				'charset' => 'ISO-8859-3',
				'international' => g_l('charset', '[titles][south_european]'),
			],
			'north_european' => [
				'national' => 'North Europe',
				'charset' => 'ISO-8859-4',
				'international' => g_l('charset', '[titles][north_european]'),
			],
			'cyrillic' => [
				'national' => '&#1077;&#1086;&#1073;&#1077;&#1089;&#1087;&#1077;&#1095;',
				'charset' => 'ISO-8859-5',
				'international' => g_l('charset', '[titles][cyrillic]'),
			],
			'arabic' => [
				'national' => '&#1578;&#1587;&#1580;&#1617;&#1604;&#1575;&#1604;&#1570;&#1606;',
				'charset' => 'ISO-8859-6',
				'international' => g_l('charset', '[titles][arabic]'),
			],
			'greek' => [
				'national' => 'Greek',
				'charset' => 'ISO-8859-7',
				'international' => g_l('charset', '[titles][greek]'),
			],
			'hebrew' => [
				'national' => '&#1488;&#1497;&#1512;&#1493;&#1508;&#1492;',
				'charset' => 'ISO-8859-8',
				'international' => g_l('charset', '[titles][hebrew]'),
			],
			'turkish' => [
				'national' => 'Turkish',
				'charset' => 'ISO-8859-9',
				'international' => g_l('charset', '[titles][turkish]'),
			],
			'nordic' => [
				'national' => 'Nordic',
				'charset' => 'ISO-8859-10',
				'international' => g_l('charset', '[titles][nordic]'),
			],
			'thai' => [
				'national' => 'Thai',
				'charset' => 'ISO-8859-11',
				'international' => g_l('charset', '[titles][thai]'),
			],
			'baltic' => [
				'national' => 'baltic',
				'charset' => 'ISO-8859-13',
				'international' => g_l('charset', '[titles][baltic]'),
			],
			'keltic' => [
				'national' => 'keltic',
				'charset' => 'ISO-8859-14',
				'international' => g_l('charset', '[titles][keltic]'),
			],
			'extended_european' => [
				'national' => 'ISO-8859-15',
				'charset' => 'ISO-8859-15',
				'international' => g_l('charset', '[titles][extended_european]'),
			],
			'unicode' => [
				'national' => 'Unicode',
				'charset' => 'UTF-8',
				'international' => g_l('charset', '[titles][unicode]'),
			],
			'windows_1251' => [
				'national' => 'Windows-1251',
				'charset' => 'Windows-1251',
				'international' => g_l('charset', '[titles][windows_1251]'),
			],
			'windows_1252' => [
				'national' => 'Windows-1252',
				'charset' => 'Windows-1252',
				'international' => g_l('charset', '[titles][windows_1252]'),
			],
		];
	}

	public static function inst(){
		return (self::$inst ?: (self::$inst = new self()));
	}

	/**
	 * @return array
	 * @param $availableChars array
	 * @desc This function returns an key = charset / value = charset - name(international) (name(national)))
	 */
	public function getCharsetsForTagWizzard(){
		$retArr = [];
		foreach($this->charsets as $val){
			$retArr[$val['charset']] = $val['charset'] . ' - ' . $val['international'] . ' (' . $val['national'] . ')';
		}
		return $retArr;
	}

	/**
	 * @return array|false
	 * @param string $charset
	 * @desc returns mixed (national, international, charset, when charset is known)
	 */
	public function getCharsetArrByCharset($charset){
		$charset = strtolower($charset);
		foreach($this->charsets as $val){
			if(strtolower($val['charset']) == $charset){
				return $val;
			}
		}
		return false;
	}

	/**
	 * @return array
	 * @param $availableChars array
	 * @desc This function returns an array for the property page of a webEdition document
	 */
	public function getCharsetsByArray($availableChars){
		$tmpCharArray = [];
		$retArr = [];

		foreach($availableChars as $char){
			$tmpCharArray[] = ($this->getCharsetArrByCharset($char) ?: ['charset' => $char]);
		}

		foreach($tmpCharArray as $val){
			$retArr[$val['charset']] = $val['charset'] .
				(isset($val['international']) ? ' - ' . $val['international'] . ' (' . $val['national'] . ')' : '');
		}

		return $retArr;
	}

	//FIXME: use array obove; currently this seems to complecated
	public static function getAvailCharsets(){
		return self::CHARSETS;
	}

	public static function isValidCharset($cs){
		return in_array($cs, self::CHARSETS);
	}

}
