<?php

/**
 * webEdition CMS
 *
 * $Rev: 14639 $
 * $Author: mokraemer $
 * $Date: 2017-12-05 02:49:40 +0100 (Tue, 05 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_gui_multiEditor{

	/**
	 * Checks if the start-document is a valid document. Content Type text/webedition or text/html
	 * @return bool
	 * @param int $id
	 */
	private static function checkIfValidStartdocument($id, $type = 'document'){

		return ($type === 'object' ?
			(f('SELECT ContentType FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($id)) === we_base_ContentTypes::OBJECT_FILE) :
			(f('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id)) === we_base_ContentTypes::WEDOCUMENT));
	}

//	Here begins the code for showing the correct frameset.
//	To improve readability the different cases are outsourced
//	in several functions, for SEEM, normal or edit_include-Mode.


	private static function _buildJsCommand($cmdArray = ['', '', 'cockpit', 'open_cockpit', '', '', '', '', '']){
		return 'if(WE().layout.weEditorFrameController){
		WE().layout.weEditorFrameController.openDocument(\'' . implode('\', \'', $cmdArray) . '\');
}';
	}

	public static function start(array &$directCmd){
		$cockpitCmd = ['', '', 'cockpit', 'open_cockpit', '', '', '', '', ''];
		$jsCommand = '';
		// check preferences for which document to open at startup
// <we:linkToSeeMode> !
		if(isset($_SESSION['weS']['SEEM']) && isset($_SESSION['weS']['SEEM']['open_selected'])){
			switch($_SESSION['weS']['SEEM']['startType']){
				case 'document':
					if(self::checkIfValidStartdocument($_SESSION['weS']['SEEM']['startId'])){
						$directCmd = [FILE_TABLE,
							$_SESSION['weS']['SEEM']['startId'],
							we_base_ContentTypes::WEDOCUMENT,
						];
					} else {
						t_e('invalid start doc ' . $_SESSION['weS']['SEEM']['startId']);
						$directCmd = $cockpitCmd;
					}
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
				case 'object':
					if(self::checkIfValidStartdocument($_SESSION['weS']['SEEM']['startId'])){
						$directCmd = [OBJECT_FILES_TABLE,
							$_SESSION['weS']['SEEM']['startId'],
							we_base_ContentTypes::OBJECT_FILE
						];
					} else {
						t_e('invalid start doc ' . $_SESSION['weS']['SEEM']['startId']);
						$directCmd = $cockpitCmd;
					}
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
				default:
					$directCmd = $cockpitCmd;
					$jsCommand = self::_buildJsCommand($directCmd);
			}
			unset($_SESSION['weS']['SEEM']['open_selected']);
		} else {// normal mode, start document depends on settings
			switch($_SESSION['prefs']['seem_start_type']){
				default:
				case 'cockpit':
					$directCmd = $cockpitCmd;
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
				case 'object':
					if($_SESSION['prefs']['seem_start_file'] != 0 && self::checkIfValidStartdocument($_SESSION['prefs']['seem_start_file'], 'object')){ //	if a stardocument is already selected - show this
						$directCmd = [
							OBJECT_FILES_TABLE,
							$_SESSION['prefs']['seem_start_file'],
							we_base_ContentTypes::OBJECT_FILE,
						];
					} else {
						t_e('start doc not valid', $_SESSION['prefs']['seem_start_file']);
						$directCmd = $cockpitCmd;
					}
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
				case '0':
					$jsCommand = 'WE().layout.weEditorFrameController.toggleFrames();';
					break;
				case 'document':
					if($_SESSION['prefs']['seem_start_file'] != 0 && self::checkIfValidStartdocument($_SESSION['prefs']['seem_start_file'])){ //	if a stardocument is already selected - show this
						$directCmd = [
							FILE_TABLE,
							$_SESSION['prefs']['seem_start_file'],
							we_base_ContentTypes::WEDOCUMENT,
						];
					} elseif($_SESSION['prefs']['seem_start_file'] != 0){
						t_e('start doc not valid', $_SESSION['prefs']['seem_start_file']);
						$directCmd = $cockpitCmd;
					}
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
				case 'weapp':
					if($_SESSION['prefs']['seem_start_weapp'] != ''){ //	if a we-app is choosen
						$directCmd = ['', '', '', 'tool_' . $_SESSION['prefs']['seem_start_weapp'] . '_edit'];
					}
					$jsCommand = self::_buildJsCommand($directCmd);
					break;
			}
		}

		return $jsCommand;
	}

}
