<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
//TODO in modulesFrames: set module-settings as class properties instead of looping them through as method parameters!

abstract class we_modules_frame{
	var $module;
	var $db;
	var $frameset;
	var $View;
	var $Tree = null;
	protected $showTreeHeader = false;
	protected $showTreeFooter = false;
	protected $treeDefaultWidth = 240;
	protected $treeWidth = 0;
	protected $hasIconbar = false;
	//FIXME: make this protected again
	public $jsCmd = null;

	function __construct($frameset){
		$this->db = new DB_WE();
		$this->frameset = $frameset;
		$this->jsCmd = new we_base_jsCmd();
	}

	public function getHTMLDocumentHeader($charset = ''){
		$charset = ($charset ?: $GLOBALS['we']['Charset']['Backend']);
		we_html_tools::headerCtCharset('text/html', $charset);
		return we_html_tools::getHtmlTop($this->module, $charset);
	}

	function getHTMLDocument($body, $extraHead = ''){
		return $this->getHTMLDocumentHeader() .
			$extraHead .
			$this->jsCmd->getCmds() .
			'</head>' . $body . '</html>';
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'frameset':
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode());
			case 'editor':
				return $this->getHTMLEditor();
			case 'edheader':
				return $this->getHTMLEditorHeader($mode);
			case 'edbody':
				return $this->getHTMLEditorBody();
			case 'edfooter':
				return $this->getHTMLEditorFooter([]);
			case 'cmd':
				return $this->getHTMLCmd();
			case 'tree':
				return $this->getHTMLTree();
			case 'search':
				return $this->getHTMLSearch();
			case 'treeheader':
				return $this->getHTMLTreeHeader();
			default:
				$extraJS = $this->jsCmd->getCmds();
				$ret = ($extraJS ? $this->getHTMLDocument(we_html_element::body(), $extraJS) : '' );
				t_e(__FILE__ . ' unknown reference: ' . $what, ($ret ? 'generated emergency document' : ''));
				return $ret;
		}
	}

	protected function getHTMLFrameset($extraHead = '', $extraUrlParams = ''){
		$this->setTreeWidthFromCookie();

		$extraHead = $this->View->getJSTop() .
			//FIXME: throw some of these functions out again and use generic version of main-window functions
			we_html_element::jsScript(JS_DIR . 'modules_tree.js') .
			we_main_headermenu::css() .
			$extraHead;

		$body = we_html_element::body(['id' => 'weMainBody', 'onload' => 'startTree();'], we_html_element::exIFrame('header', self::getHTMLHeader(
						WE_INCLUDES_PATH . 'menu/we_' . $this->module . '_menu.inc.php')) .
				($this->hasIconbar ? we_html_element::iframe('iconbar', $this->frameset . '&pnt=iconbar' . $extraUrlParams, '', '', '', false) : '') .
				$this->getHTMLResize($extraUrlParams) .
				we_html_element::iframe('cmd', $this->frameset . '&pnt=cmd' . $extraUrlParams)
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	protected static function getHTMLHeader($menuFile){
		$inc = include($menuFile);
		$jmenu = new we_base_menu($inc);
		$menu = $jmenu->getHTML();

		return we_html_element::div(['class' => 'menuDiv'], $menu .
				we_html_element::div(['id' => 'moduleMessageConsole'], we_main_headermenu::createMessageConsole('moduleFrame'))
		);
	}

	private function getHTMLResize($extraUrlParams = ''){
		$incDecTree = '<div id="baumArrows">
	<div class="baumArrow" id="incBaum" ' . ($this->treeWidth <= 30 ? 'style="background-color: grey"' : '') . ' onclick="top.content.incTree(\'' . $this->module . '\');"><i class="fa fa-plus"></i></div>
	<div class="baumArrow" id="decBaum" ' . ($this->treeWidth <= 30 ? 'style="background-color: grey"' : '') . ' onclick="top.content.decTree(\'' . $this->module . '\');"><i class="fa fa-minus"></i></div>
	<div class="baumArrow" onclick="top.content.toggleTree(\'' . $this->module . '\');"><i id="arrowImg" class="fa fa-lg fa-caret-' . ($this->treeWidth <= 30 ? "right" : "left") . '" ></i></div>
</div>';

		$content = we_html_element::div(['id' => 'moduleContent'], we_html_element::div(['id' => 'lframeDiv', 'style' => 'width: ' . $this->treeWidth . 'px;'], we_html_element::div([
						'id' => 'vtabs'], $incDecTree) .
					$this->getHTMLLeft()
				) .
				we_html_element::div(['id' => 'right', 'style' => 'left: ' . $this->treeWidth . 'px;'], we_html_element::iframe('editor', $this->frameset . '&pnt=editor' . $extraUrlParams, ' ', '', '', false)
				)
		);

		return we_html_element::div(['id' => 'resize', 'name' => 'resize', 'class' => ($this->hasIconbar ? 'withIconBar' : ''), 'style' => 'overflow:hidden'], $content);
	}

	protected function getHTMLLeft(){
		//we load tree in iFrame, because the complete tree JS is based on document.open() and document.write()
		//it makes not much sense, to rewrite trees before abandoning them anyway
		return we_html_element::div(['id' => 'left', 'name' => 'left'], we_html_element::div(['id' => 'treeheader', 'style' => ($this->showTreeHeader ? 'display:block;' : '')], $this->getHTMLTreeheader()) .
				$this->getHTMLTree() .
				($this->showTreeFooter ? we_html_element::div(['id' => 'treefooter', 'class' => 'editfooter'], $this->getHTMLTreefooter()) :
				''
				)
		);
	}

	protected function getHTMLTree($extraHead = ''){
		return we_html_element::div([
				'id' => 'tree',
				'class' => ($this->showTreeHeader ? ' withHeader' : '') . ($this->showTreeFooter ? ' withFooter' : '')
				], $extraHead . $this->Tree->getHTMLConstruct()
		);
	}

	protected function getHTMLTreeHeader(){
		return '';
		//to be overridden
	}

	protected function getHTMLTreeFooter(){
		return '';
		//to be overridden
	}

	protected function getHTMLSearchTreeFooter(){
		$hiddens = we_html_element::hiddens([
				'pnt' => 'cmd',
				'cmd' => 'show_search']);

		$table = $hiddens .
			we_html_tools::textInput('keyword', '', '', ['placeholder' => g_l('button', '[search][alt]')], 'text', '150px') .
			we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('show_search')");

		return we_html_element::div(['id' => 'search', 'style' => 'display:block'], we_html_element::form([
					'name' => 'we_form_treefooter',
					'target' => 'cmd',
					'method' => 'post',
					'onsubmit' => "we_cmd('show_search');return false;"
					], $table));
	}

	protected function getHTMLEditor($extraUrlParams = '', $extraHead = ''){
		$sid = we_base_request::_(we_base_request::STRING, 'sid');
		$body = we_html_element::body(['class' => 'moduleEditor'], we_html_element::iframe('edheader', $this->frameset . '&pnt=edheader' . ($sid !== false ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'width: 100%; overflow: hidden', '', false, 'editorHeader') .
				we_html_element::iframe('edbody', $this->frameset . '&pnt=edbody' . ($sid !== false ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'border:0;width:100%;height:100%;', '', true, 'editorBody') .
				we_html_element::iframe('edfooter', $this->frameset . '&pnt=edfooter' . ($sid !== false ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'width: 100%; overflow: hidden', '', false, 'editorButtonFrame')
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	protected function getHTMLEditorHeader($mode = 0){
		return (we_base_request::_(we_base_request::BOOL, 'home') ?
			$this->getHTMLDocument(we_html_element::body(['class' => 'home'], ''), we_html_element::cssLink(CSS_DIR . 'tools_home.css')) :
			'');
	}

	abstract protected function getHTMLEditorBody();

	protected function getHTMLEditorFooter(array $btn_cmd, $extraHead = ''){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return $this->getHTMLDocument(we_html_element::body(['style' => "background-color:#EFF0EF"], ""), $extraHead);
		}

		$table2 = new we_html_table(['class' => 'default'], 1, count($btn_cmd));
		$table2->setRowAttributes(0, ['style' => 'vertical-align:middle']);
		$pos = 0;
		foreach($btn_cmd as $but => $cur){
			list($right, $cmd) = $cur;
			if(empty($right) || we_base_permission::hasPerm($right)){
				$table2->setColContent(0, $pos++, we_html_button::create_button($but, "javascript:top.content.we_cmd('" . (is_array($cmd) ? implode("','", $cmd) : $cmd) . "')"));
			}
		}
		return $this->getHTMLDocument(we_html_element::body(['id' => 'footerBody'], $table2->getHtml()), $extraHead);
	}

	abstract protected function getHTMLCmd();

	function getHTMLSearch(){
		return '';
	}

	private function setTreeWidthFromCookie(){
		if(!empty($_COOKIE['treewidth_modules']) && ($tw = we_unserialize($_COOKIE['treewidth_modules'], [], true))){
			$this->treeWidth = empty($tw[$this->module]) ? $this->treeDefaultWidth : $tw[$this->module];
		} else {
			$this->treeWidth = $this->treeDefaultWidth;
		}
	}

	protected function formFileChooser($IDName = 'ParentID', $IDValue = '/', $cmd = '', $filter = ''){
		return we_html_tools::formElementTable(we_html_tools::textInput($IDName, $IDValue, '', ['readonly' => 'readonly'], 'text', 400, 0), '', 'left', 'defaultfont', '', (we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','" . $IDName . "','" . $filter . "',document.we_form.elements['" . $IDName . "'].value);") : ''));
	}

	/* process vars & commands
	 */

	public function process(){
		ob_start();
		$this->View->processVariables();
		$this->View->processCommands($this->jsCmd);
		//FIXME:remove
		$extraJS = ob_get_clean();
		if(!empty($extraJS)){
			t_e('bad js will not be processed', $extraJS);
		}
	}

	public static function showNotActivatedMsg($modName){
		$title = sprintf(g_l('moduleActivation', '[headline]'), $modName);
		echo we_html_tools::getHtmlTop($title, '', '', '', we_html_element::body([
				'class' => 'weDialogBody',
				'onload' => 'self.focus();',
				'onblur' => 'self.close();'
				], '<table style="width:100%" class="default defaultfont">
<tr><td colspan="2"><strong>' . $title . '</strong></td></tr>
<tr><td style="vertical-align:top"><span class="fa-stack fa-lg" style="color:#F2F200;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span></td><td>' . g_l('moduleActivation', '[content]') . '</td></tr>
</table>
'));
	}

}
