<?php

/**
 * webEdition CMS
 *
 * $Rev: 14456 $
 * $Author: mokraemer $
 * $Date: 2017-10-19 13:55:30 +0200 (Thu, 19 Oct 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */
class we_category_view extends we_modules_view{

	function __construct(){
		parent::__construct();
		$this->Model = new we_category_category();
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::CATEGORY, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_prop.js');
	}

	private function renameChildrenPath($id){
		$db = $this->db;
		$parentPath = f('SELECT Path FROM ' . CATEGORY_TABLE . ' WHERE ID=' . intval($id));
		$db->query('UPDATE ' . CATEGORY_TABLE . ' SET Path=CONCAT("' . $db->escape($parentPath) . '","/",Text) WHERE ParentID=' . intval($id));
		$db->query('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id)); //IsFolder=1 AND
		$updates = $db->getAll(true);
		foreach($updates as $id){
			$this->renameChildrenPath($id);
		}
	}

	private function CatInUse($id){
		if(f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE FIND_IN_SET(' . intval($id) . ',Category) OR FIND_IN_SET(' . intval($id) . ',temp_category) LIMIT 1', '', $this->db) ||
			(we_modules_info::isActive(we_modules_info::OBJECT) && f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE FIND_IN_SET(' . intval($id) . ',Category) LIMIT 1', '', $this->db))){
			return true;
		}
		return $this->DirInUse($id);
	}

	private function DirInUse($id){
		if($this->CatInUse($id)){
			return true;
		}

		$ids = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		while($ids){
			if($this->CatInUse($ids)){
				return true;
			}
		}
		return false;
	}

	private function delDir($id){
		$entries = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		foreach($entries as $entry){
			$this->delDir($entry);
		}
		$entries = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		$entries[] = $id;
		$this->delEntries($entries);
	}

	private function delEntries(array $ids){
		$this->db->query('DELETE FROM ' . CATEGORY_TABLE . ' WHERE ID IN (' . implode(',', $ids) . ')');
	}

	function processCommands(we_base_jsCmd $jsCmd){
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case 'we_save':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(empty($this->Model->Text)){
					$jsCmd->addMsg(g_l('weEditor', '[category][filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(strpos(',', $this->Model->Text) !== false){
					$jsCmd->addMsg(g_l('weEditor', '[category][name_komma]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(f('SELECT 1 FROM ' . CATEGORY_TABLE . ' WHERE ID!=' . intval($this->Model->ID) . ' AND ParentID=' . $this->Model->ParentID . ' AND Text="' . $this->db->escape($this->Model->Text) . '" LIMIT 1', '', $this->db)){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][response_path_exists]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif(preg_match('|[\\\'"<>/]|', $this->Model->Text)){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][we_filename_notValid]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->save()){
					$jsCmd->addMsg(sprintf(g_l('thumbnails', '[saved_successfully]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					if($this->Model->wasUpdate){
						$this->renameChildrenPath($this->Model->ID);
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						//update parent
						if($this->Model->ParentID){
							$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ParentID, 'typ' => we_tree_base::TYP_FOLDER, 'contenttype' => 'we/categories']);
						}
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => 'we/category',
							'table' => CATEGORY_TABLE, 'published' => 1]);
					}
				} else {
					echo "ERROR";
				}
				break;
			case "delete_category":
				$catsToDel = [$this->Model->ID];
				$finalDelete = [];
				$catlistNotDeleted = '';
				foreach($catsToDel as $id){
					if($this->CatInUse($id)){
						$catlistNotDeleted .= id_to_path($id, CATEGORY_TABLE, $this->db) . '\n';
					} else {
						$finalDelete[] = $id;
					}
				}
				if($finalDelete){
					foreach($finalDelete as $id){
						$this->delDir($id);
					}
				}
				if($catlistNotDeleted){
					$jsCmd->addMsg(g_l('fileselector', '[cat_in_use]') . '\n\n' . $catlistNotDeleted, we_base_util::WE_MESSAGE_ERROR);
				}
				$_SESSION['weS']['category_session'] = [];
				$this->Model = new we_category_category();
				break;
			default:
				$id = (we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1) ?: 0);

				if($id){
					$this->Model->load($id);
				}
		}
		$this->Model->saveInSession($_SESSION['weS']['category_session']);
	}

	function processVariables(){
		if(!empty($_SESSION['weS']['category_session'])){
			$this->Model->we_initSessDat($_SESSION['weS']['category_session']);

			$this->Model->Text = we_base_request::_(we_base_request::STRING, 'Text');
			$this->Model->ParentID = we_base_request::_(we_base_request::INT, 'FolderID');
			$this->Model->Path = ($this->Model->ParentID ? f('SELECT Path FROM ' . CATEGORY_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '/') . $this->Model->Text;

			$this->Model->Title = we_base_request::_(we_base_request::STRING, 'catTitle');
			$this->Model->Description = we_base_request::_(we_base_request::STRING, 'catDescription');
		} else {
			$_SESSION['weS']['category_session'] = [];
		}
	}

	public function formName(){
		$path = f('SELECT cc.Path FROM ' . CATEGORY_TABLE . ' c LEFT JOIN ' . CATEGORY_TABLE . ' cc ON c.ParentID=cc.ID WHERE c.ID=' . intval($this->Model->ID)) ?: '/';

		$weSuggest = &we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc');
		$weSuggest->setTable(CATEGORY_TABLE);
		$weSuggest->setContentType('');
		$weSuggest->setInput('FolderIDPath', $path);
		$weSuggest->setMaxResults(20);
		$weSuggest->setRequired(true);
		$weSuggest->setResult('FolderID', $this->Model->ParentID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setWidth(250);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file', document.we_form.elements.FolderID.value, '" . CATEGORY_TABLE . "', 'FolderID', 'FolderIDPath', '', '', '', '1', '', 'false', 1)"));

		$table = new we_html_table(['class' => 'default'], 2, 2);
		$table->setColContent(0, 0, '<b>' . g_l('weClass', '[category]') . '</b>');
		$table->setColContent(0, 1, we_html_tools::textInput('Text', $this->Model->Text, '', ['id' => "category"], 'text', 360));
		$table->setColContent(1, 0, '<b>' . g_l('weClass', '[dir]') . '</b>');
		$table->setColContent(1, 1, $weSuggest->getHTML());

		return $table->getHtml();
	}

	public function formSettings(){
		$table = new we_html_table(['class' => 'default'], 2, 2);

		$table->setColContent(0, 0, '<b>' . g_l('global', '[title]') . '</b>');
		$table->setColContent(0, 1, we_html_tools::textInput("catTitle", $this->Model->Title, '', [], 'text', 360));
		$table->setColContent(1, 0, '<b>' . g_l('global', '[description]') . '</b>');
		$table->setColContent(1, 1, we_html_forms::weTextarea('catDescription', $this->Model->Description, [
				"bgcolor" => "white",
				"inlineedit" => "true",
				"wysiwyg" => "true",
				'width' => 450,
				'height' => 130,
				'commands' => 'prop,fontsize,xhtmlxtras,color,justify,list,link,table,insert,fullscreen,visibleborders,editsource'
				], true, 'autobr', true, true, true, true, true, $GLOBALS['we']['Charset']['Backend']));
		return $table->getHtml();
	}

	public function getHomeScreen(){
		$hiddens = ['cmd' => 'home', 'pnt' => 'edbody'];

		$content = we_html_button::create_button('fat:add,fa-lg fa-plus', "javascript:we_cmd('add_category')");

		return parent::getActualHomeScreen(we_modules_info::CATEGORY, $content, we_html_element::form(['name' => 'we_form'], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', 0)));
	}

}
