<?php
/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 *  view class for document customer filters
 *
 */
class we_customer_documentFilterView extends we_customer_filterView{

	public function __construct(&$filter, $width = 0){
		parent::__construct($filter, $width);
		$this->viewType = 'document';
	}

	/**
	 * Gets the HTML and Javascript for the filter
	 *
	 * @return string
	 */
	function getFilterHTML($ShowModeNone = false){
		return parent::getFilterHTML() .
			we_html_multiIconBox::getHTML('', array_merge(
					$this->getAccessControlHTML(), ($GLOBALS['we_doc']->ContentType === we_base_ContentTypes::FOLDER ? $this->getFolderApplyHTML() : [])));
	}

	/**
	 * Gets the HTML and Javascript for the access control ui
	 *
	 * @return array
	 */
	private function getAccessControlHTML(){
		$filter = $this->getFilter();

		$weSuggest = & we_gui_suggest::getInstance();

		/*		 * ** AUTOSELECTOR FOR ErrorDocument, Customer is not logged in *** */
		$id_selectorNoLoginId = $filter->getErrorDocNoLogin();
		$path_selectorNoLoginId = $id_selectorNoLoginId ? id_to_path($id_selectorNoLoginId) : "";
		if(!$path_selectorNoLoginId){
			$id_selectorNoLoginId = "";
		}

		$selectorNoLoginId = "wecf_noLoginId";
		$selectorNoLoginText = "wecf_InputNoLoginText";
		//$selectorNoLoginError = "wecf_ErrorMarkNoLoginText";

		$weSuggest->setAcId("NoLogin");
		$weSuggest->setContentType("folder," . we_base_ContentTypes::WEDOCUMENT);
		$weSuggest->setInput($selectorNoLoginText, $path_selectorNoLoginId);
		$weSuggest->setLabel(g_l('modules_customerFilter', '[documentNoLogin]'));
		$weSuggest->setMaxResults(20);
		$weSuggest->setResult($selectorNoLoginId, $id_selectorNoLoginId);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setWidth(409);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $selectorNoLoginId . "'].value,'" . FILE_TABLE . "','" . $selectorNoLoginId . "','" . $selectorNoLoginText . "','setHot','','','" . we_base_ContentTypes::WEDOCUMENT . "',1)") . "<div id=\"wecf_container_noLoginId\"></div>");

		$weAcSelector = $weSuggest->getHTML();

		/*		 * ** AUTOSELECTOR FOR ErrorDocument, Customer might be logged in, but has no access *** */
		$id_selectorNoAccessId = $filter->getErrorDocNoAccess();
		$path_selectorNoAccessId = $id_selectorNoAccessId ? id_to_path($id_selectorNoAccessId) : "";
		if(!$path_selectorNoAccessId){
			$id_selectorNoAccessId = "";
		}

		$selectorNoAccessId = "wecf_noAccessId";
		$selectorNoAccessText = "wecf_InputNoAccessText";
		//$selectorNoAccessError = "wecf_ErrorMarkNoAccessText";

		$weSuggest->setAcId("NoAccess");
		$weSuggest->setContentType("folder," . we_base_ContentTypes::WEDOCUMENT);
		$weSuggest->setInput($selectorNoAccessText, $path_selectorNoAccessId);
		$weSuggest->setLabel(g_l('modules_customerFilter', '[documentNoAccess]'));
		$weSuggest->setMaxResults(20);
		$weSuggest->setResult($selectorNoAccessId, $id_selectorNoAccessId);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setWidth(409);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $selectorNoAccessId . "'].value,'" . FILE_TABLE . "','" . $selectorNoAccessId . "','" . $selectorNoAccessText . "','setHot','','','" . we_base_ContentTypes::WEDOCUMENT . "',1)"));

		$weAcSelector2 = $weSuggest->getHTML();

		$accesControl = we_html_multiIconBox::getHTML('', [['headline' => g_l('modules_customerFilter', '[accessControl]'),
				'html' => we_html_forms::radiobutton(
					"onTemplate", $filter->getAccessControlOnTemplate(), "wecf_accessControlOnTemplate", g_l('modules_customerFilter', '[accessControlOnTemplate]'), true, "defaultfont", "updateView_document();top.content.setHot();") .
				we_html_forms::radiobutton(
					"errorDoc", !$filter->getAccessControlOnTemplate(), "wecf_accessControlOnTemplate", g_l('modules_customerFilter', '[accessControlOnErrorDoc]'), true, "defaultfont", "updateView_document();top.content.setHot();") .
				we_customer_documentFilterView::getDiv($weAcSelector . $weAcSelector2, 'accessControlSelectorDiv', (!$filter->getAccessControlOnTemplate()), 25),
				'noline' => true
		]]);


		return [[
			'html' => we_html_element::div([
				'id' => 'accessControlDiv',
				'style' => 'display:' . ($filter->getMode() === we_customer_abstractFilter::OFF ? 'none' : 'block' ) . ';'
				], $accesControl),
			'noline' => true
		]];
	}

	/**
	 * Gets the HTML and Javascript for the folder apply ui (copy filter)
	 *
	 * @return array
	 */
	private function getFolderApplyHTML(){
		return [
			['headline' => g_l('modules_customerFilter', '[apply_filter]') . we_html_tools::alertAttentionBox(g_l('modules_customerFilter', '[apply_filter_info]'), we_html_tools::TYPE_HELP, 432, false),
				'html' => we_html_button::create_button(we_html_button::OK, "javascript:if (_EditorFrame.getEditorIsHot()) {WE().util.showMessage(WE().consts.g_l.alert.apply_filter_isHot,WE().consts.message.WE_MESSAGE_ERROR, window);} else { we_cmd('copyWeDocumentCustomerFilter', '" . $GLOBALS['we_doc']->ID . "', '" . $GLOBALS['we_doc']->Table . "');}"),
//				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'forceRightHeadline' => true
			],
		];
	}

}
