<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_tree extends we_tree_base{

	protected function customJSFile(){
		return we_html_element::jsScript(JS_DIR . 'navigation_tree.js', 'initTree();');
	}

	public static function getItems($ParentID, $offset = 0, $segment = 500, $sort = false){
		$db = new DB_WE();

		$items = $aWsQuery = $parentpaths = [];

		if(($ws = get_ws(NAVIGATION_TABLE, true))){
			$wsPathArray = id_to_path($ws, NAVIGATION_TABLE, $db, true);
			foreach($wsPathArray as $path){
				$aWsQuery[] = ' Path LIKE "' . $db->escape($path) . '/%" OR ' . we_tree_base::getParents($path);
				while($path != "/" && $path != "\\" && $path){
					$parentpaths[] = $path;
					$path = dirname($path);
				}
			}
		}

		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => NAVIGATION_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'published' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $prevoffset,
			];
		}

		$where = ($aWsQuery ? '(' . implode(' OR ', $aWsQuery) . ') AND ' : '') . ' ParentID=' . intval($ParentID) . ' ';

		$db->query('SELECT ID,ParentID,Path,Text,IsFolder,Ordn,Depended,Charset,abs(text) as Nr, (text REGEXP "^[0-9]") AS isNr FROM ' . NAVIGATION_TABLE . ' WHERE ' . $where . ' ORDER BY Ordn, isNr DESC,Nr,Text ' . ($segment ? 'LIMIT ' . $offset . ',' . $segment : ''));

		while($db->next_record(MYSQLI_ASSOC)){
			$typ = [
				'typ' => ($db->f('IsFolder') ? self::TYP_FOLDER : self::TYP_ITEM),
				'open' => 0,
				'tooltip' => intval($db->f('ID')),
				'offset' => $offset,
				'order' => intval($db->f('Ordn')),
				'published' => $db->f('Depended') ? 0 : 1,
				'disabled' => we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? 0 : ($db->f('IsFolder') && !$db->f('ParentID') || in_array($db->f('Path'), $parentpaths) ? 1 : 0),
				'contenttype' => ($db->f('IsFolder') ? we_base_ContentTypes::FOLDER : 'we/navigation'),
			];

			$fileds = array_change_key_case($db->getRecord());

			$charset = ($db->f('IsFolder') == 0 ?
				we_navigation_navigation::findCharset($db->f('ParentID')) :
				$db->f('Charset'));

			$text = strip_tags(strtr($db->f('Text'), ['&amp;' => '&', '<br/>' => ' ']));
			$path = str_replace('&amp;', '&', $db->f('Path'));
			$convert = !empty($charset) && function_exists('mb_convert_encoding');
			$typ['text'] = $convert ? mb_convert_encoding($text, 'HTML-ENTITIES', $charset) : $text;
			$typ['path'] = $convert ? mb_convert_encoding($path, 'HTML-ENTITIES', $charset) : $path;

			$items[] = array_merge($fileds, $typ);
		}

		$total = f('SELECT COUNT(1) FROM ' . NAVIGATION_TABLE . ' WHERE ' . $where, '', $db);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = ['id' => 'next_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => NAVIGATION_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

}
