<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_shop_revenueTop{
	private static $selectedYear;
	private static $selectedMonth;
	private static $orderBy;
	private static $orderDesc;
	private static $actPage;

	private static function getTitleLink($text, $orderKey){
		$href = WEBEDITION_DIR . 'we_showMod.php?mod=shop&pnt=edit_shop_revenueTop' .
			'&ViewYear=' . self::$selectedYear .
			'&ViewMonth=' . self::$selectedMonth .
			'&actPage=' . self::$actPage .
			'&orderBy=' . $orderKey .
			( (self::$orderBy == $orderKey && !self::$orderDesc) ? '&orderDesc=true' : '' );

		return '<span onclick="document.location=\'' . $href . '\';">' . $text . '</span>' . (self::$orderBy == $orderKey ? ' <i class="fa fa-sort-' . (self::$orderDesc ? 'desc' : 'asc') . ' fa-lg"></i>' : '<i class="fa fa-sort fa-lg"></i>');
	}

	private static function getPagerLink(){
		return WEBEDITION_DIR . 'we_showMod.php?mod=shop&pnt=edit_shop_revenueTop' .
			'&ViewYear=' . self::$selectedYear .
			'&ViewMonth=' . self::$selectedMonth .
			'&orderBy=' . self::$orderBy .
			(self::$orderDesc ? '&orderDesc=true' : '' );
	}

	private static function yearSelect($select_name){
		$yearStart = 2001;
		$yearNow = date('Y');
		$opts = [];

		while($yearNow > $yearStart){
			$opts[$yearNow] = $yearNow;
			$yearNow--;
		}
		return we_html_tools::select($select_name, $opts, 1, we_base_request::_(we_base_request::INT, $select_name, ''), false, ['id' => $select_name]);
	}

	private static function monthSelect($select_name, $selectedMonth){
		$opts = g_l('modules_shop', '[month]');
		$opts[0] = '-';
		ksort($opts, SORT_NUMERIC);
		return we_html_tools::select($select_name, $opts, 1, $selectedMonth, false, ['id' => $select_name]);
	}

	public static function getDialog(){
		self::$selectedYear = we_base_request::_(we_base_request::INT, 'ViewYear', date('Y'));
		self::$selectedMonth = we_base_request::_(we_base_request::INT, 'ViewMonth', 1);
		self::$orderBy = we_base_request::_(we_base_request::STRING, 'orderBy', 'ID');
		self::$orderDesc = we_base_request::_(we_base_request::BOOL, 'orderDesc');
		self::$actPage = we_base_request::_(we_base_request::INT, 'actPage', 0);
		$DB_WE = $GLOBALS['DB_WE'];

// get some preferences!
		$feldnamen = explode('|', f('SELECT pref_value from ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		$waehr = "&nbsp;" . oldHtmlspecialchars($feldnamen[0]);
		/* 		$numberformat = $feldnamen[2];
		  $classid = (isset($feldnamen[3]) ? $feldnamen[3] : ''); */
		$defaultVat = $feldnamen[1] ?: 0;

		$nrOfPage = isset($feldnamen[4]) ? $feldnamen[4] : 20;
		if($nrOfPage === "default"){
			$nrOfPage = 20;
		}

		$parts = [
			[
				//'headline' => '<label for="ViewYear">' . g_l('modules_shop', '[selectYear]') . '</label>',
				'html' => self::yearSelect("ViewYear") . self::monthSelect("ViewMonth", self::$selectedMonth) . we_html_button::create_button(we_html_button::VIEW, "javascript:we_submitDateform();"),
				'space' => we_html_multiIconBox::SPACE_SMALL,
				'noline' => true
			],
		];

		$to = (self::$selectedMonth ?
			self::$selectedYear . '-' . (self::$selectedMonth + 1) :
			(self::$selectedYear + 1) . '-01');

		$queryShopDateCondtion = '(o.DateOrder BETWEEN "' . self::$selectedYear . '-' . (self::$selectedMonth ?: '01') . '-01" AND "' . $to . '-01")';

		if(($maxRows = f('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion, '', $DB_WE))){


			$amountOrders = f('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion, '', $DB_WE);
			$editedOrders = f('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion . ' AND (!ISNULL(o.DateShipping) OR !ISNULL(o.DateFinished) )', '', $DB_WE);

			$query = 'SELECT
SUM((oi.Price*oi.quantity*IF(o.pricesNet && o.calcVat && COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . '),(1+COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')/100),1))) AS brutto,
SUM((oi.Price*oi.quantity*IF(o.pricesNet,1,(1/(1+COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')/100))))) AS netto
FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion;


			$payed = getHash($query . ' AND o.DatePayment IS NOT NULL AND o.DateCancellation IS NULL');
			//$canceled = f($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NOT NULL');
			$unpayed = getHash($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NULL');

			$articleVatArray = $DB_WE->getAllFirstq('SELECT oi.Vat, SUM((IF(o.pricesNet,oi.Price*oi.quantity*(COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')/100), (oi.Price*oi.quantity)-( (oi.Price*oi.quantity)/(1+ (COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')/100) )) ))) FROM tblOrder o JOIN tblOrderItem oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion . ' AND o.DateCancellation IS NULL AND o.calcVat AND COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')>0 GROUP BY oi.Vat', false);


			// generate vat table
			if(!empty($articleVatArray)){
				$vatTable = '
			<tr>
				<td colspan="7" class="shopContentfontR" style="padding-top:10px;">' . g_l('modules_shop', '[plusVat]') . ':</td>
			</tr>';
				foreach($articleVatArray as $vat => $amount){
					$vatTable .= '
				<tr>
					<td colspan="5"></td>
					<td class="shopContentfontR">' . $vat . '&nbsp;%</td>
					<td class="shopContentfontR">' . we_base_util::formatNumber($amount) . $waehr . '</td>
				</tr>';
				}
			} else {
				$vatTable = '';
			}

			$parts[] = [
				'html' => '
			<table class="defaultfont" style="width:680px;">
			<tr>
				<th>' . g_l('modules_shop', '[anual]') . '</th>
				<th>' . (self::$selectedMonth ? g_l('modules_shop', '[monat]') : '' ) . '</th>
				<th>' . g_l('modules_shop', '[anzahl]') . '</th>
				<th>' . g_l('modules_shop', '[unbearb]') . '</th>
				<th>' . g_l('modules_shop', '[schonbezahlt]') . '</th>
				<th>' . g_l('modules_shop', '[unbezahlt]') . '</th>
				<th class="shopContentfontR">' . g_l('modules_shop', '[umsatzgesamt]') . '</th>
			</tr>' . '
			<tr class="shopContentfont">
				<td>' . self::$selectedYear . '</td>
				<td>' . (self::$selectedMonth > 0 ? self::$selectedMonth : '' ) . '</td>
				<td>' . $amountOrders . '</td>
				<td class="defaultfont shopNotPayed">' . ($amountOrders - $editedOrders) . '</td>
				<td>' . we_base_util::formatNumber($payed['netto']) . $waehr . '</td>
				<td class="defaultfont shopNotPayed">' . we_base_util::formatNumber($unpayed['netto']) . $waehr . '</td>
				<td class="shopContentfontR">' . we_base_util::formatNumber($payed['netto'] + $unpayed['netto']) . $waehr . '</td>
			</tr>' .
				$vatTable .
				'<tr><td colspan="7"><hr style="color:black"/></td></tr>' .
				'<tr style="border-top:2px solid black" class="shopContentfont">
				<td colspan="4"></td>
				<td>' . we_base_util::formatNumber($payed['brutto']) . $waehr . '</td>
				<td class="defaultfont shopNotPayed">' . we_base_util::formatNumber($unpayed['brutto']) . $waehr . '</td>
				<td class="shopContentfontR">' . we_base_util::formatNumber($payed['brutto'] + $unpayed['brutto']) . $waehr . '</td>
			</tr>' .
				'</table>',
			];

			$headline = [
				self::getTitleLink(g_l('modules_shop', '[bestellung]'), 'o.ID'),
				g_l('modules_shop', '[ArtName]'), // 'shoptitle'
				g_l('modules_shop', '[anzahl]'),
				self::getTitleLink(g_l('modules_shop', '[artPrice]'), 'Price'),
				g_l('modules_shop', '[Gesamt]'),
				self::getTitleLink(g_l('modules_shop', '[artOrdD]'), 'DateOrder'),
				self::getTitleLink(g_l('modules_shop', '[ArtID]'), 'orderDocID'),
				self::getTitleLink(g_l('modules_shop', '[artPay]'), 'DatePayment'),
			];
			$content = [];

			$DB_WE->query('SELECT o.ID,
	@price:=(oi.Price*IF(o.pricesNet && o.calcVat && COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . '),(1+COALESCE(oi.Vat,' . (isset($defaultVat) ? $defaultVat : 0) . ')/100),1)) AS priceToShow,
	(oi.quantity*@price) AS articleSum,
	oi.quantity,
	oi.customFields,
	od.DocID,
	od.title,
	od.variant,
	DATE_FORMAT(DateOrder, "%d.%m.%Y") AS formatDateOrder,
	DATE_FORMAT(DatePayment, "%d.%m.%Y") AS formatDatePayment,
	DateCancellation IS NOT NULL AS isCancelled
FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID JOIN ' . SHOP_ORDER_DOCUMENT_TABLE . ' od ON oi.orderDocID=od.ID
WHERE ' .
				$queryShopDateCondtion . '
ORDER BY ' . we_base_request::_(we_base_request::STRING, 'orderBy', 'o.ID') . (self::$orderDesc ? ' DESC' : '') . ' LIMIT ' . (self::$actPage * $nrOfPage) . ',' . $nrOfPage);

			while($DB_WE->next_record(MYSQLI_ASSOC)){
				$hash = $DB_WE->getRecord();

				$variantStr = ($hash['variant'] ? '<br/><strong>' . g_l('modules_shop', '[variant]') . ': ' . $hash['variant'] . '</strong>' : '');

				if($hash['customFields']){
					$cf = we_unserialize($hash['customFields']);
					$customFields = we_html_element::br();
					foreach($cf as $key => $val){
						$customFields .= $key . '=' . $val . we_html_element::br();
					}
				} else {
					$customFields = '';
				}

				$content[] = [
					['dat' => '<a href="javascript:we_cmd(\'openOrder\',' . $hash['ID'] . ',\'shop\',\'' . SHOP_ORDER_TABLE . '\');">' . $hash['ID'] . '</a>'],
					['dat' => $hash['title'] . '<span class="small">' . $variantStr . ' ' . $customFields . '</span>'],
					['dat' => $hash['quantity']],
					['dat' => we_base_util::formatNumber($hash['priceToShow']) . $waehr],
					['dat' => we_base_util::formatNumber($hash['articleSum']) . $waehr],
					['dat' => $hash['formatDateOrder']],
					['dat' => $hash['DocID']],
					['dat' => ($hash['formatDatePayment'] ?: ( $hash['isCancelled'] ? '<span class="defaultfont shopNotPayed">' . g_l('modules_shop', '[artCanceled]') . '</span>' : '<span class="defaultfont shopNotPayed">' . g_l('modules_shop', '[artNPay]') . '</span>'))],
				];
			}


			$parts[] = ['html' => we_html_tools::dialogBorder(670, $content, $headline),
				'noline' => true
			];

			$parts[] = ['html' => we_shop_pager::getStandardPagerHTML(self::getPagerLink(), self::$actPage, $nrOfPage, $maxRows),
			];
		} else {
			$parts[] = ['html' => g_l('modules_shop', '[NoRevenue]') . ' (' . (self::$selectedMonth ? g_l('modules_shop', '[month][' . self::$selectedMonth . ']') . ' ' : '') . self::$selectedYear . ')',
			];
		}

		return we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/edit_shop_revenueTop.js'), we_html_element::body(['class' => "weEditorBody",
					'onload' => "self.focus(); setHeaderTitle('" . (self::$selectedMonth ? g_l('modules_shop', '[month][' . self::$selectedMonth . ']') . ' ' : '') . self::$selectedYear . "');"], '<form action="' . WEBEDITION_DIR . 'we_showMod.php?mod=shop&pnt=edit_shop_revenueTop" method="post">' . we_html_multiIconBox::getHTML('revenues', $parts, 30, '', -1, '', '', false, sprintf(g_l('tabs', '[module][revenueTotal]'), self::$selectedYear)) . '</form>'));
	}

}
