<?php

/**
 * webEdition CMS
 *
 * $Rev: 14287 $
 * $Author: mokraemer $
 * $Date: 2017-09-28 14:04:27 +0200 (Thu, 28 Sep 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */
class we_thumbnail_view extends we_modules_view{

	function __construct(){
		parent::__construct();
		$this->Model = new we_thumbnail_thumbnail();
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::THUMB, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/thumbnail/thumbnail_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/thumbnail/thumbnail_prop.js');
	}

	function processCommands(we_base_jsCmd $jsCmd){
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case 'we_save':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model->Path = ($this->Model->ParentID ? f('SELECT Path FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '/') . $this->Model->Text;

				if(preg_match('|[\'",]|', $this->Model->Text)){
					$jsCmd->addMsg(g_l('alert', '[thumbnail_hochkomma]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(!$this->Model->Text){
					$jsCmd->addMsg(g_l('alert', '[thumbnail_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(f('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE Text="' . $this->db->escape($this->Model->Text) . '" AND ID!=' . intval($this->Model->ID) . ' LIMIT 1')){
					$jsCmd->addMsg(sprintf(g_l('alert', '[thumbnail_exists]'), $this->Model->Text), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->we_save()){
					$jsCmd->addMsg(sprintf(g_l('thumbnails', '[saved_successfully]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					if($this->Model->wasUpdate){
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => 'we/thumbnail',
							'table' => THUMBNAILS_TABLE, 'published' => 1]);
					}
				} else {
					echo "ERROR";
				}
				break;
			case 'delete_thumbnail':
// Check if we need to delete a thumbnail
				if(($delId = $this->Model->ID) && we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					// Delete thumbnails in filesystem
					we_contents_thumbnail::deleteByThumbID($delId);

					// Delete entry in database
					$this->db->query('DELETE FROM ' . THUMBNAILS_TABLE . ' WHERE ID=' . $delId);
					$jsCmd->addMsg(sprintf(g_l('thumbnail', '[deleted]'), $this->Model->Text), we_base_util::WE_MESSAGE_INFO);
				} else {
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
				}
				$_SESSION['weS']['thumbnail_session'] = [];
				$this->Model = new we_thumbnail_thumbnail();

				break;
			case 'add_thumbnail'://new
				$this->Model = new we_thumbnail_thumbnail();
				break;
			default:
				$id = (we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1) ?: 0);

				if($id){
					$this->Model->initByID($id, THUMBNAILS_TABLE);
				}
		}
		$this->Model->saveInSession($_SESSION['weS']['thumbnail_session']);
	}

	function processVariables(){
		if(!empty($_SESSION['weS']['thumbnail_session'])){
			$this->Model->we_initSessDat($_SESSION['weS']['thumbnail_session']);

			$this->Model->Text = we_base_request::_(we_base_request::STRING, 'Text');
			$this->Model->description = we_base_request::_(we_base_request::STRING, 'description');
			$this->Model->Quality = we_base_request::_(we_base_request::INT, 'thumbnail_quality');
			$this->Model->Height = we_base_request::_(we_base_request::INT, 'thumbnail_height');
			$this->Model->Width = we_base_request::_(we_base_request::INT, 'thumbnail_width');

			$this->Model->Format = we_base_request::_(we_base_request::INT, 'Format');
			$this->Model->Options = implode(',', array_filter(we_base_request::_(we_base_request::STRING, 'Options', []), function($v){
					return $v !== we_contents_thumbnail::OPTION_DEFAULT;
				}));
		} else {
			$_SESSION['weS']['thumbnail_session'] = [];
		}
	}

	public function getHomeScreen(){
		$hiddens = ['cmd' => 'home', 'pnt' => 'edbody'];

		$content = we_html_button::create_button('fat:add,fa-lg fa-plus', "javascript:we_cmd('add_thumbnail')");

		return parent::getActualHomeScreen(we_modules_info::THUMB, $content, we_html_element::form(['name' => 'we_form'], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', 0)));
	}

}
