<?php

/**
 * webEdition CMS
 *
 * $Rev: 14628 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 12:39:29 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_users_user{

	const TYPE_USER = 'user';
	const TYPE_USER_GROUP = 'group';
	const TYPE_ALIAS = 'alias';
	const INVALID_CREDENTIALS = 1;
	const MAX_LOGIN_COUNT_REACHED = 2;
	const ERR_USER_PATH_NOK = -5;
	const SALT_NONE = 0;
	const SALT_MD5 = 1;
	const SALT_CRYPT = 2;
	const TAB_DATA = 0;
	const TAB_PERMISSION = 1;
	const TAB_WORKSPACES = 2;
	const TAB_SETTINGS = 3;
	const DEFAULT_PASS_REGEX = '(.{6,20})';
	const LOGIN_DENIED = 4;
	const LOGIN_OK = 2;
	const LOGIN_DENIED_SECURITY = 3;
	const LOGIN_CREDENTIALS_INVALID = 1;
	const LOGIN_UNKNOWN = 0;

	// Name of the class => important for reconstructing the class from outside the class
	var $ClassName = __CLASS__;
	// In this array are all storagable class variables
	public $persistent_slots = [
		'ID' => we_base_request::INT,
		'Type' => we_base_request::INT,
		'ParentID' => we_base_request::INT,
		'Salutation' => we_base_request::STRING,
		'First' => we_base_request::STRING,
		'Second' => we_base_request::STRING,
		'Address' => we_base_request::STRING,
		'HouseNo' => we_base_request::STRING, //e.g. 2a
		'City' => we_base_request::STRING,
		'PLZ' => we_base_request::STRING, //leading 0
		'State' => we_base_request::STRING,
		'Country' => we_base_request::STRING,
		'Telephone' => we_base_request::STRING,
		'Fax' => we_base_request::STRING,
		'Handy' => we_base_request::STRING,
		'Email' => we_base_request::EMAIL,
		'Company' => we_base_request::STRING,
		'Department' => we_base_request::STRING,
		'username' => we_base_request::STRING,
		'passwd' => we_base_request::STRING,
		'clearpasswd' => we_base_request::STRING,
		'Path' => we_base_request::FILE,
		'Permissions' => we_base_request::RAW,
		'ParentPerms' => we_base_request::BOOL,
		'Description' => we_base_request::STRING,
		'Alias' => we_base_request::STRING,
		'IsFolder' => we_base_request::INT,
		'CreatorID' => we_base_request::INT,
		'CreateDate' => we_base_request::INT,
		'ModifierID' => we_base_request::INT,
		'ModifyDate' => we_base_request::INT,
		'Ping' => we_base_request::INT,
		'workSpace' => we_base_request::INTLIST,
		//'workSpaceDef' => we_base_request::INTLIST,
		'workSpaceTmp' => we_base_request::INTLIST,
		'workSpaceNav' => we_base_request::INTLIST,
		'workSpaceNwl' => we_base_request::INTLIST,
		'workSpaceObj' => we_base_request::INTLIST,
		'workSpaceCust' => we_base_request::INTLIST,
		'ParentWs' => we_base_request::BOOL,
		'ParentWst' => we_base_request::BOOL,
		'ParentWsn' => we_base_request::BOOL,
		'ParentWso' => we_base_request::BOOL,
		'ParentWsnl' => we_base_request::BOOL,
		'ParentWsCust' => we_base_request::BOOL,
		'altID' => we_base_request::INT,
		'LoginDenied' => we_base_request::BOOL,
	];
	// Name of the Object that was createt from this class
	var $Name = '';
	// ID from the database record
	var $ID = 0;
	// database table in which the object is stored
	var $Table = USER_TABLE;
	// Database Object
	var $DB_WE;
	// Parent identificator
	var $ParentID = 0;
	// Flag which indicates which kind of user is 0-user;1-group;2-owner group;3 - alias
	var $Type = self::TYPE_USER;
	// Flag which indicates if user is group
	var $IsFolder = 0;
	// Salutation
	var $Salutation = '';
	// User first name
	var $First = '';
	// User second name
	var $Second = '';
	// Address
	var $Address = '';
	// House number
	var $HouseNo = '';
	// City
	var $City = '';
	var $State = '';
	// ZIP Code
	var $PLZ = '';
	// Country
	var $Country = '';
	// Telephone
	var $Telephone = '';
	// Fax
	var $Fax = '';
	// Cell phone
	var $Handy = '';
	// Email
	var $Email = '';
	var $Company = '';
	var $Department = '';
	// Username
	var $username = '';
	// User password (md5 salted)
	var $passwd = '';
	// User password
	var $clearpasswd = '';
	// User permissions
	var $Permissions = '';
	// Flag which indicated if user inherits permissions from parent
	var $ParentPerms = 0;
	// Description
	var $Description = '';
	// User Prefrences
	var $Preferences = [];
	var $Path = '';
	var $Alias = '';
	var $CreatorID = 0;
	var $CreateDate = 0;
	var $ModifierID = 0;
	var $ModifyDate = 0;
	// Ping flag
	var $Ping = 0;
	// Documents workspaces
	var $workSpace = '';
	/* // Default documents workspaces
	  var $workSpaceDef = '';
	 */
	// Templates workspaces
	var $workSpaceTmp = '';
	// Navigation workspaces
	var $workSpaceNav = '';
	// Objects workspaces
	var $workSpaceObj = '';
	// Newsletter workspaces
	var $workSpaceNwl = '';
	// Customer workspaces
	private $workSpaceCust = '';
	// Flag which indicated if user inherits files workspaces from parent
	var $ParentWs = 1;
	// Flag which indicated if user inherits templates workspaces from parent
	var $ParentWst = 1;
	// Flag which indicated if user inherits templates workspaces from parent
	var $ParentWsn = 1;
	// Flag which indicated if user inherits objetcs workspaces from parent
	var $ParentWso = 1;
	// Flag which indicated if user inherits newsletters workspaces from parent
	var $ParentWsnl = 1;
	// Flag which indicated if user inherits customer "workspaces" from parent
	private $ParentWsCust = 1;
	var $LoginDenied = 0;
	// Flag which indicated if user inherits templates workspaces from parent
	var $initExt = 0;

	/*
	 * ADDITIONAL
	 */
	// Workspace array
	var $workspaces = [
		FILE_TABLE => [],
		TEMPLATES_TABLE => [],
		NAVIGATION_TABLE => [],
	];
	// Permissions headers array
	var $permissions_main_titles = [];
	// Permissions values array
	var $permissions_slots = [];
	// Permissions titles
	var $permissions_titles = [];
	private $permissions_defaults = [];
	// Preferences array
	private $preference_slots = ['sizeOpt', 'weWidth', 'weHeight', 'usePlugin', 'autostartPlugin', 'promptPlugin', 'Language', 'BackendCharset', 'seem_start_file',
		'seem_start_type', 'seem_start_weapp', 'editorSizeOpt', 'editorWidth', 'editorHeight', 'editorFontname', 'editorFontsize', 'editorFont', 'default_tree_count', /* 'force_glossary_action', 'force_glossary_check', */
		'cockpit_amount_columns', 'cockpit_amount_last_documents', 'editorMode'];

	// Constructor
	public function __construct(){
		$GLOBALS['editor_reloaded'] = false;

		$this->Name = 'user_' . md5(uniqid(__FILE__, true));

		$this->DB_WE = new DB_WE();
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->workspaces[OBJECT_FILES_TABLE] = [];
		}
		if(we_modules_info::isActive(we_modules_info::NEWSLETTER)){
			$this->workspaces[NEWSLETTER_TABLE] = [];
		}
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->workspaces[CUSTOMER_TABLE] = [];
		}

		foreach($this->preference_slots as $val){
			$this->Preferences[$val] = null;
		}

		$this->initType(self::TYPE_USER);
	}

	function initType($typ){
		$this->Type = $typ;
		$this->mapPermissions();
	}

	// Intialize the class

	function initFromDB($id){
		if(!$id || !f('SELECT 1 FROM ' . USER_TABLE . ' WHERE ID=' . intval($id), '', $this->DB_WE)){
			return false;
		}

		$this->ID = $id;
		$this->getPersistentSlotsFromDB();
		$this->getPreferenceSlotsFromDB();
		$this->loadWorkspaces();
		$this->mapPermissions();

		return true;
	}

	function savePersistentSlotsInDB(){
		$this->ModDate = time();
		$tableInfo = $this->DB_WE->metadata($this->Table, we_database_base::META_NAME);

		$updt = [];

		foreach($tableInfo as $fieldName){
			$val = (isset($this->$fieldName) ? $this->$fieldName : 0 );
			switch($fieldName){
				case 'ID':
					continue 2;
				case 'passwd':
					if($this->clearpasswd === ''){
						continue 2;
					}
					$val = $this->passwd = self::makeSaltedPassword($this->clearpasswd);
					break;
				case 'editorFontname':
					if($this->Preferences['editorFont'] == '0'){
						$val = 'none';
					}
					break;
				case 'editorFontsize':
					if($this->Preferences['editorFont'] == '0'){
						$val = '-1';
					}
					break;
			}
			if($val !== ''){
				$updt[$fieldName] = $val;
			}
		}
		$this->DB_WE->query(($this->ID ? 'UPDATE ' : 'INSERT INTO ') . $this->DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter($updt) . ($this->ID ? ' WHERE ID=' . intval($this->ID) : ''));
		$this->ID = $this->ID ?: $this->DB_WE->getInsertId();
	}

	function createAccount(){

	}

	function removeAccount(){
		we_base_history::deleteByUserID($this->ID);
	}

	function getPersistentSlotsFromDB(){
		$tmp = getHash('SELECT *,DATE_FORMAT(Ping,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS Ping FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ID), $this->DB_WE);
		foreach($tmp as $fieldName => $val){
			if(!isset($this->persistent_slots[$fieldName])){
				continue;
			}
			$this->$fieldName = $val;
		}
	}

	function saveToDB(we_base_jsCmd $jsCmd){
		$db_tmp = new DB_WE();
		$isnew = $this->ID ? false : true;
		if($this->Type == self::TYPE_USER_GROUP && $this->ID != 0){
			$ppath = ($this->ParentID == 0 ? '/' : $this->getPath($this->ParentID));
			$dpath = $this->getPath($this->ID);
			if(preg_match('|' . $dpath . '|', $ppath)){
				return self::ERR_USER_PATH_NOK;
			}
		}
		if($this->Type == self::TYPE_ALIAS){
			$foo = getHash('SELECT ID,username FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->Alias), $this->DB_WE);
			$uorginal = $foo['ID'];
			$search = true;
			$ount = 0;
			$try_name = '@' . $foo['username'];
			$try_text = $foo['username'];
			while($search){
				if(f('SELECT 1 FROM ' . USER_TABLE . ' WHERE ID!=' . intval($this->ID) . ' AND ID!=' . intval($uorginal) . ' AND username="' . $this->DB_WE->escape($try_name) . '" LIMIT 1', '', $this->DB_WE)){
					$try_name = $try_name . '_' . ++$ount;
				} else {
					$search = false;
				}
			}
			$this->username = $try_name;
		}
		$this->IsFolder = ($this->Type == self::TYPE_USER_GROUP ? 1 : 0);
		$this->Path = $this->getPath($this->ID);
		$oldpath = $this->Path;
		$this->saveWorkspaces();
		$this->savePermissions();
		if($isnew){
			$this->CreatorID = $_SESSION['user']['ID'];
			$this->CreateDate = time();
		}
		$this->ModifierID = $_SESSION['user']['ID'];
		$this->ModifyDate = time();
		$this->savePersistentSlotsInDB();
		$this->createAccount();
		if($oldpath != '' && $oldpath != '/'){
			$this->DB_WE->query('SELECT ID,username FROM ' . USER_TABLE . ' WHERE Path LIKE "' . $this->DB_WE->escape($oldpath) . '%"');
			while($this->DB_WE->next_record()){
				$db_tmp->query('UPDATE ' . USER_TABLE . ' SET Path="' . $this->getPath($this->DB_WE->f('ID')) . '" WHERE ID=' . $this->DB_WE->f('ID'));
			}
		}
		$this->savePreferenceSlotsInDB($isnew);

		$_REQUEST['uid'] = $this->ID;

		$this->saveToSession($jsCmd);
		return true;
	}

	function saveToSession(we_base_jsCmd $jsCmd){
		if($this->ID != $_SESSION['user']['ID']){
			return;
		}

		$this->rememberPreference($jsCmd, isset($this->Preferences['Language']) ? $this->Preferences['Language'] : null, 'Language');
		$this->rememberPreference($jsCmd, isset($this->Preferences['BackendCharset']) ? $this->Preferences['BackendCharset'] : null, 'BackendCharset');
		$this->rememberPreference($jsCmd, isset($this->Preferences['default_tree_count']) ? $this->Preferences['default_tree_count'] : null, 'default_tree_count');

		if(isset($this->Preferences['seem_start_type'])){
			switch($this->Preferences['seem_start_type']){
				case 'cockpit':
					$this->rememberPreference($jsCmd, 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, "cockpit", 'seem_start_type');
					$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				case 'object':
					$this->rememberPreference($jsCmd, isset($this->Preferences['seem_start_object']) ? $this->Preferences['seem_start_object'] : 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, 'object', 'seem_start_type');
					$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				case 'weapp':
					$this->rememberPreference($jsCmd, (isset($this->Preferences['seem_start_weapp']) ? $this->Preferences['seem_start_weapp'] : ''), 'seem_start_weapp');
					$this->rememberPreference($jsCmd, 'weapp', 'seem_start_type');
					$this->rememberPreference($jsCmd, 0, 'seem_start_file');
					break;
				case 'document':
					$this->rememberPreference($jsCmd, isset($this->Preferences['seem_start_document']) ? $this->Preferences['seem_start_document'] : 0, 'seem_start_file');
					$this->rememberPreference($jsCmd, 'document', 'seem_start_type');
					$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					break;
				default:
					$this->rememberPreference($jsCmd, '0', 'seem_start_type');
					$this->rememberPreference($jsCmd, '', 'seem_start_weapp');
					$this->rememberPreference($jsCmd, 0, 'seem_start_file');
			}
		}

		$this->rememberPreference($jsCmd, isset($this->Preferences['sizeOpt']) ? $this->Preferences['sizeOpt'] : null, 'sizeOpt');
		$this->rememberPreference($jsCmd, isset($this->Preferences['weWidth']) ? $this->Preferences['weWidth'] : null, 'weWidth');
		$this->rememberPreference($jsCmd, isset($this->Preferences['weHeight']) ? $this->Preferences['weHeight'] : null, 'weHeight');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorMode']) ? $this->Preferences['editorMode'] : null, 'editorMode');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorFont']) ? $this->Preferences['editorFont'] : null, 'editorFont');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorFontname']) ? $this->Preferences['editorFontname'] : null, 'editorFontname');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorFontsize']) ? $this->Preferences['editorFontsize'] : null, 'editorFontsize');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorSizeOpt']) ? $this->Preferences['editorSizeOpt'] : null, 'editorSizeOpt');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorWidth']) ? $this->Preferences['editorWidth'] : null, 'editorWidth');
		$this->rememberPreference($jsCmd, isset($this->Preferences['editorHeight']) ? $this->Preferences['editorHeight'] : null, 'editorHeight'); /*
		  $this->rememberPreference($jsCmd, isset($this->Preferences['force_glossary_action']) ? $this->Preferences['force_glossary_action'] : null, 'force_glossary_action') .;
		  $this->rememberPreference($jsCmd, isset($this->Preferences['force_glossary_check']) ? $this->Preferences['force_glossary_check'] : null, 'force_glossary_check') */
	}

	private function mapPermissions(){
		$this->permissions_main_titles = [];
		$this->permissions_slots = [];
		$this->permissions_titles = [];
		$this->permissions_defaults = [];
		$permissions = we_unserialize($this->Permissions);

		$entries = include($_SERVER['DOCUMENT_ROOT'] . WE_MODULES_DIR . 'users/perms.inc.php');

		foreach($entries as $perm_group_name => $entry){
			if(!($perm_group_name === 'administrator' && $this->Type != self::TYPE_USER) && $perm_group_name){
				$this->permissions_main_titles[$perm_group_name] = $entry['title'];
				$this->permissions_defaults = $this->permissions_defaults + $entry['defaults'];
				$this->permissions_slots[$perm_group_name] = [];
				$this->permissions_titles[$perm_group_name] = [];

				foreach(array_keys($entry['defaults']) as $cur){
					$this->permissions_titles[$perm_group_name][$cur] = g_l('perms_' . $perm_group_name, '[' . $cur . ']');
					$this->permissions_slots[$perm_group_name][$cur] = (is_array($permissions) && isset($permissions[$cur]) ?
							$permissions[$cur] :
							0); //always set unknown fields to 0
				}
			}
		}
	}

	private function setPermission($perm_name, $perm_value){
		foreach($this->permissions_slots as $key => $val){
			foreach($val as $k => $v){
				if($perm_name == $k){
					$this->permissions_slots[$key][$k] = $perm_value;
				}
			}
		}
	}

	private function savePermissions(){
		$permissions = [];
		$isAdmin = false;
		foreach($this->permissions_slots as $val){
			foreach($val as $k => $v){
				switch($k){
					case we_base_perm::ADMINISTRATOR:
						$isAdmin = $v;
					default:
						$permissions[$k] = $v;
				}
			}
		}
		if($isAdmin){
			$permissions = [we_base_perm::ADMINISTRATOR => true];
		}
		$permissions = array_filter($permissions);
		ksort($permissions);

		$this->Permissions = we_serialize($permissions, SERIALIZE_JSON);
	}

	private function loadWorkspaces(){
		if($this->workSpace){
			$this->workspaces[FILE_TABLE] = makeArrayFromCSV($this->workSpace);
		}
		if($this->workSpaceTmp){
			$this->workspaces[TEMPLATES_TABLE] = makeArrayFromCSV($this->workSpaceTmp);
		}
		if($this->workSpaceNav){
			$this->workspaces[NAVIGATION_TABLE] = makeArrayFromCSV($this->workSpaceNav);
		}
		if(we_modules_info::isActive(we_modules_info::OBJECT) && $this->workSpaceObj){
			$this->workspaces[OBJECT_FILES_TABLE] = makeArrayFromCSV($this->workSpaceObj);
		}
		if(we_modules_info::isActive(we_modules_info::NEWSLETTER) && $this->workSpaceNwl){
			$this->workspaces[NEWSLETTER_TABLE] = makeArrayFromCSV($this->workSpaceNwl);
		}
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->workspaces[CUSTOMER_TABLE] = we_unserialize($this->workSpaceCust);
		}
	}

	private function saveWorkspaces(){
		foreach($this->workspaces as $k => $v){
			if($k == CUSTOMER_TABLE){
				continue;
			}
			$new_array = [];

			foreach($v as $key => $val){
				if($val != 0){
					$new_array[] = $this->workspaces[$k][$key];
				}
			}
			$this->workspaces[$k] = $new_array;
		}

		$this->workSpace = implode(',', $this->workspaces[FILE_TABLE]);
		$this->workSpaceTmp = implode(',', $this->workspaces[TEMPLATES_TABLE]);
		$this->workSpaceNav = implode(',', $this->workspaces[NAVIGATION_TABLE]);
		$this->workSpaceObj = implode(',', $this->workspaces[OBJECT_FILES_TABLE]);
		$this->workSpaceNwl = implode(',', $this->workspaces[NEWSLETTER_TABLE]);
		$this->workSpaceCust = $this->workspaces[CUSTOMER_TABLE] ? we_serialize($this->workspaces[CUSTOMER_TABLE], SERIALIZE_JSON) : '';

		// if no workspaces are set, take workspaces from creator
		if(empty($this->workSpace)){
			$uws = get_ws(FILE_TABLE, true);
			if(!empty($uws)){
				$this->workSpace = implode(',', $uws);
				$this->workspaces[FILE_TABLE] = $uws;
			}
		}
		if(empty($this->workSpaceTmp)){
			$uws = get_ws(TEMPLATES_TABLE, true);
			if(!empty($uws)){
				$this->workSpaceTmp = implode(',', $uws);
				$this->workspaces[TEMPLATES_TABLE] = $uws;
			}
		}
		if(empty($this->workSpaceNav)){
			$uws = get_ws(NAVIGATION_TABLE, true);
			if(!empty($uws)){
				$this->workSpaceNav = implode(',', $uws);
				$this->workspaces[NAVIGATION_TABLE] = $uws;
			}
		}

		if(empty($this->workSpaceObj)){
			$uws = get_ws(OBJECT_FILES_TABLE, true);
			if(!empty($uws)){
				$this->workSpaceObj = implode(',', $uws);
				$this->workspaces[OBJECT_FILES_TABLE] = $uws;
			}
		}
		if(empty($this->workSpaceNwl)){
			$uws = get_ws(NEWSLETTER_TABLE, true);
			if(!empty($uws)){
				$this->workSpaceNwl = implode(',', $uws);
				$this->workspaces[NEWSLETTER_TABLE] = $uws;
			}
		}
	}

	private function getPreferenceSlotsFromDB(){
		$tmp = self::readPrefs($this->ID, $this->DB_WE);
		$this->Preferences = array_intersect_key($tmp, array_flip($this->preference_slots));
	}

	private function setPreference($name, $value){
		if(in_array($name, $this->preference_slots)){
			$this->Preferences[$name] = $value;
		}
	}

	private function savePreferenceSlotsInDB($isnew = false){
		if($this->Type != self::TYPE_USER){
			return;
		}

		$this->ModDate = time();
		$updt = ['userID' => intval($this->ID)];
		foreach($this->preference_slots as $fieldName){
			switch($fieldName){
				case 'editorFontsize':
				case 'editorFontname':
					if($this->Preferences['editorFont'] != '1'){
						$this->Preferences[$fieldName] = '-1';
					}
				default:
					$updt[$fieldName] = $this->Preferences[$fieldName];
			}
		}
		if($isnew){
			$updt['userID'] = intval($this->ID);
			//$updt['FileFilter'] = '0';
			$updt['openFolders_tblFile'] = '';
			$updt['openFolders_tblTemplates'] = '';
			$updt['DefaultTemplateID'] = '0';
		}

		self::writePrefs(intval($this->ID), $this->DB_WE, $updt);
		$_SESSION['prefs'] = ($_SESSION['prefs']['userID'] == intval($this->ID) ? self::readPrefs(intval($this->ID), $this->DB_WE) : $_SESSION['prefs']);
	}

	private function rememberPreference(we_base_jsCmd $jsCmd, $settingvalue, $settingname){
		if(isset($settingvalue) && ($settingvalue != null)){
			switch($settingname){
				case 'Language'://FIXME: reload corrupt. fix when needless frames are eliminated
					$_SESSION['prefs']['Language'] = $settingvalue;

					if($settingvalue != $GLOBALS['WE_LANGUAGE']){
						$jsCmd->addCmd('rememberPreference', 'Language', we_base_request::_(we_base_request::INT, 'tab', 0));
					}
					break;
				case 'BackendCharset'://FIXME: reload corrupt. fix when needless frames are eliminated
					$_SESSION['prefs']['BackendCharset'] = $settingvalue;

					if($settingvalue != $GLOBALS['we']['Charset']['Backend']){
						$jsCmd->addCmd('rememberPreference', 'BackendCharset', we_base_request::_(we_base_request::INT, 'tab', 0));
					}
					break;

				case 'seem_start_type':
					switch($settingvalue){
						case 'cockpit':
							$_SESSION['prefs']['seem_start_file'] = 0;
							$_SESSION['prefs']['seem_start_type'] = 'cockpit';
							break;
						case 'object':
							$_SESSION['prefs']['seem_start_file'] = we_base_request::_(we_base_request::INT, 'seem_start_object', 0);
							$_SESSION['prefs']['seem_start_type'] = 'object';
							break;
						case 'weapp':
							$_SESSION['prefs']['seem_start_weapp'] = we_base_request::_(we_base_request::STRING, 'seem_start_weapp', '');
							$_SESSION['prefs']['seem_start_type'] = 'weapp';
							break;
						case 'document':
							$_SESSION['prefs']['seem_start_file'] = we_base_request::_(we_base_request::INT, 'seem_start_document', 0);
							$_SESSION['prefs']['seem_start_type'] = 'document';
							break;
						default:
							$_SESSION['prefs']['seem_start_file'] = 0;
							$_SESSION['prefs']['seem_start_type'] = '0';
					}
					break;

				case 'sizeOpt':
					if($settingvalue == 0){
						$_SESSION['prefs']['weWidth'] = 0;
						$_SESSION['prefs']['weHeight'] = 0;
						$_SESSION['prefs']['sizeOpt'] = 0;
					} else if(($settingvalue == 1) && (we_base_request::_(we_base_request::INT, 'weWidth')) && we_base_request::_(we_base_request::INT, 'weHeight')){
						$_SESSION['prefs']['sizeOpt'] = 1;
					}
					break;

				case 'weWidth':
					if($_SESSION['prefs']['sizeOpt'] == 1){
						$generate_java_script = false;

						if($_SESSION['prefs']['weWidth'] != $settingvalue){
							$generate_java_script = true;
						}

						$_SESSION['prefs']['weWidth'] = $settingvalue;

						if($generate_java_script){
							$height = we_base_request::_(we_base_request::INT, 'weHeight');
							$jsCmd->addCmd('rememberPreference', 'weWidth', [$settingvalue, $height]);
						}
					}
					break;

				case 'weHeight':
					if($_SESSION['prefs']['sizeOpt'] == 1){
						$_SESSION['prefs']['weHeight'] = $settingvalue;
					}
					break;

				case 'editorMode':
					$_SESSION['prefs']['editorMode'] = $settingvalue;
					break;

				case 'editorFont':
					if($settingvalue == 0){
						$_SESSION['prefs']['editorFontname'] = 'none';
						$_SESSION['prefs']['editorFontsize'] = -1;
						$_SESSION['prefs']['editorFont'] = 0;
					} else if(($settingvalue == 1) && we_base_request::_(we_base_request::STRING, 'editorFontname') && we_base_request::_(we_base_request::INT, 'editorFontsize')){
						$_SESSION['prefs']['editorFont'] = 1;
					}
					$jsCmd->addCmd('rememberPreference', 'reload', '');
					break;

				case 'editorFontname':
					if($_SESSION['prefs']['editorFont'] == 1){
						$_SESSION['prefs']['editorFontname'] = $settingvalue;
					}
					break;

				case 'editorFontsize':
					if($_SESSION['prefs']['editorFont'] == 1){
						$_SESSION['prefs']['editorFontsize'] = $settingvalue;
					}
					break;

				case 'editorSizeOpt':
					if($settingvalue == 0){
						$_SESSION['prefs']['editorWidth'] = 0;
						$_SESSION['prefs']['editorHeight'] = 0;
						$_SESSION['prefs']['editorSizeOpt'] = 0;
					} else if(($settingvalue == 1) && we_base_request::_(we_base_request::INT, 'editorWidth') && we_base_request::_(we_base_request::INT, 'editorHeight')){
						$_SESSION['prefs']['editorSizeOpt'] = 1;
					}

					if(empty($GLOBALS['editor_reloaded'])){
						$GLOBALS['editor_reloaded'] = true;
						$jsCmd->addCmd('rememberPreference', 'reload', '');
					}
					break;

				case 'editorWidth':
					if($_SESSION['prefs']['editorSizeOpt'] == 1){
						$_SESSION['prefs']['editorWidth'] = $settingvalue;
					}
					break;

				case 'editorHeight':
					if($_SESSION['prefs']['editorSizeOpt'] == 1){
						$_SESSION['prefs']['editorHeight'] = $settingvalue;
					}
					break;

				case 'default_tree_count':
					$_SESSION['prefs']['default_tree_count'] = $settingvalue;
					break;
				/*
				  case 'force_glossary_check':
				  $_SESSION['prefs']['force_glossary_check'] = $settingvalue;
				  break;

				  case 'force_glossary_action':
				  $_SESSION['prefs']['force_glossary_action'] = $settingvalue;
				  break;
				 */
				case 'cockpit_amount_columns':
					$_SESSION['prefs']['cockpit_amount_columns'] = $settingvalue;
					break;

				default:
					break;
			}
		} else {

			switch($settingname){

				case 'editorFont':
					$_SESSION['prefs']['editorFontname'] = 'none';
					$_SESSION['prefs']['editorFontsize'] = -1;
					$_SESSION['prefs']['editorFont'] = 0;
					$jsCmd->addCmd('rememberPreference', 'reload', '');
					break;

				case 'force_glossary_check':
					$_SESSION['prefs']['force_glossary_check'] = 0;
					break;

				case 'force_glossary_action':
					$_SESSION['prefs']['force_glossary_action'] = 0;
					break;

				default:
					break;
			}
		}
	}

	function preserveState($tab){
		switch($tab){
			case self::TAB_DATA:
				foreach($this->persistent_slots as $name => $type){
					if(($val = we_base_request::_($type, $this->Name . '_' . $name, '__no')) !== '__no'){
						$this->$name = $val;
					}
				}

				if($this->Type == self::TYPE_ALIAS){
					$this->ParentPerms = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentPerms');
					$this->ParentWs = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWs');
					$this->ParentWst = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWst');
					$this->ParentWso = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWso');
					$this->ParentWsn = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsn');
					$this->ParentWsnl = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsnl');
					$this->ParentWsCust = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsCust');
				}
				break;
			case self::TAB_PERMISSION:
				foreach($this->permissions_slots as $pval){
					foreach($pval as $k => $v){
						$this->setPermission($k, we_base_request::_(we_base_request::BOOL, $this->Name . '_Permission_' . $k));
					}
				}
				$this->ParentPerms = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentPerms');
				break;
			case self::TAB_WORKSPACES:
				foreach($this->workspaces as $k => $v){
					$obj = $this->Name . '_Workspace_' . $k;
					if(($val = we_base_request::_(we_base_request::RAW, $obj, false, 'id')) !== false){
						$this->workspaces[$k] = $val;
					}
					$obj = $this->Name . '_Workspace_' . $k . '_AddDel';
					if(($val = we_base_request::_(we_base_request::RAW, $obj, '')) != ''){
						if($val === 'new'){//add
							$this->workspaces[$k][] = 0;
						} else {
							unset($this->workspaces[$k][$val]);
						}
					}
				}
				if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
					$this->workspaces[CUSTOMER_TABLE] = we_customer_abstractFilter::getFilterFromRequest();
				}

				$this->ParentWs = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWs');
				$this->ParentWst = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWst');
				$this->ParentWso = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWso');
				$this->ParentWsn = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsn');
				$this->ParentWsnl = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsnl');
				$this->ParentWsCust = we_base_request::_(we_base_request::BOOL, $this->Name . '_ParentWsCust');
				break;
			case self::TAB_SETTINGS:
				foreach($this->preference_slots as $val){
					switch($val){
						case 'seem_start_file':
						case 'seem_start_type':
						case 'seem_start_weapp':
							$obj = '';
							break;
						default:
							$obj = $this->Name . '_Preference_' . $val;
					}
					$this->setPreference($val, we_base_request::_(we_base_request::RAW_CHECKED, $obj, 0));
				}
				switch(we_base_request::_(we_base_request::STRING, 'seem_start_type')){
					case 'cockpit':
						$this->setPreference('seem_start_file', 0);
						$this->setPreference('seem_start_type', 'cockpit');
						break;
					case 'object':
						$this->setPreference('seem_start_file', we_base_request::_(we_base_request::INT, 'seem_start_object'));
						$this->setPreference('seem_start_type', 'object');
						break;
					case 'weapp':
						$this->setPreference('seem_start_weapp', we_base_request::_(we_base_request::STRING, 'seem_start_weapp'));
						$this->setPreference('seem_start_type', 'weapp');
						break;
					case 'document':
						$this->setPreference('seem_start_file', we_base_request::_(we_base_request::INT, 'seem_start_document'));
						$this->setPreference('seem_start_type', 'document');
						break;
					default:
						$this->setPreference('seem_start_file', 0);
						$this->setPreference('seem_start_type', '0');
				}
				break;
		}
	}

	function checkPermission($perm){
		foreach($this->permissions_slots as $key => $val){
			foreach($val as $k => $v){
				if($k === $perm){
					return ($v ? true : false);
				}
			}
		}
		return false;
	}

	function resetOwnersCreatorModifier(){
		//FIXME: replace will not work for "1,2" if 1 is removed
		$newID = intval($_SESSION['user']['ID']);
		$this->ID = intval($this->ID);
		$this->DB_WE->query('UPDATE ' . FILE_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
		$this->DB_WE->query('UPDATE ' . FILE_TABLE . " SET Owners='' WHERE Owners=','");
		$this->DB_WE->query('UPDATE ' . TEMPLATES_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
		$this->DB_WE->query('UPDATE ' . TEMPLATES_TABLE . " SET Owners='' WHERE Owners=','");
		$this->DB_WE->query('UPDATE ' . FILE_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
		$this->DB_WE->query('UPDATE ' . TEMPLATES_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
		$this->DB_WE->query('UPDATE ' . FILE_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
		$this->DB_WE->query('UPDATE ' . TEMPLATES_TABLE . ' SET ModifierID=' . $newID . ' WHERE ModifierID=' . $this->ID);
		$this->DB_WE->query('UPDATE ' . USER_TABLE . ' SET CreatorID=' . $newID . ' WHERE CreatorID=' . $this->ID);
		$this->DB_WE->query('UPDATE ' . USER_TABLE . ' SET ModifierID=' . $newID . ' WHERE ModifierID=' . $this->ID);

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->DB_WE->query('UPDATE ' . OBJECT_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
			$this->DB_WE->query('UPDATE ' . OBJECT_TABLE . " SET Owners='' WHERE Owners=','");
			$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . " SET Owners=REPLACE(Owners,'," . $this->ID . ",',',')");
			$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . " SET Owners='' WHERE Owners=','");
			$this->DB_WE->query('UPDATE ' . OBJECT_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
			$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . " SET CreatorID=$newID WHERE CreatorID=" . $this->ID);
			$this->DB_WE->query('UPDATE ' . OBJECT_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
			$this->DB_WE->query('UPDATE ' . OBJECT_FILES_TABLE . " SET ModifierID=$newID WHERE ModifierID=" . $this->ID);
		}
	}

	function deleteMe(){
		$this->ID = intval($this->ID);
		switch($this->Type){
			case '':
			case self::TYPE_USER:
				$this->DB_WE->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				$this->DB_WE->query('DELETE FROM ' . PREFS_TABLE . ' WHERE userID=' . $this->ID);
				$this->resetOwnersCreatorModifier();
				$this->removeAccount();
				return true;
			case self::TYPE_USER_GROUP:
				$this->DB_WE->query('SELECT ID FROM ' . USER_TABLE . ' WHERE ParentID=' . $this->ID);
				while($this->DB_WE->next_record()){
					$tmpobj = new we_users_user();
					$tmpobj->initFromDB($this->DB_WE->f('ID'));
					$tmpobj->deleteMe();
				}
				$this->DB_WE->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				$this->resetOwnersCreatorModifier();
				return true;
			case self::TYPE_ALIAS:
				$this->DB_WE->query('DELETE FROM ' . USER_TABLE . ' WHERE ID=' . $this->ID);
				return true;
		}
		return false;
	}

	function isLastAdmin(){
		$this->ID = intval($this->ID);

		if(f('SELECT 1 FROM ' . USER_TABLE . " WHERE Permissions LIKE ('%\"ADMINISTRATOR\";%') AND ID!=" . $this->ID . ' LIMIT 1', '', $this->DB_WE)){
			return false;
		}
		if(($id = intval(f('SELECT ID FROM ' . USER_TABLE . " WHERE (Permissions LIKE ('%\"ADMINISTRATOR\";s:1:\"1\";%') OR Permissions LIKE ('%\"ADMINISTRATOR\":\"1\"%') OR Permissions LIKE ('%\"ADMINISTRATOR\":true%')) AND ID!=" . $this->ID, '', $this->DB_WE)))){
			echo $id . we_html_element::br();
			return false;
		}

		return true;
	}

	function getPath($id = 0){
		$db_tmp = new DB_WE();
		$path = '';
		if($id == 0){
			$id = intval($this->ParentID);
			$path = $db_tmp->escape($this->username);
		}
		$foo = getHash('SELECT username,ParentID FROM ' . USER_TABLE . ' WHERE ID=' . intval($id), $db_tmp);
		$path = '/' . (isset($foo['username']) ? $foo['username'] : '') . $path;
		$pid = isset($foo['ParentID']) ? $foo['ParentID'] : '';
		while($pid > 0){
			$db_tmp->query('SELECT username,ParentID FROM ' . USER_TABLE . ' WHERE ID=' . intval($pid));
			while($db_tmp->next_record()){
				$path = '/' . $db_tmp->f('username') . $path;
				$pid = $db_tmp->f('ParentID');
			}
		}
		return $path;
	}

	public static function getAllPermissions($uid, $onlyParent = false){
		$user_permissions = [];

		$db = $GLOBALS['DB_WE'];
		$db_tmp = new DB_WE();
		$db->query('SELECT ParentID,' . ($onlyParent ? '1 AS ' : '') . ' ParentPerms,' . ($onlyParent ? '"{}" AS ' : '') . 'Permissions,Alias FROM ' . USER_TABLE . ' WHERE ID=' . intval($uid) . ($onlyParent ? '' : ' OR Alias=' . intval($uid)));
		while($db->next_record(MYSQLI_ASSOC)){
			if($db->f('Alias') != $uid){
				$group_permissions = we_unserialize($db->f('Permissions'));
				foreach($group_permissions as $key => $val){
					$user_permissions[$key] = (isset($user_permissions[$key]) ? $user_permissions[$key] : 0) | $group_permissions[$key];
				}
			}
			$lpid = $db->f('ParentID');
			if($db->f('ParentPerms')){
				while($lpid){
					$db_tmp->query('SELECT ParentID,ParentPerms,Permissions FROM ' . USER_TABLE . ' WHERE ID=' . intval($lpid));
					if($db_tmp->next_record(MYSQLI_ASSOC)){
						$group_permissions = we_unserialize($db_tmp->f('Permissions'));
						foreach($group_permissions as $key => $val){
							$user_permissions[$key] = (isset($user_permissions[$key]) ? $user_permissions[$key] : 0) | $group_permissions[$key];
						}
						$lpid = ($db_tmp->f('ParentPerms') ? $db_tmp->f('ParentID') : 0);
						continue;
					}
					$lpid = 0;
				}
			}
		}
		/* 		if(!$onlyParent && !array_filter($user_permissions)){
		  t_e('error reading user permissions! Check parent permissions & resave parent folders! UID: ' . $uid, $user_permissions);
		  } */
		return (!empty($user_permissions[we_base_perm::ADMINISTRATOR]) ? [we_base_perm::ADMINISTRATOR => 1] : array_filter($user_permissions));
	}

	/**
	 * LAYOUT FUNCTIONS
	 */
	function formDefinition(we_base_jsCmd $jsCmd, $tab){
		$weSuggest = & we_gui_suggest::getInstance();
		switch($tab){
			case self::TAB_DATA:
				return we_html_element::jsScript(JS_DIR . 'comparePwd.js', '', ['id' => 'loadVarComparePwd', 'data-passwd' => setDynamicVar([
								'pwdCheck' => SECURITY_USER_PASS_REGEX
					])]) .
						$this->formGeneralData();
			case self::TAB_PERMISSION:
				return $this->formPermissions($jsCmd);
			case self::TAB_WORKSPACES:
				return $this->formWorkspace();
			case self::TAB_SETTINGS:
				return $this->formPreferences($jsCmd);
		}
		return $this->formGeneralData();
	}

	function formGeneralData(){
		switch($this->Type){
			default:
			case self::TYPE_USER:
				return $this->formUserData();
			case self::TYPE_USER_GROUP:
				return $this->formGroupData();
			case self::TYPE_ALIAS:
				return $this->formAliasData();
		}
	}

	function formGroupData(){
		$tableObj = new we_html_table([], 3, 1);

		$username = $this->getUserfield("username", "group_name", "text", 255, false, ['id' => 'yuiAcInputPathName', 'onblur' => "parent.frames[0].weTabs.setTitlePath(this.value);"]);
		$description = '<textarea name="' . $this->Name . '_Description" cols="25" rows="5" style="width:560px" class="defaultfont" onchange="top.content.setHot();">' . $this->Description . '</textarea>';

		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup');
		$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
		$weSuggest->setInput($this->Name . '_ParentID_Text', (f('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID), '', $this->DB_WE) ?: '/'), [], false, true);
		$weSuggest->setMaxResults(10);
		$weSuggest->setRequired(true);
		$weSuggest->setResult($this->Name . '_ParentID', $this->ParentID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setTable(USER_TABLE);
		$weSuggest->setWidth(450);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',document.we_form." . $this->Name . "_ParentID.value,'" . $this->Name . "_ParentID_Text','group','" . $this->Name . "_ParentID');"));

		$weAcSelector = $weSuggest->getHTML();

		$tableObj->setColContent(0, 0, $username);
		$tableObj->setCol(1, 0, ['style' => 'padding-top:5px;'], we_html_tools::formElementTable($description, g_l('modules_users', '[description]')));
		$tableObj->setCol(2, 0, ['style' => 'padding-top:10px;'], we_html_tools::formElementTable($weAcSelector, g_l('modules_users', '[group]')));

		$content = '<select name="' . $this->Name . '_Users" size="8" style="width:560px" onchange="if(this.selectedIndex > -1){WE().layout.button.switch_button_state(document, \'edit\', \'enabled\');}else{WE().layout.button.switch_button_state(document, \'edit\', \'disabled\');}" ondblclick="top.content.we_cmd(\'display_user\',document.we_form.' . $this->Name . '_Users.value)">';
		if($this->ID){
			$this->DB_WE->query('SELECT ID,username,username,Type FROM ' . USER_TABLE . ' WHERE Type IN ("' . self::TYPE_USER . '","' . self::TYPE_ALIAS . '") AND ParentID=' . intval($this->ID));
			while($this->DB_WE->next_record()){
				$content .= '<option value="' . $this->DB_WE->f('ID') . '">' . (($this->DB_WE->f('Type') == self::TYPE_ALIAS) ? '[' : '') . $this->DB_WE->f('username') . (($this->DB_WE->f('Type') == self::TYPE_ALIAS) ? ']' : '');
			}
		}

		$content .= '</select><br/><br/>' . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('display_user',document.we_form." . $this->Name . "_Users.value)", '', "", true, false);

		$parts = [['headline' => g_l('modules_users', '[group_data]'),
		'html' => $tableObj->getHtml(),
		'space' => we_html_multiIconBox::SPACE_MED
			],
			['headline' => g_l('modules_users', '[user]'),
				'html' => $content,
				'space' => we_html_multiIconBox::SPACE_MED
			]
		];

		return we_html_multiIconBox::getHTML('', $parts, 30);
	}

	function getUserfield($name, $lngkey, $type = 'text', $maxlen = 255, $noNull = false, array $attribs = []){
		$val = $this->$name;
		if($noNull && !$val){
			$val = '';
		}
		$attribs['onchange'] = "top.content.setHot()";
		return we_html_tools::formElementTable(we_html_tools::textInput($this->Name . '_' . $name, $val, $maxlen, $attribs, $type, 240), g_l('modules_users', '[' . $lngkey . ']'));
	}

	function formUserData(){

		$tableAdvObj = new we_html_table(['class' => 'withMediumSpace'], 12, 2, [
			[[null, $this->getUserfield('Salutation', 'salutation')]],
			[[null, $this->getUserfield('First', 'first_name')],
				[null, $this->getUserfield('Second', 'second_name')]
			],
			[[null, $this->getUserfield('Address', 'address')],
				[null, $this->getUserfield('HouseNo', 'houseno')]
			],
			[[null, $this->getUserfield('PLZ', 'PLZ', 'text', 16, true)],
				[null, $this->getUserfield('City', 'city')]
			],
			[[null, $this->getUserfield('State', 'state')],
				[null, $this->getUserfield('Country', 'country')]
			],
			[[null, $this->getUserfield('Telephone', 'telephone', 'tel')],
				[null, $this->getUserfield('Fax', 'fax', 'tel')]
			],
			[[null, $this->getUserfield('Handy', 'mobile')],
				[null, $this->getUserfield('Email', 'email', 'email', 255, false, ['required' => 'required'])]
			],
			[[null, $this->getUserfield('Company', 'company')],
				[null, $this->getUserfield('Department', 'department')]
			],
			[[['colspan' => 2], we_html_tools::formElementTable('<textarea name="' . $this->Name . '_Description" cols="25" rows="5" style="width:520px" class="defaultfont" onchange="top.content.setHot();">' . $this->Description . '</textarea>', g_l('modules_users', '[description]'))]
			]
		]);


		$username = $this->getUserfield('username', 'username', 'text', 255, false, ['id' => 'yuiAcInputPathName', 'onblur' => "parent.frames[0].weTabs.setTitlePath(document.getElementsByName(" . $this->Name . "_ParentID_Text')[0].value,this.value);",
			'required' => 'required', 'autocomplete' => 'off']);

		$password = '<div id="badPwd" style="display:none;" class="arrow_box">' . g_l('global', '[pass_to_short]') . '</div>' .
				(!empty($_SESSION['user']['ID']) && $_SESSION['user']['ID'] == $this->ID && !we_base_permission::hasPerm(we_base_perm::USER_EDIT_PASSWD) ?
				'****************' :
				'<input type="hidden" name="' . $this->Name . '_clearpasswd" value="' . $this->clearpasswd . '" />' . we_html_tools::textInput('input_pass', "", 255, ['onchange' => "if(comparePwd('input_pass','input_pass')){document.getElementById('badPwd').style.display='block';}else{document.getElementById('badPwd').style.display='none';}top.content.setHot();",
					'autocomplete' => 'off'], 'password', 240));


		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathGroup');
		$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
		$weSuggest->setInput($this->Name . '_ParentID_Text', (f('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID), '', $this->DB_WE) ?: '/'), [], false, true);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($this->Name . '_ParentID', $this->ParentID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setTable(USER_TABLE);
		$weSuggest->setWidth(403);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',document.we_form." . $this->Name . "_ParentID.value,'" . $this->Name . "_ParentID_Text','group','" . $this->Name . "_ParentID');"));


		$CreatorIDtext = ($this->CreatorID ?
				(f('SELECT CONCAT(username," (",First," ",Second,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->CreatorID), '', $this->DB_WE) ?:
				g_l('modules_users', '[lostID]') . $this->CreatorID . g_l('modules_users', '[lostID2]')) :
				'-');

		$ModifierIDtext = ($this->ModifierID ?
				($this->ModifierID == $this->ID ?
				$this->username . ' (' . $this->First . ' ' . $this->Second . ')' :
				(f('SELECT CONCAT(username," (",First," ",Second,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ModifierID), '', $this->DB_WE) ?:
				g_l('modules_users', '[lostID]') . $this->ModifierID . g_l('modules_users', '[lostID2]'))
				) :
				'-');

		$tableObj = new we_html_table([], 5, 2, [
			[[['style' => 'width:280px;'], $username],
		[['style' => 'width:280px;'], we_html_tools::formElementTable($password, g_l('modules_users', '[password]'))]
			],
			[[null, we_html_forms::checkboxWithHidden($this->LoginDenied, $this->Name . '_LoginDenied', g_l('modules_users', '[login_denied]'), false, "defaultfont", "top.content.setHot();", ($_SESSION['user']['ID'] == $this->ID || !we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)))],
				[null, g_l('modules_users', '[lastPing]') . ' ' . ($this->Ping ?: '-')]
			],
			[[['colspan' => 2,], we_html_tools::formElementTable($weSuggest->getHTML(), g_l('modules_users', '[group]'))]
			],
			[[null, g_l('modules_users', '[CreatorID]') . ' ' . $CreatorIDtext],
				[null, g_l('modules_users', '[CreateDate]') . ' ' . (($this->CreateDate) ? date('d.m.Y H:i:s', $this->CreateDate) : '-')]
			],
			[[null, g_l('modules_users', '[ModifierID]') . ' ' . $ModifierIDtext],
				[null, g_l('modules_users', '[ModifyDate]') . ' ' . (($this->ModifyDate) ? date('d.m.Y H:i:s', $this->ModifyDate) : '-')]
			],
		]);

		$parts = [
			['headline' => g_l('modules_users', '[user_data]'),
				'html' => $tableObj->getHtml(),
				'icon' => we_html_multiIconBox::PROP_PATH
			],
			['headline' => g_l('modules_users', '[general_data]'),
				'html' => $tableAdvObj->getHtml(),
				'icon' => we_html_multiIconBox::PROP_USER,
			],
		];

		return we_html_multiIconBox::getHTML('', $parts, true);
	}

	private function fold_checkbox_groups(array $filter){
		asort($this->permissions_main_titles);
		$parentGroups = $this->ParentID ? self::getAllPermissions($this->ID, true) : false;

		foreach($this->permissions_main_titles as $groupKey => $groupName){
			// Filter out groups not to be shown
			if(in_array($groupKey, $filter)){
				continue;
			}
			$cur = '<table class="default" style="width:500px;margin-left:5em;">';

			foreach($this->permissions_titles[$groupKey] as $perm => $permTitle){
				if(!(we_base_permission::hasPerm($perm))){
					continue;
				}
				$cur .= '<tr><td>' . ($parentGroups === false ? '' : '<i class="showParentPerms fa fa-' . (isset($parentGroups[$perm]) ? 'check" style="color:lightgreen"' : 'close" style="color:red"') . '></i>') . '</td>
	<td style="padding:5px 0;">' . we_html_forms::checkbox(1, ($this->permissions_slots[$groupKey][$perm] ? true : false), $this->Name . '_Permission_' . $perm, $permTitle, false, "defaultfont", "top.content.setHot();") . '</td></tr>';
			}

			$parts[] = [
				'headline' => $groupName,
				'html' => $cur . '</table>',
				//'space' => we_html_multiIconBox::SPACE_SMALL,
				'noline' => true,
				//'rightHeadline' => true,
				'fold' => true
			];
		}

		return $parts;
	}

	/**
	 * This function outputs the group of selectable user permissions
	 *
	 * @param      $branch                                 string
	 *
	 * @return     string
	 */
	private function formPermissions(we_base_jsCmd $jsCmd){
		// Set output text
		// Create a object of the class dynamicControls
		//FIXME: change we_html_dynamicControls
		// Now we create the overview of the user rights
		$parts = $this->fold_checkbox_groups(['administrator']);

		$javascript = 'function toggleRebuildPerm(disabledOnly) {';
		if(!empty($this->permissions_slots['rebuildpermissions'])){
			foreach($this->permissions_slots['rebuildpermissions'] as $pname => $pvalue){
				if($pname != 'REBUILD'){
					$javascript .= '
	if (document.we_form.' . $this->Name . '_Permission_REBUILD && document.we_form.' . $this->Name . '_Permission_' . $pname . ') {
		if(document.we_form.' . $this->Name . '_Permission_REBUILD.checked) {
			document.we_form.' . $this->Name . '_Permission_' . $pname . '.disabled = false;
			if (!disabledOnly) {
				document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = true;
			}
		} else {
			document.we_form.' . $this->Name . '_Permission_' . $pname . '.disabled = true;
			if (!disabledOnly) {
				document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = false;
			}
		}
	}';
				} else {
					$handler = '
	document.we_form.' . $this->Name . '_Permission_' . $pname . ".onclick =  (document.we_form." . $this->Name . '_Permission_' . $pname . "?
		rebuildCheckboxClicked():
		top.content.setHot());
	toggleRebuildPerm(true);";
				}
			}
		}
		$javascript .= '}' . (isset($handler) ? $handler : '');

		// js to uncheck all permissions
		$uncheckjs = $checkjs = $defaultjs = '';
		foreach($this->permissions_defaults as $pname => $pvalue){
			if($pname != we_base_perm::ADMINISTRATOR){
				$uncheckjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = false;';
				$checkjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = true;';
				$defaultjs .= 'document.we_form.' . $this->Name . '_Permission_' . $pname . '.checked = ' . ($pvalue ? 'true' : 'false') . ';';
			}
		}

		$button_uncheckall = we_html_button::create_button('uncheckall', 'javascript:' . $uncheckjs . 'top.content.setHot();');
		$button_checkall = we_html_button::create_button('checkall', 'javascript:' . $checkjs . 'top.content.setHot();');
		$button_default = we_html_button::create_button('default', 'javascript:' . $defaultjs . 'top.content.setHot();');
		$parts[] = ['headline' => '',
			'html' => $button_default . $button_uncheckall . $button_checkall,
		];

		// Check if user has right to decide to give administrative rights
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) && $this->Type == self::TYPE_USER && is_array($this->permissions_slots['administrator'])){
			$content = '';
			foreach($this->permissions_slots['administrator'] as $k => $v){
				$content .= '<tr><td>' . we_html_forms::checkbox(1, $v, $this->Name . "_Permission_" . $k, $this->permissions_titles['administrator'][$k], false, 'defaultfont', ($k === 'REBUILD' ? 'setRebuidPerms();top.content.setHot();' : 'top.content.setHot();')) . '</td></tr>';
			}
			$parts[] = [
				'headline' => $this->permissions_main_titles['administrator'],
				'html' => '<table class="default" style="width:500px;margin-top:5px;">' . $content . '</table>',
			];
		}
		if($this->ParentID){
			$jsCmd->addCmd('showParentPerms', ($this->ParentPerms ? 1 : 0));
			$parts[] = ['headline' => '',
				'html' => $this->formInherits('_ParentPerms', $this->ParentPerms, g_l('modules_users', '[inherit]'), 'showParentPerms(this.checked);'),
			];
		}

		return we_html_multiIconBox::getHTML('', $parts, true) . we_html_element::jsElement($javascript);
	}

	private function formWorkspace(){
		$parentWsp = self::setEffectiveWorkspaces($this->ID, $this->DB_WE, true);
		$parts = [];

		$weSuggest = & we_gui_suggest::getInstance();

		foreach($this->workspaces as $k => $v){
			switch($k){
				case TEMPLATES_TABLE:
					$title = g_l('modules_users', '[workspace_templates]');
					$setValue = 'TEMPLATES_TABLE';
					$showParent = $this->ParentWst;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWst', $this->ParentWst, g_l('modules_users', '[inherit_wst]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case NAVIGATION_TABLE:
					$title = g_l('modules_users', '[workspace_navigations]');
					$setValue = 'NAVIGATION_TABLE';
					$showParent = $this->ParentWsn;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWsn', $this->ParentWsn, g_l('modules_users', '[inherit_wsn]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case NEWSLETTER_TABLE :
					$title = g_l('modules_users', '[workspace_newsletter]');
					$setValue = 'NEWSLETTER_TABLE';
					$showParent = $this->ParentWsnl;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWsnl', $this->ParentWsnl, g_l('modules_users', '[inherit_wsnl]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case OBJECT_FILES_TABLE :
					$title = g_l('modules_users', '[workspace_objects]');
					$setValue = 'OBJECT_TABLE';
					$showParent = $this->ParentWso;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWso', $this->ParentWso, g_l('modules_users', '[inherit_wso]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				case FILE_TABLE:
					$title = g_l('modules_users', '[workspace_documents]');
					$setValue = 'FILE_TABLE';
					$showParent = $this->ParentWs;
					$content1 = $this->ParentID ? $this->formInherits('_ParentWs', $this->ParentWs, g_l('modules_users', '[inherit_ws]'), 'document.getElementById(\'info' . $setValue . '\').style.display=(this.checked?\'inline\':\'none\');') : '';
					break;
				default:
					continue 2;
			}
			$obj_values = $this->Name . '_Workspace_' . $k . '_AddDel';
			$obj_names = $this->Name . '_Workspace_' . $k;
			//$obj_def_names = $this->Name . '_defWorkspace_' . $k;
			//$content .= '<p>';

			$content1 .= '<input type="hidden" name="' . $obj_values . '" value="" /><table style="width:520px">';
			foreach($v as $key => $val){
				$value = $val;
				$path = f('SELECT Path FROM ' . $k . ' WHERE ' . $k . '.ID=' . $value, '', $this->DB_WE);
				if(!$path){
					$foo = get_def_ws($k);
					$fooA = makeArrayFromCSV($foo);
					$value = (count($fooA) ? $fooA[0] : 0);
					$path = id_to_path($value);
				}


				$weSuggest->setAcId('WS' . $k . $key);
				$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
				$weSuggest->setInput($obj_names . '[Text][' . $key . ']', $path);
				$weSuggest->setMaxResults(10);
				$weSuggest->setResult($obj_names . '[id][' . $key . ']', $value);
				$weSuggest->setSelector(we_gui_suggest::DirSelector);
				$weSuggest->setTable($k);
				$weSuggest->setWidth(290);

				switch($k){
					case NEWSLETTER_TABLE :
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_newsletter_dirSelector',document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,'" . $obj_names . '[id][' . $key . "]','" . $obj_names . '[Text][' . $key . "]','','','" . we_base_request::_(we_base_request::INT, "rootDirID", 0) . "' )"));
						break;

					case NAVIGATION_TABLE:
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_navigation_dirSelector',document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,'" . $obj_names . '[id][' . $key . "]','" . $obj_names . '[Text][' . $key . "]','','','" . we_base_request::_(we_base_request::INT, "rootDirID", 0) . "' )"));
						break;

					default:
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.getElementsByName('" . $obj_names . "[id][" . $key . "]')[0].value,'" . $k . "','" . $obj_names . '[id][' . $key . "]','" . $obj_names . '[Text][' . $key . "]','','','" . we_base_request::_(we_base_request::INT, "rootDirID", 0) . "' )"));
				}

				$weAcSelector = $weSuggest->getHTML();

				$content1 .= '
<tr><td colspan="2">' . $weAcSelector . '</td>
	<td><div style="position:relative; top:-1px">' . we_html_button::create_button(we_html_button::TRASH, "javascript:delElement(document.we_form." . $obj_values . "," . $key . ");switchPage(" . self::TAB_WORKSPACES . ");") . '</td></div>' . '
</tr>';
			}

			$content1 .= '<colgroup><col style="width:300px"/><col style="width:110px"/><col style="width:40px"/><col style="width:90px"/></colgroup>
	<tr><td colspan="4">' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addElement(document.we_form." . $obj_values . ");") . '</td></tr>
</table>';

			if($parentWsp[$k]){
				$this->DB_WE->query('SELECT Path FROM ' . $k . ' WHERE ID IN(' . implode(',', $parentWsp[$k]) . ')');
				$parent = implode("<br/>", $this->DB_WE->getAll(true));
			} else {
				$parent = ' - ';
			}

			$parts[] = ['headline' => $title,
				'html' => ($this->ParentID ? '<div id="info' . $setValue . '" style="' . ($showParent ? '' : 'display:none;') . '">' . we_html_tools::alertAttentionBox($parent, we_html_tools::TYPE_INFO, 600, false) . '</div>' : '') . $content1,
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$filter = new we_navigation_customerFilter(we_customer_abstractFilter::FILTER, [], [], [], $this->workspaces[CUSTOMER_TABLE]);
			$view = new we_customer_filterView($filter, 520);
			if($parentWsp[CUSTOMER_TABLE]){

				$parent = '';
				foreach($parentWsp[CUSTOMER_TABLE] as $setting){
					$setting = we_unserialize($setting);
					foreach($setting as $cur){
						$parent .= ($parent ? ' ' . $cur['logic'] . ' ' : '') . we_customer_abstractFilter::evalSingleFilterQuery($cur['operation'], $cur['field'], $cur['value']) . "\n";
					}
				}
			} else {
				$parent = ' - ';
			}
			$parts[] = ['headline' => g_l('modules_users', '[workspace_customer]'),
				'html' => ($this->ParentID ?
				'<div id="infoCUSTOMER" style="' . ($this->ParentWsCust ? '' : 'display:none;') . '">' . we_html_tools::alertAttentionBox($parent, we_html_tools::TYPE_INFO, 600) . '</div>' .
				$this->formInherits('_ParentWsCust', $this->ParentWsCust, g_l('modules_users', '[inherit_cust]'), 'document.getElementById(\'infoCUSTOMER\').style.display=(this.checked?\'inline\':\'none\');') : '') .
				$view->getFilterCustomers(),
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		return we_html_multiIconBox::getHTML('', $parts, 30);
	}

	private function formPreferences(we_base_jsCmd $jsCmd){
		return we_html_multiIconBox::getHTML('', $this->formPreferencesUI($jsCmd), true);
	}

	/*
	  private function formPreferencesGlossary(){
	  $settings = [];
	  return [];
	  // Create checkboxes
	  $table = new we_html_table(['class' => 'default withSpace'], 2, 1);
	  //FIXME: where is the difference between force_glossary_check + force_glossary_action?!

	  $table->setColContent(0, 0, we_html_forms::checkbox(1, $this->Preferences['force_glossary_check'], $this->Name . '_Preference_force_glossary_check', g_l('prefs', '[force_glossary_check]'), 'false', 'defaultfont', "top.content.setHot()"));
	  $table->setColContent(1, 0, we_html_forms::checkbox(1, $this->Preferences['force_glossary_action'], $this->Name . "_Preference_force_glossary_action", g_l('prefs', '[force_glossary_action]'), "false", "defaultfont", "top.content.setHot()"));

	  // Build dialog if user has permission
	  if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
	  $settings[] = ['headline' => g_l('prefs', '[glossary_publishing]'), 'html' => $table->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true];
	  }

	  return $settings;
	  }
	 */

	private function formPreferencesUI(we_base_jsCmd $jsCmd){
		$settings = [];
		// LANGUAGE
		//	Look which languages are installed ...
		$language_directory = dir(WE_INCLUDES_PATH . 'we_language');

		while(false !== ($entry = $language_directory->read())){
			if($entry != '.' && $entry != '..'){
				if(is_dir(WE_INCLUDES_PATH . 'we_language/' . $entry)){
					$language[$entry] = $entry;
				}
			}
		}
		global $languages;

		if(!empty($language)){ // Build language select box
			$languages = new we_html_select(['name' => $this->Name . '_Preference_Language', 'class' => 'weSelect']);
			$myCompLang = (!empty($this->Preferences['Language']) ? $this->Preferences['Language'] : $GLOBALS['WE_LANGUAGE']);

			foreach($language as $key => $value){
				$languages->addOption($key, $value);

				// Set selected extension
				if($key == $myCompLang){
					$languages->selectOption($key);
				} else {
					// do nothing
				}
			}

			// Build dialog
			$settings[] = ['headline' => g_l('prefs', '[choose_language]'), 'html' => $languages->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED, 'noline' => true];
		}


		$charset = new we_html_select(['name' => $this->Name . '_Preference_BackendCharset', 'class' => 'weSelect', 'onchange' => 'top.content.setHot();']);
		$c = we_base_charsetHandler::getAvailCharsets();
		foreach($c as $char){
			$charset->addOption($char, $char);
		}
		$myCompChar = (!empty($this->Preferences['BackendCharset']) ? $this->Preferences['BackendCharset'] : $GLOBALS['we']['Charset']['Backend']);
		$charset->selectOption($myCompChar);
		$settings[] = ['headline' => g_l('prefs', '[choose_backendcharset]'),
			'html' => $charset->getHtml(),
			'space' => we_html_multiIconBox::SPACE_MED
		];

		//AMOUNT Number of Columns
		$amount = new we_html_select(['name' => $this->Name . '_Preference_cockpit_amount_columns', 'class' => 'weSelect', 'onchange' => "top.content.setHot();"]);
		if(!$this->Preferences['cockpit_amount_columns']){
			$this->Preferences['cockpit_amount_columns'] = 3;
		}
		for($i = 1; $i <= 10; $i++){
			$amount->addOption($i, $i);
			if($i == $this->Preferences['cockpit_amount_columns']){
				$amount->selectOption($i);
			}
		}

		$settings[] = ['headline' => g_l('prefs', '[cockpit_amount_columns]'),
			'html' => $amount->getHtml(),
			'space' => we_html_multiIconBox::SPACE_MED
		];

		//SEEM
		// Cockpit
		$document_path = $object_path = '';
		$document_id = $object_id = 0;

		switch($this->Preferences['seem_start_type']){
			default:
				$seem_start_type = '0';
				break;
			case 'cockpit':
				$_SESSION['prefs']['seem_start_file'] = 0;
				$seem_start_type = 'cockpit';
				break;
			case 'object':
				$seem_start_type = 'object';
				if($this->Preferences['seem_start_file'] != 0){
					$object_id = $this->Preferences['seem_start_file'];
					$get_object_paths = getPathsFromTable(OBJECT_FILES_TABLE, $GLOBALS['DB_WE'], we_base_constants::FILE_ONLY, $object_id);

					if(isset($get_object_paths[$object_id])){ //	seeMode start file exists
						$object_path = $get_object_paths[$object_id];
					}
				}
				break;
			case 'weapp':
				$seem_start_type = 'weapp';
				break;
			// Document
			case 'document':
				$seem_start_type = 'document';
				if($this->Preferences['seem_start_file'] != 0){
					$document_id = $this->Preferences['seem_start_file'];
					$get_document_paths = getPathsFromTable(FILE_TABLE, $GLOBALS['DB_WE'], we_base_constants::FILE_ONLY, $document_id);

					if(isset($get_document_paths[$document_id])){ //	seeMode start file exists
						$document_path = $get_document_paths[$document_id];
					}
				}
		}

		$start_type = new we_html_select(['name' => 'seem_start_type', 'class' => 'weSelect', 'id' => 'seem_start_type', 'onchange' => "show_seem_chooser(this.value); top.content.setHot();"]);
		$start_type->addOption(0, '-');
		$start_type->addOption('cockpit', g_l('prefs', '[seem_start_type_cockpit]'));
		$start_type->addOption('document', g_l('prefs', '[seem_start_type_document]'));
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$start_type->addOption('object', g_l('prefs', '[seem_start_type_object]'));
		}

		//weapp
		//$start_weapp = new we_html_select(['name' => 'seem_start_weapp', 'class' => 'weSelect', 'id' => 'seem_start_weapp', 'onchange' => 'top.content.setHot();']);
		/* $tools = we_tool_lookup::getAllTools(true, false);
		  foreach($tools as $tool){
		  if(!$tool['appdisabled'] && we_base_permission::hasPerm($tool['startpermission'])){
		  $start_weapp->addOption($tool['name'], $tool['text']);
		  }
		  }
		  if($start_weapp->getOptionNum()){
		  $start_type->addOption('weapp', g_l('prefs', '[seem_start_type_weapp]'));
		  } */

		$start_type->selectOption($seem_start_type);
		//$start_weapp->selectOption($this->Preferences['seem_start_weapp']);
		//$seem_weapp_chooser = we_html_element::span(['id' => 'seem_start_weapp', 'style' => 'display:none'], $start_weapp->getHtml());
		// Build SEEM select start document chooser
		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc');
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::IMAGE, we_base_ContentTypes::HTML, we_base_ContentTypes::JS,
			we_base_ContentTypes::CSS, we_base_ContentTypes::APPLICATION, we_base_ContentTypes::PDF]);
		$weSuggest->setInput('seem_start_document_name', $document_path);
		$weSuggest->setMaxResults(20);
		$weSuggest->setResult('seem_start_document', $document_id);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setWidth(191);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));

		$seem_document_chooser = we_html_element::span(['id' => 'seem_start_document', 'style' => 'display:none'], $weSuggest->getHTML());

		// Build SEEM select start object chooser
		$weSuggest->setAcId('Obj');
		$weSuggest->setContentType('folder,' . we_base_ContentTypes::OBJECT_FILE);
		$weSuggest->setInput('seem_start_object_name', $object_path);
		$weSuggest->setMaxResults(20);
		$weSuggest->setResult('seem_start_object', $object_id);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$weSuggest->setTable(OBJECT_FILES_TABLE);
		}
		$weSuggest->setWidth(191);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));

		$seem_object_chooser = we_html_element::span(['id' => 'seem_start_object', 'style' => 'display:none'], $weSuggest->getHTML());

		// Build final HTML code
		$seem_html = new we_html_table(['class' => 'default'], 2, 1);
		$seem_html->setColContent(0, 0, $start_type->getHtml());
		$seem_html->setColContent(1, 0, $seem_document_chooser . $seem_object_chooser /* . $seem_weapp_chooser */);

		if(we_base_permission::hasPerm(we_base_perm::SEEM_CHANGE_START_DOCUMENT)){
			$jsCmd->addCmd('show_seem_chooser', $seem_start_type);
			$settings[] = ['headline' => g_l('prefs', '[seem_startdocument]'),
				'html' => $seem_html->getHtml(),
				'space' => we_html_multiIconBox::SPACE_MED
			];
		}

		// TREE

		$value_selected = false;
		$tree_count = $this->Preferences['default_tree_count'];

		$file_tree_count = new we_html_select(['name' => $this->Name . '_Preference_default_tree_count', 'class' => 'weSelect', 'onchange' => 'top.content.setHot();']);

		$file_tree_count->addOption(0, g_l('prefs', '[all]'));
		if(0 == $tree_count){
			$file_tree_count->selectOption(0);
			$value_selected = true;
		}

		for($i = 10; $i < 51; $i += 10){
			$file_tree_count->addOption($i, $i);

			// Set selected extension
			if($i == $tree_count){
				$file_tree_count->selectOption($i);
				$value_selected = true;
			}
		}

		for($i = 100; $i < 501; $i += 100){
			$file_tree_count->addOption($i, $i);

			// Set selected extension
			if($i == $tree_count){
				$file_tree_count->selectOption($i);
				$value_selected = true;
			}
		}

		if(!$value_selected){
			$file_tree_count->addOption($tree_count, $tree_count);
			// Set selected extension
			$file_tree_count->selectOption($tree_count);
		}

		$settings[] = ['headline' => g_l('prefs', '[tree_title]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[tree_count_description]'), we_html_tools::TYPE_INFO) . '<br/>' . $file_tree_count->getHtml(),
			'space' => we_html_multiIconBox::SPACE_MED];

		// WINDOW DIMENSIONS

		$window_max = $window_specify = false;

		if($this->Preferences['sizeOpt'] == 0){
			$window_max = true;
		} elseif($this->Preferences['sizeOpt'] == 1){
			$window_specify = true;
		}

		// Build maximize window
		$window_max_code = we_html_forms::radiobutton(0, $this->Preferences['sizeOpt'] == 0, $this->Name . '_Preference_sizeOpt', g_l('prefs', '[maximize]'), true, 'defaultfont', "document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = true;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = true;top.content.setHot();");

		// Build specify window dimension
		$window_specify_code = we_html_forms::radiobutton(1, !($this->Preferences['sizeOpt'] == 0), $this->Name . '_Preference_sizeOpt', g_l('prefs', '[specify]'), true, 'defaultfont', "document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;top.content.setHot();");

		// Create specify window dimension input
		$window_specify_table = new we_html_table(['class' => 'default', 'style' => 'margin:5px 0;'], 1, 3);

//		$window_specify_table->setCol(0, 0, ['class' => 'defaultfont', 'style' => 'padding-left:40px;padding-right:10px;'], g_l('prefs', '[width]') . ':');
		$window_specify_table->setColContent(0, 0, we_html_tools::textInput($this->Name . '_Preference_weWidth', ($this->Preferences['weWidth'] != '' && $this->Preferences['weWidth'] != '0' ? $this->Preferences['weWidth'] : 800), 4, [
					'disabled' => ($this->Preferences['sizeOpt'] == 0 ? "disabled" : ""), 'onchange' => 'top.content.setHot();'], "text", 60));
		//$window_specify_table->setCol(1, 0, ['class' => 'defaultfont', 'style' => 'padding-left:40px;padding-right:10px;'], g_l('prefs', '[height]') . ':');
		$window_specify_table->setCol(0, 1, ['style' => 'padding:0 .5em;vertical-align: middle;'], 'x');
		$window_specify_table->setColContent(0, 2, we_html_tools::textInput($this->Name . "_Preference_weHeight", ( ($this->Preferences['weHeight'] != '' && $this->Preferences['weHeight'] != '0') ? $this->Preferences['weHeight'] : 600), 4, [
					'disabled' => ($this->Preferences['sizeOpt'] == 0 ? "disabled" : ""), 'onchange' => 'top.content.setHot();'], "text", 60));

		// Build apply current window dimension
		$window_current_dimension_table = '<div style="padding-left:90px;">' .
				we_html_button::create_button('apply_current_dimension', "javascript:top.content.setHot();document.getElementsByName('" . $this->Name . "_Preference_sizeOpt')[1].checked = true;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].value = top.opener.top.window.outerWidth;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].value =top.opener.top.window.outerHeight;") . '</div>';

		// Build final HTML code
		$window_html = new we_html_table(['class' => 'default withBigSpace'], 3, 1);
		$window_html->setColContent(0, 0, $window_max_code);
		$window_html->setColContent(1, 0, $window_specify_code . $window_specify_table->getHtml());
		$window_html->setColContent(2, 0, $window_current_dimension_table);

		// Build dialog
		$settings[] = ['headline' => g_l('prefs', '[dimension]'), 'html' => $window_html->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED];

		// Create predefined window dimension buttons
		$window_predefined_table = new we_html_table(['class' => 'withBigSpace', 'style' => 'text-align:right'], 2, 1);

		$window_predefined_table->setColContent(0, 0, we_html_button::create_button('res_800', "javascript:top.content.setHot();document.getElementsByName('" . $this->Name . "_Preference_sizeOpt')[1].checked = true;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].value = '800';document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].value = '600';") .
				we_html_button::create_button('res_1024', "javascript:top.content.setHot();document.getElementsByName('" . $this->Name . "_Preference_sizeOpt')[1].checked = true;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].value = '1024';document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].value = '768';"));
		$window_predefined_table->setColContent(1, 0, we_html_button::create_button('res_1280', "javascript:top.content.setHot();document.getElementsByName('" . $this->Name . "_Preference_sizeOpt')[1].checked = true;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].value = '1280';document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].value = '960';") . we_html_button::create_button('res_1600', "javascript:top.content.setHot();document.getElementsByName('" . $this->Name . "_Preference_sizeOpt')[1].checked = true;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_weWidth')[0].value = '1600';document.getElementsByName('" . $this->Name . "_Preference_weHeight')[0].value = '1200';"));

		// Build dialog
		$settings[] = ['headline' => g_l('prefs', '[predefined]'), 'html' => $window_predefined_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED];

		return $settings;
	}

	function formPreferencesEditor(){
		return [];
		//FIXME: this is not correct!
		//Editor Mode
		$template_editor_mode = new we_html_select(['class' => "weSelect", 'name' => $this->Name . "_Preference_editorMode", 'onchange' => "displayEditorOptions(this.options[this.options.selectedIndex].value);"]);
		$template_editor_mode->addOption('textarea', g_l('prefs', '[editor_plaintext]'));
		$template_editor_mode->addOption('codemirror2', g_l('prefs', '[editor_javascript2]'));
		$template_editor_mode->selectOption($this->Preferences['editorMode']);
		$settings = [['headline' => g_l('prefs', '[editor_mode]'), 'html' => $template_editor_mode->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED2]
		];
//FIXME: use code from preferences for font-selection
		$template_fonts = ['Arial', 'Courier', 'Courier New', 'Helvetica', 'Monaco', 'Mono', 'Tahoma', 'Verdana', 'serif', 'sans-serif', 'none'];
		$template_font_sizes = [8, 9, 10, 11, 12, 14, 16, 18, 24, 32, 48, 72, -1];

		$template_editor_font_specify = false;
		$template_editor_font_size_specify = false;

		if($this->Preferences['editorFontname'] != "" && $this->Preferences['editorFontname'] != "none"){
			$template_editor_font_specify = true;
		}

		if($this->Preferences['editorFontsize'] != "" && $this->Preferences['editorFontsize'] != -1){
			$template_editor_font_size_specify = true;
		}

		// Build specify font
		$template_editor_font_specify_code = we_html_forms::checkbox(1, $template_editor_font_specify, $this->Name . "_Preference_editorFont", g_l('prefs', '[specify]'), true, "defaultfont", "top.content.setHot(); if (document.getElementsByName('" . $this->Name . "_Preference_editorFont')[0].checked) { document.getElementsByName('" . $this->Name . "_Preference_editorFontname')[0].disabled = false;document.getElementsByName('" . $this->Name . "_Preference_editorFontsize')[0].disabled = false; } else { document.getElementsByName('" . $this->Name . "_Preference_editorFontname')[0].disabled = true;document.getElementsByName('" . $this->Name . "_Preference_editorFontsize')[0].disabled = true; }");

		$template_editor_font_select_box = new we_html_select(['class' => "weSelect", 'name' => $this->Name . "_Preference_editorFontname", 'style' => "width: 90px;",
			($template_editor_font_specify ? "enabled" : "disabled") => ($template_editor_font_specify ? "enabled" : "disabled"), 'onchange' => "top.content.setHot();"]);

		foreach($template_fonts as $tf){
			$template_editor_font_select_box->addOption($tf, $tf);

			if(!$template_editor_font_specify){
				if($tf === "Courier New"){
					$template_editor_font_select_box->selectOption($tf);
				}
			} else {
				if($tf == $this->Preferences['editorFontname']){
					$template_editor_font_select_box->selectOption($tf);
				}
			}
		}

		$template_editor_font_sizes_select_box = new we_html_select(['class' => 'weSelect', 'name' => $this->Name . '_Preference_editorFontsize', 'style' => "width: 90px;",
			($template_editor_font_size_specify ? "enabled" : "disabled") => ($template_editor_font_size_specify ? "enabled" : "disabled"), 'onchange' => "top.content.setHot();"]);

		foreach($template_font_sizes as $tf){
			$template_editor_font_sizes_select_box->addOption($tf, $tf);

			if(!$template_editor_font_specify){
				if($tf == 11){
					$template_editor_font_sizes_select_box->selectOption($tf);
				}
			} else {
				if($tf == $this->Preferences['editorFontsize']){
					$template_editor_font_sizes_select_box->selectOption($tf);
				}
			}
		}
		// Create specify window dimension input
		$template_editor_font_specify_table = new we_html_table(['class' => 'default withSpace', 'style' => 'margin:5px 0 0 50px;'], 2, 2);

		$template_editor_font_specify_table->setCol(0, 0, ['class' => 'defaultfont', 'style' => 'padding-right:10px;'], g_l('prefs', '[editor_fontname]') . ":");
		$template_editor_font_specify_table->setColContent(0, 1, $template_editor_font_select_box->getHtml());
		$template_editor_font_specify_table->setCol(1, 0, ['class' => 'defaultfont', 'style' => 'padding-right:10px;'], g_l('prefs', '[editor_fontsize]') . ":");
		$template_editor_font_specify_table->setColContent(1, 1, $template_editor_font_sizes_select_box->getHtml());

		// Build dialog
		$settings[] = ['headline' => g_l('prefs', '[editor_font]'),
			'html' => $template_editor_font_specify_code . $template_editor_font_specify_table->getHtml(),
			'space' => we_html_multiIconBox::SPACE_MED
		];

		return $settings;
	}

	function formAliasData(){
		$alias_text = ($this->ID ? f('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->Alias), 'Path', $this->DB_WE) : '');
		$parent_text = ($this->ParentID == 0 ? '/' : f('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->ParentID), 'Path', $this->DB_WE));

		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('PathName');
		$weSuggest->setContentType(self::TYPE_USER . ',' . self::TYPE_USER_GROUP); // in USER_TABLE is Type 0 folder, Type 1 user and Type 2 alias. Field ContentType is not setted so in weSelectorQuery is a workaroun for USER_TABLE
		$weSuggest->setInput($this->Name . '_Alias_Text', $alias_text, [], false, true);
		$weSuggest->setMaxResults(10);
		$weSuggest->setRequired(true);
		$weSuggest->setResult($this->Name . '_Alias', $this->Alias);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable(USER_TABLE);
		$weSuggest->setWidth(200);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',document.we_form." . $this->Name . "_Alias.value,'" . $this->Name . "_Alias_Text','noalias','" . $this->Name . "_Alias')"));

		$weAcSelectorName = $weSuggest->getHTML();

		$weSuggest->setAcId("PathGroup");
		$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
		$weSuggest->setInput($this->Name . '_ParentID_Text', $parent_text, [], false, true);
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($this->Name . '_ParentID', $this->ParentID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setTable(USER_TABLE);
		$weSuggest->setWidth(200);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',document.we_form." . $this->Name . "_ParentID.value,'" . $this->Name . "_ParentID_Text','group','" . $this->Name . "_ParentID')"));

		$weAcSelectorGroup = $weSuggest->getHTML();

		$content = '
<table class="default" style="width:530px;">
<colgroup><col style="width:170px;"/><col style="width:330px;"/></colgroup>
	<tr>
		<td class="defaultfont">' . g_l('modules_users', '[user]') . ':</td>
		<td>' . $weAcSelectorName . '</td>
	</tr>
	<tr>
		<td colspan="2" style="height:5px;"></td>
	</tr>
	<tr>
		<td class="defaultfont">' . g_l('modules_users', '[group_member]') . ':</td>
		<td>' . $weAcSelectorGroup . '</td>
	</tr>
	<tr>
		<td colspan="2" style="height:1px;"></td>
	</tr>
</table>';

		$parts = [['headline' => g_l('modules_users', '[alias_data]'),
		'html' => $content,
		'space' => we_html_multiIconBox::SPACE_MED
			], ['headline' => g_l('modules_users', '[rights_and_workspaces]'),
				'html' =>
				$this->formInherits("_ParentPerms", $this->ParentPerms, g_l('modules_users', '[inherit]')) .
				$this->formInherits("_ParentWs", $this->ParentWs, g_l('modules_users', '[inherit_ws]')) .
				$this->formInherits("_ParentWst", $this->ParentWst, g_l('modules_users', '[inherit_wst]')),
				'space' => we_html_multiIconBox::SPACE_MED
			]
		];

		return we_html_multiIconBox::getHTML('', $parts, 30);
	}

	function formInherits($name, $value, $title, $onClick = ''){
		return '
<table class="default" style="width:500px;">
	<tr>
		<td class="defaultfont">' .
				we_html_forms::checkbox(1, ($value ? true : false), $this->Name . $name, $title, '', 'defaultfont', 'top.content.setHot();' . $onClick) . '
	</tr>
</table>';
	}

	function formHeader($tab = self::TAB_DATA){
		switch($this->Type){
			case self::TYPE_USER_GROUP:
				$headline1 = g_l('modules_users', '[group]') . ': ';
				$tabs = [
					self::TAB_DATA => ['data', 'fa-users'],
					self::TAB_PERMISSION => ['permissions', 'fa-user-secret'],
					self::TAB_WORKSPACES => ['workspace', we_gui_tabs::WE_ICON_WORKSPACE]
				];
				break;
			case self::TYPE_ALIAS:
				$tabs = [
					self::TAB_DATA => ['data', 'fa-user']
				];
				$headline1 = g_l('menu_users', '[menu_alias]') . ': ';
				break;
			case self::TYPE_USER:
			default:
				$tabs = [
					self::TAB_DATA => ['data', 'fa-user'],
					self::TAB_PERMISSION => ['permissions', 'fa-user-secret'],
					self::TAB_WORKSPACES => ['workspace', we_gui_tabs::WE_ICON_WORKSPACE],
					self::TAB_SETTINGS => ['preferences', we_gui_tabs::WE_ICON_PROPERTIES]
				];
				$headline1 = g_l('menu_users', '[menu_user]') . ': ';
		}

		$we_tabs = new we_gui_tabs();
		foreach($tabs as $key => $val){
			$we_tabs->addTab('', $val[1], ($tab == $key), $key, ['title' => g_l('tabs', '[module][' . $val[0] . ']')]);
		}


		return we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_property.js') .
				'<div id="main"><div id="headrow"><b>' . str_replace(" ", "&nbsp;", $headline1) . '&nbsp;</b><span id="h_path" class="header_small"><b id="titlePath">' . str_replace(" ", "&nbsp;", ($this->Path ?: $this->getPath($this->ParentID))) . '</b> (ID:' . $this->ID . ')</span></div>' . $we_tabs->getHTML() . '</div>';
	}

	public static function getUsername($id, we_database_base $db = null){
		$db = $db ?: new DB_WE();
		$user = f('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($id), '', $db);
		return $user ?: g_l('modules_messaging', '[userid_not_found]');
	}

	public static function getUserID($username, we_database_base $db){
		$uid = f('SELECT ID FROM ' . USER_TABLE . ' WHERE username="' . $db->escape(trim($username)) . '"', '', $db);
		return $uid ?: -1;
	}

	static function filenameNotValid($username){
		return preg_match('|^[A-Z0-9._\-][A-Z0-9.,_\-@]+$|i', $username);
	}

	/**
	 *
	 * @param int $user
	 * @param we_database_base $db
	 * @param boolean $onlyParent
	 * @return array|void
	 */
	static function setEffectiveWorkspaces($user, we_database_base $db, $onlyParent = false){
		$workspaces = [
			FILE_TABLE => ['key' => 'workSpace', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWs', 'explodeValue' => true],
			TEMPLATES_TABLE => ['key' => 'workSpaceTmp', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWst', 'explodeValue' => true],
			NAVIGATION_TABLE => ['key' => 'workSpaceNav', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsn', 'explodeValue' => true],
			OBJECT_FILES_TABLE => ['key' => 'workSpaceObj', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWso', 'explodeValue' => true],
			NEWSLETTER_TABLE => ['key' => 'workSpaceNwl', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsnl', 'explodeValue' => true],
			CUSTOMER_TABLE => ['key' => 'workSpaceCust', 'value' => [], 'parent' => 0, 'parentKey' => 'ParentWsCust', 'explodeValue' => false],
		];

//FIXME: onlyParent doesn't work correctly
		$fields = ['ParentID'];
		foreach($workspaces as $cur){
			$fields[] = $cur['key'];
			$fields[] = $cur['parentKey'];
		}
		$fields = implode(',', $fields);

		$userGroups = []; //	Get Groups user belongs to.

		$pids = [];
		$db->query('SELECT ' . $fields . ' FROM ' . USER_TABLE . ' WHERE ID=' . intval($user) . ($onlyParent ? '' : ' OR Alias=' . intval($user)));
		while($db->next_record()){
			$pids[] = $db->f('ParentID');
			foreach($workspaces as &$cur){
				$cur['parent'] = $db->f($cur['parentKey']);
				if($onlyParent){//we only need parentID
					continue;
				}
				if($cur['explodeValue']){
					// get workspaces
					$a = explode(',', trim($db->f($cur['key']), ','));
					foreach($a as $v){
						$cur['value'][] = $v;
					}
				} else {
					$cur['value'][] = $db->f($cur['key']);
				}
			}
			unset($cur);
		}

		foreach($pids as $pid){
			while($pid){ //	For each group
				$userGroups[] = $pid;

				if(($row = getHash('SELECT ' . $fields . ' FROM ' . USER_TABLE . ' WHERE ID=' . intval($pid), $db))){
					$pid = $row['ParentID'];
					foreach($workspaces as &$cur){
						if($cur['parent']){
							// get workspaces
							$a = explode(',', trim($row[$cur['key']], ','));
							foreach($a as $v){
								$cur['value'][] = $v;
							}
						}
						$cur['parent'] = $row[$cur['parentKey']];
					}
					unset($cur);
				} else {
					$pid = 0;
				}
			}
		}

		foreach($workspaces as &$cur){
			$cur = array_unique(array_filter($cur['value']));
		}
		unset($cur);
		if($onlyParent){
			return $workspaces;
		}
		$_SESSION['user']['workSpace'] = $workspaces;
		$_SESSION['user']['groups'] = $userGroups; //	order: first is folder with user himself (deepest in tree)

		if($_SESSION['user']['workSpace'][CUSTOMER_TABLE]){
			//setup customer
			$filter = [];
			foreach($_SESSION['user']['workSpace'][CUSTOMER_TABLE] as $cur){
				$filter[] = we_customer_abstractFilter::getQueryFromFilter(we_unserialize($cur));
			}

			//FIXME: this won't hold for alias users
			$_SESSION['user']['workSpace'][CUSTOMER_TABLE] = implode(' AND ', array_filter($filter));
		}
	}

	/**
	 *
	 * @param string $username DB-field
	 * @param string $storedPassword DB-field!!! //needs to be cause of salt!
	 * @param string $clearPassword //posted password
	 * @return boolean
	 */
	static function comparePasswords($username, $storedPassword, $clearPassword){
		$matches = [];
		$useSalt = (!preg_match('|^\$([^$]{2,4})\$([^$]+)\$(.+)$|', $storedPassword, $matches) ?
				//old md5
				self::SALT_MD5 :
				$matches[1]);

		switch($useSalt){
			default:
			/* unsupported
			 * 			case self::SALT_NONE:
			  $passwd = md5($clearPassword);
			  break; */
			case self::SALT_MD5:
				$passwd = md5($clearPassword . md5($username));
				break;
			case '2y':
				$passwd = crypt($passwd = substr($clearPassword, 0, 64), $storedPassword);
				break;
		}
		return ($passwd === $storedPassword);
	}

	public static function getHashIV($len){
		static $WE_SALTCHARS = './0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$salt = '';
		for($i = 0; $i < $len; $i++){
			$tmp_str = str_shuffle($WE_SALTCHARS);
			$salt .= $tmp_str[0];
		}
		return $salt;
	}

	public static function makeSaltedPassword($passwd, $strength = 15){
		return crypt(substr($passwd, 0, 64), '$2y$' . sprintf('%02d', $strength) . '$' . self::getHashIV(22));
	}

	static function readPrefs($id, $db, $login = false){
//set defaults
		$ret = ['userID' => $id];
		require_once(WE_INCLUDES_PATH . 'we_editors/we_preferences_config.inc.php');
		foreach($GLOBALS['configs']['user'] as $key => $vals){
			$ret[$key] = $vals[1];
		}
		if($login){
			$db->query('DELETE FROM ' . PREFS_TABLE . ' WHERE `key` NOT IN("' . implode('","', array_keys($GLOBALS['configs']['user'])) . '")');
		}
		$db->query('SELECT `key`,`value` FROM ' . PREFS_TABLE . ' WHERE userID=' . intval($id));
		//read db
		while($db->next_record(MYSQLI_ASSOC)){
			switch($GLOBALS['configs']['user'][$db->f('key')][0]){
				case we_base_request::INTLIST:
					$ret[$db->f('key')] = array_filter(explode(',', $db->f('value')));
					break;
				default:
					$ret[$db->f('key')] = $db->f('value');
					break;
			}
		}
		if($login){
			$_SESSION['prefs'] = $ret;
			self::writePrefs($id, $db);
		}
		return $ret;
	}

	/** write settings for a user, all default values are applied before data is written
	 * @id int user id to write settings for
	 * @db socket database connection
	 * @data array optional if empty settings of current session are used.
	 */
	static function writePrefs($id, $db, array $data = []){
		$id = intval($id);
		if($data){//note data must be different to session data
			$old = ['userID' => $id];
			require_once(WE_INCLUDES_PATH . 'we_editors/we_preferences_config.inc.php');
			foreach($GLOBALS['configs']['user'] as $key => $vals){
				//only write config data, if data is read! otherwise we overwrite some settings
				if(isset($data[$key])){
					$old[$key] = $vals[1];
				}
			}
		} else {
			$old = self::readPrefs($id, $db);
			$data = $_SESSION['prefs'];
		}
		$upd = [];
		foreach($old as $key => $val){
			if($key != 'userID' && (!isset($data[$key]) || $data[$key] != $val)){
				$upd[] = '(' . $id . ',"' . $db->escape($key) . '","' . $db->escape(isset($data[$key]) ? (is_array($data[$key]) ? implode(',', $data[$key]) : $data[$key]) : (is_array($val) ? implode(',', $val) : $val)) . '")';
			}
		}
		if(!empty($upd)){
			$db->query('REPLACE INTO ' . PREFS_TABLE . ' (`userID`,`key`,`value`) VALUES ' . implode(',', $upd));
		}
	}

	/**
	 * @internal
	 * @param string $table
	 * @param string $user
	 */
	public static function logLoginFailed($table, $user){
		$db = $GLOBALS['DB_WE'];
		$db->query('INSERT INTO ' . FAILED_LOGINS_TABLE . ' SET ' . we_database_base::arraySetter(['UserTable' => $table,
					'Username' => $user,
					'IP' => $_SERVER['REMOTE_ADDR'],
					'Servername' => $_SERVER['SERVER_NAME'],
					'Port' => $_SERVER['SERVER_PORT'],
					'Script' => getScriptName()
		]));
	}

	public static function updateActiveUser($onlyUpdate = false){
		if(!$_SESSION['user']['ID']){
			return [];
		}
		$ret = [
			'requests' => [],
			'reply' => [],
			'unlock' => []
		];

		$GLOBALS['DB_WE']->query('UPDATE ' . USER_TABLE . ' SET Ping=NOW() WHERE ID=' . $_SESSION['user']['ID']);
		if($onlyUpdate){
			return $ret;
		}
		//FIXME do we need to check session_id??? for release requests?
		//check release requests
		$ret['requests'] = $GLOBALS['DB_WE']->getAllq('SELECT l.ID,l.tbl,l.releaseRequestText AS text,DATE_FORMAT(l.releaseRequestForce,"' . g_l('date', '[format][mysql]') . '") AS forceDate,TIMESTAMPDIFF(SECOND,NOW(),releaseRequestForce) AS forceTime,CONCAT(u.First," ",u.Second) AS User FROM ' . LOCK_TABLE . ' l JOIN ' . USER_TABLE . ' u ON (l.releaseRequestID=u.ID) WHERE l.UserID=' . intval($_SESSION['user']['ID']) . ' AND l.releaseRequestText IS NOT NULL AND l.releaseRequestID!=' . intval($_SESSION['user']['ID']));

		//check my release requests - currently unused
		$ret['reply'] = []; //$GLOBALS['DB_WE']->getAllq('SELECT l.ID,l.tbl,l.releaseRequestReply AS text,CONCAT(u.First," ",u.Second," (",u.Username,")") AS User FROM ' . LOCK_TABLE . ' l JOIN ' . USER_TABLE . ' u ON (l.UserID=u.ID) WHERE l.releaseRequestReply IS NOT NULL AND l.releaseRequestID=' . intval($_SESSION['user']['ID']) . ' AND l.UserID!=' . intval($_SESSION['user']['ID']));
		//get all documents that have been released
		$ret['unlock'] = $GLOBALS['DB_WE']->getAllq('SELECT l.ID,l.tbl FROM ' . LOCK_TABLE . ' l WHERE l.releaseRequestID=l.UserID AND l.sessionID=x\'0000000000000000000000000000000000000000\'');
		$GLOBALS['DB_WE']->query('UPDATE ' . LOCK_TABLE . ' SET releaseRequestID=NULL, sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\' WHERE releaseRequestID=UserID AND sessionID=x\'0000000000000000000000000000000000000000\'');

		//reset all replies
		//$GLOBALS['DB_WE']->query('UPDATE ' . LOCK_TABLE . ' SET releaseRequestReply=NULL,releaseRequestForce=NULL WHERE releaseRequestReply IS NOT NULL AND releaseRequestID=' . intval($_SESSION['user']['ID']));
		//update my locks, reset releaseRequestText
		$GLOBALS['DB_WE']->query('UPDATE ' . LOCK_TABLE . ' SET lockTime=NOW()+INTERVAL ' . (we_base_constants::PING_TIME + we_base_constants::PING_TOLERANZ) . ' SECOND,releaseRequestText=NULL WHERE UserID=' . intval($_SESSION['user']['ID']) . ' AND sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\'');

		return $ret;
	}

	public static function unlockDocuments(){
		// prepare the queries, 4 as maximum.
		$ids = we_base_request::_(we_base_request::INTLISTA, 'we_cmd', [], 1); // we_cmd[1] is commaseperated list of ids
//FIXME: make tblList???
		$tables = we_base_request::_(we_base_request::STRING_LIST, 'we_cmd', '', 3); // we_cmd[3] is commaseparated list of tables
		$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', null, 4); // we_cmd[4] is a single transaction, to delete data from session

		$queries = [];

		if($transaction){ // clean session
			if(isset($_SESSION['weS']['we_data'][$transaction])){
				$doc = $_SESSION['weS']['we_data'][$transaction][0];
				if(isset($_SESSION['weS']['versions']['versionToCompare'][$doc['Table']][$doc['ID']])){
					unset($_SESSION['weS']['versions']['versionToCompare'][$doc['Table']][$doc['ID']]);
				}

				unset($_SESSION['weS']['we_data'][$transaction]); // we_transaction is resetted here
			}
		}

		foreach($ids as $i => $id){
			if($tables[$i] && $id){
				$queries[$tables[$i]][] = $id;
			}
		}

		//$uid = we_base_request::_(we_base_request::INT, 'we_cmd', 0, 2);
		foreach($queries as $table => $ids){
			//don't clean all locks! - is this really a needed statement???
			$ids = implode(', ', array_filter($ids));
			if($ids){
				$DB_WE = $GLOBALS['DB_WE'];
				$DB_WE->query('UPDATE ' . LOCK_TABLE . ' SET sessionID=x\'00\',UserID=releaseRequestID WHERE releaseRequestID IS NOT NULL AND releaseRequestID!=UserID AND tbl="' . $DB_WE->escape(stripTblPrefix($table)) . '" AND ID IN (' . $ids . ') AND UserID=' . intval($_SESSION['user']['ID']) . ' AND sessionID=x\'' . we_base_sessionHandler::getCurrentHex() . '\'');
				$DB_WE->query('DELETE FROM ' . LOCK_TABLE . ' WHERE tbl="' . $DB_WE->escape(stripTblPrefix($table)) . '" AND ID IN (' . $ids . ') AND sessionID!=x\'0000000000000000000000000000000000000000\' AND UserID=' . intval($_SESSION['user']['ID']));
			}
		}
		echo '{"UNLOCKED":true}';
	}

	public static function getJSLangConsts(){
		return we_base_util::setLangString([
					'view' => [
						'give_org_name' => g_l('modules_users', '[give_org_name]'),
						'password_alert' => (g_l('modules_users', '[password_alert]')),
						'save_changed_user' => g_l('modules_users', '[save_changed_user]'),
						'delete_alert' => [
							self::TYPE_USER_GROUP => g_l('modules_users', '[delete_alert_group]'),
							self::TYPE_ALIAS => g_l('modules_users', '[delete_alert_alias]'),
							self::TYPE_USER => g_l('modules_users', '[delete_alert_user]'),
						]
					]
		]);
	}

}
