<?php

/**
 * webEdition CMS
 *
 * $Rev: 14634 $
 * $Author: mokraemer $
 * $Date: 2017-12-04 17:21:43 +0100 (Mon, 04 Dec 2017) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class representing the model data of the search
 */
class we_search_modelBase extends we_base_model{

	/**
	 * @var string: classname
	 */
	public $ModelClassName = __CLASS__;

	/**
	 * @var string: toolname
	 */
	public $toolName = 'weSearch';
	public $mode = 0;
	// in these variables we hold data of actual search (= whichSearch) after initByHttp();
	protected $currentSearchstart = 0;
	protected $currentSearch = [];
	protected $currentLocation = [];
	protected $currentSearchTables = [];
	protected $currentSearchFields = [];
	protected $currentOrder = 'Text';
	protected $currentAnzahl = 10;
	protected $currentAnzahlMedialinks = 0;
	protected $currentSetView = 0;
	protected $currentFolderID = 0;
	protected $currentSearchForField = ['text' => 0,
		'title' => 0,
		'meta' => 0,
	];
	protected $currentSearchForContentType = ['image' => 0,
		'audio' => 0,
		'video' => 0,
		'other' => 0,
	];
	//FIXME: copied from tool_model, check what is needed
	var $ID = 0;
	var $Text;
	var $ParentID = 0;
	var $Path;
	var $IsFolder;
	var $requiredFields = [];

	/**
	 * Default Constructor
	 * Can load or create new searchtool object depends of parameter
	 */
	public function __construct($table = ''){
		parent::__construct($table);
	}

	function save($force_new = false, $isAdvanced = false, $jsonSer = false){
		return parent::save($force_new, $isAdvanced, true); // allways use jsonSer!
	}

	public function load($id = 0, $isAdvanced = false){
		return parent::load($id);
	}

	public function initByHttp($whichSearch = '', $isWeCmd = true){

	}

	public function prepareModelForSearch(){

	}

	public function getProperty($property = ''){
		switch($property){
			default:
			case 'currentSearchstart':
				return $this->currentSearchstart;
			case 'currentSearch':
				return $this->currentSearch;
			case 'currentLocation':
				return $this->currentLocation;
			case 'currentSearchTables':
				return $this->currentSearchTables;
			case 'currentSearchFields':
				return $this->currentSearchFields;
			case 'currentOrder':
				return $this->currentOrder;
			case 'currentAnzahl':
				return $this->currentAnzahl;
			case 'currentAnzahlMedialinks':
				return intval($this->currentAnzahlMedialinks);
			case 'currentSetView':
				return $this->currentSetView;
			case 'currentFolderID':
				return $this->currentFolderID;
			case 'currentSearchForField':
				return $this->currentSearchForField;
			case 'currentSearchForContentType':
				return $this->currentSearchForContentType;
		}
	}

	public function setIsFolder($value){
		$this->IsFolder = $value;
	}

	public function filenameNotValid($text = ''){
		return preg_match('|[^a-z0-9._-]|i', $text);
	}

	public static function getLangText($path, $text){
		switch($path){
			case '/_PREDEF_':
				return g_l('searchtool', '[vordefinierteSuchanfragen]');
			case '/_PREDEF_/document':
				return g_l('searchtool', '[dokumente]');
			case '/_PREDEF_/document/unpublished':
				return g_l('searchtool', '[unveroeffentlicheDokumente]');
			case '/_PREDEF_/document/static':
				return g_l('searchtool', '[statischeDokumente]');
			case '/_PREDEF_/document/dynamic':
				return g_l('searchtool', '[dynamischeDokumente]');

			case '/_PREDEF_/object':
				return g_l('searchtool', '[objekte]');
			case '/_PREDEF_/object/unpublished':
				return g_l('searchtool', '[unveroeffentlicheObjekte]');
			case '/_CUSTOM_':
				return g_l('searchtool', '[eigeneSuchanfragen]');
			case '/_VERSION_':
				return g_l('searchtool', '[versionen]');
			case '/_VERSION_/document':
				return g_l('searchtool', '[dokumente]');
			case '/_VERSION_/document/deleted':
				return g_l('searchtool', '[geloeschteDokumente]');

			case '/_VERSION_/object':
				return g_l('searchtool', '[objekte]');
			case '/_VERSION_/object/deleted':
				return g_l('searchtool', '[geloeschteObjekte]');
			default:
				return $text;
		}
	}

	function saveInSession(){
		$_SESSION['weS']['weSearch_session'] = $this;
	}

	function clearSessionVars(){
		if(!empty($this->toolName) && isset($_SESSION['weS']['weSearch_session'])){
			unset($_SESSION['weS']['weSearch_session']);
		}
	}

	//FIXME: copied from tool_model, check what is needed

	function isRequiredField($fieldname){
		return in_array($fieldname, $this->requiredFields);
	}

	function hasRequiredFields(&$failed){
		foreach($this->requiredFields as $req){
			if(empty($this->$req)){
				$failed[] = $req;
			}
		}
		return empty($failed);
	}

	function setPath(){
		$ppath = f('SELECT Path FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->ParentID) . ';', 'Path', $this->db);
		$this->Path = $ppath . "/" . $this->Text;
	}

	function pathExists($path){
		$this->db->query('SELECT * FROM ' . $this->db->escape($this->table) . ' WHERE Path="' . $this->db->escape($path) . '" AND ID!=' . intval($this->ID));
		return ($this->db->next_record() ? true : false);
	}

	function isSelf(){
		if($this->ID){
			$count = 0;
			$parentid = $this->ParentID;
			while($parentid != 0){
				if($parentid == $this->ID){
					return true;
				}
				$parentid = f('SELECT ParentID FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($parentid), 'ParentID', $this->db);
				$count++;
				if($count == 9999){
					return false;
				}
			}
			return false;
		}
		return false;
	}

	function isAllowedForUser(){
		return true;
	}

	function evalPath($id = 0){
		$db_tmp = new DB_WE();
		$path = '';
		if($id == 0){
			$id = $this->ParentID;
			$path = $this->Text;
		}

		$foo = getHash('SELECT Text,ParentID FROM ' . $db_tmp->escape($this->table) . ' WHERE ID=' . intval($id), $db_tmp);
		$path = '/' . (isset($foo['Text']) ? $foo['Text'] : '') . $path;

		$pid = isset($foo['ParentID']) ? $foo['ParentID'] : '';
		while($pid > 0){
			$db_tmp->query('SELECT Text,ParentID FROM ' . $db_tmp->escape($this->table) . ' WHERE ID=' . intval($pid));
			while($db_tmp->next_record()){
				$path = '/' . $db_tmp->f('Text') . $path;
				$pid = $db_tmp->f('ParentID');
			}
		}
		return $path;
	}

	function updateChildPaths($oldpath){
		if($this->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Path){
			$db_tmp = new DB_WE();
			$this->db->query('SELECT ID FROM ' . $db_tmp->escape($this->table) . ' WHERE Path LIKE \'' . $db_tmp->escape($oldpath) . '%\' AND ID!=' . intval($this->ID));
			while($this->db->next_record()){
				$db_tmp->query('UPDATE ' . $db_tmp->escape($this->table) . ' SET Path=\'' . $db_tmp->escape($this->evalPath($this->db->f("ID"))) . '\' WHERE ID=' . intval($this->db->f("ID")));
			}
		}
	}

	private function deleteChilds(){
		$this->db->query('SELECT ID FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . intval($this->ID));
		while($this->db->next_record()){
			$child = new $this->ModelClassName($this->db->f("ID"));
			$child->delete();
		}
	}

	function delete(){

		if(!$this->ID){
			return false;
		}
		if($this->IsFolder){
			$this->deleteChilds();
		}
		parent::delete();

		return true;
	}

}
